Vega Graph Examples
============

# Bar Plot


### Vertical Bar 

This is an example of using an external vertical_bar.json for Vega. 
If you view the generated HTML locally, your browser may refuse to load any 
JSON files from local directories. This is expected as the JavaScript sandbox 
prevents any access to local files. You can either change your browser security
settings, or set up a simple HTTP server locally to view the graphs.

Please take a look at 
[this discussion](https://github.com/mrdoob/three.js/wiki/How-to-run-things-locally)
if you want to test Vega graphs with a JSON file locally.

\htmlonly
<div id="verticalbar"></div>
<script type="text/javascript">
plotjson("verticalbar", "../graphs/vertical_bar.json");
</script>
\endhtmlonly

### Horizontal Bar 

Using json embedded in this .md file.


\htmlonly
<div id="horizontalbar"></div>
<script type="text/javascript">
plot("horizontalbar",
	{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"height":{"offset":-1,"scale":"x","band":true},"x2":{"value":0,"scale":"y"},"x":{"field":"y","scale":"y"},"width":{"offset":-1,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"x","scale":"x"}}},"from":{"data":"table"},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"x","scale":"y"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"y","scale":"x"}],"data":[{"name":"table","values":[{"x":1,"y2":0,"group":1,"y":1},{"x":2,"y2":0,"group":1,"y":2},{"x":3,"y2":0,"group":1,"y":3},{"x":4,"y2":0,"group":1,"y":2},{"x":5,"y2":0,"group":1,"y":1}]}],"scales":[{"name":"x","range":"height","domain":{"data":"table","field":"x"},"type":"ordinal"},{"name":"y","range":"width","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450}
	);
</script>
\endhtmlonly

### Group Argument

This is an example of loading group_bar.json for data only; graph formatting parameters embedded in Markdown.

\htmlonly
<div id="groupbar"></div>
<script type="text/javascript">
plot("groupbar",
        {"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"x":{"field":"x","scale":"x"},"y2":{"field":"y2","scale":"y"},"width":{"offset":-1,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"y","scale":"y"}}},"from":{"data":"table"},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"ordinal"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}],"data":[{"name":"table", "url":"../graphs/group_bar.json"}]}
        );
</script>
\endhtmlonly

### Stacked Bar (with additional `colorscheme!` change)


\htmlonly
<div id="stacked"></div>
<script type="text/javascript">
plot("stacked",

{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"x":{"field":"x","scale":"x"},"y2":{"field":"layout_end","scale":"y"},"width":{"offset":-1,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"layout_start","scale":"y"}}},"from":{"data":"table","transform":[{"sortby":["group"],"offset":"zero","field":"y","groupby":["x"],"type":"stack"}]},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":0,"y2":0,"group":0,"y":28},{"x":1,"y2":0,"group":0,"y":43},{"x":2,"y2":0,"group":0,"y":81},{"x":3,"y2":0,"group":0,"y":19},{"x":4,"y2":0,"group":0,"y":52},{"x":5,"y2":0,"group":0,"y":24},{"x":6,"y2":0,"group":0,"y":87},{"x":7,"y2":0,"group":0,"y":17},{"x":8,"y2":0,"group":0,"y":68},{"x":9,"y2":0,"group":0,"y":49},{"x":0,"y2":0,"group":1,"y":55},{"x":1,"y2":0,"group":1,"y":91},{"x":2,"y2":0,"group":1,"y":53},{"x":3,"y2":0,"group":1,"y":87},{"x":4,"y2":0,"group":1,"y":48},{"x":5,"y2":0,"group":1,"y":49},{"x":6,"y2":0,"group":1,"y":66},{"x":7,"y2":0,"group":1,"y":27},{"x":8,"y2":0,"group":1,"y":16},{"x":9,"y2":0,"group":1,"y":15}]},{"name":"stats","source":"table","transform":[{"groupby":["x"],"type":"aggregate","summarize":[{"ops":["sum"],"field":"y"}]}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"ordinal"},{"name":"y","range":"height","domain":{"data":"stats","field":"sum_y"},"type":"linear"},{"name":"group","range":["rgb(229,245,224)","rgb(161,217,155)","rgb( 49,163, 84)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}
	);
</script>
\endhtmlonly

### Normalized Horizontal Stacked Bar (with additional `hover!` change)


\htmlonly
<div id="horizhover"></div>
<script type="text/javascript">
plot("horizhover",

{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"update":{"fillOpacity":{"value":1}},"hover":{"fillOpacity":{"value":0.75}},"enter":{"height":{"offset":-1,"scale":"x","band":true},"x2":{"field":"layout_end","scale":"y"},"x":{"field":"layout_start","scale":"y"},"width":{"offset":-1,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"x","scale":"x"}}},"from":{"data":"table","transform":[{"sortby":["group"],"offset":"normalize","field":"y","groupby":["x"],"type":"stack"}]},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"x","scale":"y"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"y","scale":"x"}],"data":[{"name":"table","values":[{"x":0,"y2":0,"group":0,"y":28},{"x":1,"y2":0,"group":0,"y":43},{"x":2,"y2":0,"group":0,"y":81},{"x":3,"y2":0,"group":0,"y":19},{"x":4,"y2":0,"group":0,"y":52},{"x":5,"y2":0,"group":0,"y":24},{"x":6,"y2":0,"group":0,"y":87},{"x":7,"y2":0,"group":0,"y":17},{"x":8,"y2":0,"group":0,"y":68},{"x":9,"y2":0,"group":0,"y":49},{"x":0,"y2":0,"group":1,"y":55},{"x":1,"y2":0,"group":1,"y":91},{"x":2,"y2":0,"group":1,"y":53},{"x":3,"y2":0,"group":1,"y":87},{"x":4,"y2":0,"group":1,"y":48},{"x":5,"y2":0,"group":1,"y":49},{"x":6,"y2":0,"group":1,"y":66},{"x":7,"y2":0,"group":1,"y":27},{"x":8,"y2":0,"group":1,"y":16},{"x":9,"y2":0,"group":1,"y":15}]},{"name":"stats","source":"table","transform":[{"groupby":["x"],"type":"aggregate","summarize":[{"ops":["sum"],"field":"y"}]}]}],"scales":[{"name":"x","range":"height","domain":{"data":"table","field":"x"},"type":"ordinal"},{"reverse":false,"name":"y","zero":true,"domainMax":1,"domain":{"data":"stats","field":"sum_y"},"range":"width","type":"linear","round":false},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}
	);
</script>
\endhtmlonly

# Histogram

### Histogram - Regular

\htmlonly
<div id="hist"></div>
<script type="text/javascript">
plot("hist",
	{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"x":{"field":"x","scale":"x"},"y2":{"field":"y2","scale":"y"},"width":{"offset":0,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"y","scale":"y"}}},"from":{"data":"table"},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":1.0,"y2":0,"group":1,"y":8106.0},{"x":2.0,"y2":0,"group":1,"y":24001.0},{"x":3.0,"y2":0,"group":1,"y":25313.0},{"x":4.0,"y2":0,"group":1,"y":18580.0},{"x":5.0,"y2":0,"group":1,"y":11453.0},{"x":6.0,"y2":0,"group":1,"y":6312.0},{"x":7.0,"y2":0,"group":1,"y":3297.0},{"x":8.0,"y2":0,"group":1,"y":1556.0},{"x":9.0,"y2":0,"group":1,"y":771.0},{"x":10.0,"y2":0,"group":1,"y":326.0},{"x":11.0,"y2":0,"group":1,"y":158.0},{"x":12.0,"y2":0,"group":1,"y":73.0},{"x":13.0,"y2":0,"group":1,"y":25.0},{"x":14.0,"y2":0,"group":1,"y":13.0},{"x":15.0,"y2":0,"group":1,"y":10.0},{"x":16.0,"y2":0,"group":1,"y":4.0},{"x":17.0,"y2":0,"group":1,"y":1.0},{"x":18.0,"y2":0,"group":1,"y":1.0}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"ordinal"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450}

	);
</script>
\endhtmlonly

### Histogram - Relative Frequency


\htmlonly
<div id="relhist"></div>
<script type="text/javascript">
plot("relhist",
	{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"x":{"field":"x","scale":"x"},"y2":{"field":"y2","scale":"y"},"width":{"offset":0,"scale":"x","band":true},"fill":{"field":"group","scale":"group"},"y":{"field":"y","scale":"y"}}},"from":{"data":"table"},"type":"rect"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":1.0,"y2":0,"group":1,"y":0.08106},{"x":2.0,"y2":0,"group":1,"y":0.24001},{"x":3.0,"y2":0,"group":1,"y":0.25313},{"x":4.0,"y2":0,"group":1,"y":0.1858},{"x":5.0,"y2":0,"group":1,"y":0.11453},{"x":6.0,"y2":0,"group":1,"y":0.06312},{"x":7.0,"y2":0,"group":1,"y":0.03297},{"x":8.0,"y2":0,"group":1,"y":0.01556},{"x":9.0,"y2":0,"group":1,"y":0.00771},{"x":10.0,"y2":0,"group":1,"y":0.00326},{"x":11.0,"y2":0,"group":1,"y":0.00158},{"x":12.0,"y2":0,"group":1,"y":0.00073},{"x":13.0,"y2":0,"group":1,"y":0.00025},{"x":14.0,"y2":0,"group":1,"y":0.00013},{"x":15.0,"y2":0,"group":1,"y":0.0001},{"x":16.0,"y2":0,"group":1,"y":4.0e-5},{"x":17.0,"y2":0,"group":1,"y":1.0e-5},{"x":18.0,"y2":0,"group":1,"y":1.0e-5}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"ordinal"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450}
	);
</script>
\endhtmlonly


# Line Plot


### Multiple Lines In Single Plot


\htmlonly
<div id="lineplot"></div>
<script type="text/javascript">
plot("lineplot",
    {"name":"lineplot","height":450,"padding":"auto","marks":[{"marks":[{"properties":{"enter":{"stroke":{"field":"group","scale":"group"},"x":{"field":"x","scale":"x"},"strokeWidth":{"value":2},"y":{"field":"y","scale":"y"}}},"type":"line"}],"from":{"data":"table","transform":[{"groupby":["group"],"type":"facet"}]},"type":"group"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":1,"y2":0,"group":1,"y":-0.21321369986580918},{"x":2,"y2":0,"group":1,"y":1.3985070732602853},{"x":3,"y2":0,"group":1,"y":2.8315969128222176},{"x":4,"y2":0,"group":1,"y":5.10847552856312},{"x":5,"y2":0,"group":1,"y":5.797644115914002},{"x":6,"y2":0,"group":1,"y":5.914610430797741},{"x":7,"y2":0,"group":1,"y":6.358133569529957},{"x":8,"y2":0,"group":1,"y":8.724427862853243},{"x":9,"y2":0,"group":1,"y":8.348623156839507},{"x":10,"y2":0,"group":1,"y":9.688147067106232},{"x":11,"y2":0,"group":1,"y":10.06112602165281},{"x":12,"y2":0,"group":1,"y":12.407836524004129},{"x":13,"y2":0,"group":1,"y":12.166519018329044},{"x":14,"y2":0,"group":1,"y":13.732125499038986},{"x":15,"y2":0,"group":1,"y":14.733586123242661},{"x":16,"y2":0,"group":1,"y":14.488612805533847},{"x":17,"y2":0,"group":1,"y":17.516473995333012},{"x":18,"y2":0,"group":1,"y":18.251977860211433},{"x":19,"y2":0,"group":1,"y":18.15792770870133},{"x":20,"y2":0,"group":1,"y":21.04063141312778},{"x":21,"y2":0,"group":1,"y":21.812152189304243},{"x":22,"y2":0,"group":1,"y":21.808301374717583},{"x":23,"y2":0,"group":1,"y":22.722977307050957},{"x":24,"y2":0,"group":1,"y":24.081096920490943},{"x":25,"y2":0,"group":1,"y":26.859041975538684},{"x":26,"y2":0,"group":1,"y":25.162163164847215},{"x":27,"y2":0,"group":1,"y":26.571685780072542},{"x":28,"y2":0,"group":1,"y":28.721574341634167},{"x":29,"y2":0,"group":1,"y":29.894064600438004},{"x":30,"y2":0,"group":1,"y":31.965721152914448},{"x":31,"y2":0,"group":1,"y":31.070567235019375},{"x":32,"y2":0,"group":1,"y":32.305612978826574},{"x":33,"y2":0,"group":1,"y":32.46357128268016},{"x":34,"y2":0,"group":1,"y":35.226342922540006},{"x":35,"y2":0,"group":1,"y":35.348539381967065},{"x":36,"y2":0,"group":1,"y":36.649149265710896},{"x":37,"y2":0,"group":1,"y":36.561535257826435},{"x":38,"y2":0,"group":1,"y":38.83016849547384},{"x":39,"y2":0,"group":1,"y":37.88531526555204},{"x":40,"y2":0,"group":1,"y":41.953830739664824},{"x":41,"y2":0,"group":1,"y":40.6704309868311},{"x":42,"y2":0,"group":1,"y":43.90959115734531},{"x":43,"y2":0,"group":1,"y":43.41137075670251},{"x":44,"y2":0,"group":1,"y":43.439708012187154},{"x":45,"y2":0,"group":1,"y":44.973493872253364},{"x":46,"y2":0,"group":1,"y":46.73305937810806},{"x":47,"y2":0,"group":1,"y":44.524780522197915},{"x":48,"y2":0,"group":1,"y":48.53010059957868},{"x":49,"y2":0,"group":1,"y":49.86780586391065},{"x":50,"y2":0,"group":1,"y":49.01557973572676},{"x":51,"y2":0,"group":1,"y":51.06983350683799},{"x":52,"y2":0,"group":1,"y":51.05888914249255},{"x":53,"y2":0,"group":1,"y":52.5115001831221},{"x":54,"y2":0,"group":1,"y":53.97393201664967},{"x":55,"y2":0,"group":1,"y":53.66053030674649},{"x":56,"y2":0,"group":1,"y":56.81971516458666},{"x":57,"y2":0,"group":1,"y":56.54959472971153},{"x":58,"y2":0,"group":1,"y":59.65139846648794},{"x":59,"y2":0,"group":1,"y":60.91139767808923},{"x":60,"y2":0,"group":1,"y":58.94542235843979},{"x":61,"y2":0,"group":1,"y":61.337909839945034},{"x":62,"y2":0,"group":1,"y":62.0909948309055},{"x":63,"y2":0,"group":1,"y":62.93246429097792},{"x":64,"y2":0,"group":1,"y":64.71022829267858},{"x":65,"y2":0,"group":1,"y":66.83522495852513},{"x":66,"y2":0,"group":1,"y":66.32649430944169},{"x":67,"y2":0,"group":1,"y":66.31951475210055},{"x":68,"y2":0,"group":1,"y":68.59769112425644},{"x":69,"y2":0,"group":1,"y":71.24129489433732},{"x":70,"y2":0,"group":1,"y":69.99823432374605},{"x":71,"y2":0,"group":1,"y":71.11322741154369},{"x":72,"y2":0,"group":1,"y":71.05344626161155},{"x":73,"y2":0,"group":1,"y":74.85440091977581},{"x":74,"y2":0,"group":1,"y":75.55317648562344},{"x":75,"y2":0,"group":1,"y":75.45503129141386},{"x":76,"y2":0,"group":1,"y":76.0223616001447},{"x":77,"y2":0,"group":1,"y":74.92783509731498},{"x":78,"y2":0,"group":1,"y":78.43358351371874},{"x":79,"y2":0,"group":1,"y":78.42153512627573},{"x":80,"y2":0,"group":1,"y":81.0553229766386},{"x":81,"y2":0,"group":1,"y":80.92601877652054},{"x":82,"y2":0,"group":1,"y":83.42152175231054},{"x":83,"y2":0,"group":1,"y":83.14269072793677},{"x":84,"y2":0,"group":1,"y":82.66049263182241},{"x":85,"y2":0,"group":1,"y":84.1226306401764},{"x":86,"y2":0,"group":1,"y":86.3227262400373},{"x":87,"y2":0,"group":1,"y":86.91795276393403},{"x":88,"y2":0,"group":1,"y":86.9567853292563},{"x":89,"y2":0,"group":1,"y":87.57613065700745},{"x":90,"y2":0,"group":1,"y":91.01575331700467},{"x":91,"y2":0,"group":1,"y":90.2519113725953},{"x":92,"y2":0,"group":1,"y":91.17616274054065},{"x":93,"y2":0,"group":1,"y":94.92983067686791},{"x":94,"y2":0,"group":1,"y":94.46040371685923},{"x":95,"y2":0,"group":1,"y":94.36177173493158},{"x":96,"y2":0,"group":1,"y":95.18760778064903},{"x":97,"y2":0,"group":1,"y":97.69778288965564},{"x":98,"y2":0,"group":1,"y":98.42700274410771},{"x":99,"y2":0,"group":1,"y":98.46411758202628},{"x":100,"y2":0,"group":1,"y":99.99237690251375},{"x":1,"y2":0,"group":2,"y":3.9685809095608855},{"x":2,"y2":0,"group":2,"y":6.149192141676936},{"x":3,"y2":0,"group":2,"y":8.593510164930834},{"x":4,"y2":0,"group":2,"y":9.98002741794392},{"x":5,"y2":0,"group":2,"y":11.451741017455385},{"x":6,"y2":0,"group":2,"y":10.779480801437428},{"x":7,"y2":0,"group":2,"y":12.673383443036764},{"x":8,"y2":0,"group":2,"y":15.779652415816079},{"x":9,"y2":0,"group":2,"y":16.07431024737684},{"x":10,"y2":0,"group":2,"y":19.718430791034823},{"x":11,"y2":0,"group":2,"y":18.44486731156199},{"x":12,"y2":0,"group":2,"y":20.36963341907325},{"x":13,"y2":0,"group":2,"y":21.538324403291618},{"x":14,"y2":0,"group":2,"y":24.216897481249482},{"x":15,"y2":0,"group":2,"y":25.44331378078537},{"x":16,"y2":0,"group":2,"y":28.17716062504715},{"x":17,"y2":0,"group":2,"y":29.324767259984746},{"x":18,"y2":0,"group":2,"y":28.590212431477553},{"x":19,"y2":0,"group":2,"y":33.647350741958796},{"x":20,"y2":0,"group":2,"y":31.94460195122258},{"x":21,"y2":0,"group":2,"y":32.10993058176156},{"x":22,"y2":0,"group":2,"y":37.176152378517315},{"x":23,"y2":0,"group":2,"y":37.107742736814956},{"x":24,"y2":0,"group":2,"y":39.40000330874773},{"x":25,"y2":0,"group":2,"y":40.669962925596174},{"x":26,"y2":0,"group":2,"y":42.08993134457572},{"x":27,"y2":0,"group":2,"y":44.30713225843759},{"x":28,"y2":0,"group":2,"y":42.288610524025245},{"x":29,"y2":0,"group":2,"y":48.53422810409889},{"x":30,"y2":0,"group":2,"y":48.990086221864914},{"x":31,"y2":0,"group":2,"y":49.43089313321655},{"x":32,"y2":0,"group":2,"y":52.70011370097463},{"x":33,"y2":0,"group":2,"y":51.59397651674797},{"x":34,"y2":0,"group":2,"y":53.325677936525764},{"x":35,"y2":0,"group":2,"y":55.8915802885221},{"x":36,"y2":0,"group":2,"y":56.70748947211033},{"x":37,"y2":0,"group":2,"y":58.749497717585434},{"x":38,"y2":0,"group":2,"y":59.527144181456336},{"x":39,"y2":0,"group":2,"y":60.348222325162354},{"x":40,"y2":0,"group":2,"y":63.54229414658719},{"x":41,"y2":0,"group":2,"y":64.67848723143688},{"x":42,"y2":0,"group":2,"y":64.92756798228612},{"x":43,"y2":0,"group":2,"y":67.76726519535438},{"x":44,"y2":0,"group":2,"y":71.21237589264443},{"x":45,"y2":0,"group":2,"y":69.77569015550937},{"x":46,"y2":0,"group":2,"y":73.1012721388432},{"x":47,"y2":0,"group":2,"y":73.59417127368785},{"x":48,"y2":0,"group":2,"y":75.84215611638032},{"x":49,"y2":0,"group":2,"y":76.73475947093556},{"x":50,"y2":0,"group":2,"y":77.46857405209},{"x":51,"y2":0,"group":2,"y":79.9144765767943},{"x":52,"y2":0,"group":2,"y":82.08105451776464},{"x":53,"y2":0,"group":2,"y":81.14068541228696},{"x":54,"y2":0,"group":2,"y":83.0548092512663},{"x":55,"y2":0,"group":2,"y":83.18705331674356},{"x":56,"y2":0,"group":2,"y":86.6682304790251},{"x":57,"y2":0,"group":2,"y":89.35823774633533},{"x":58,"y2":0,"group":2,"y":88.81169496810575},{"x":59,"y2":0,"group":2,"y":91.29064909938221},{"x":60,"y2":0,"group":2,"y":95.16679912340854},{"x":61,"y2":0,"group":2,"y":94.64503334447201},{"x":62,"y2":0,"group":2,"y":95.91747999563408},{"x":63,"y2":0,"group":2,"y":97.92024281683334},{"x":64,"y2":0,"group":2,"y":99.20470318694356},{"x":65,"y2":0,"group":2,"y":102.1070866223268},{"x":66,"y2":0,"group":2,"y":100.91658655349063},{"x":67,"y2":0,"group":2,"y":104.70489423099393},{"x":68,"y2":0,"group":2,"y":103.361897018721},{"x":69,"y2":0,"group":2,"y":105.24580823943818},{"x":70,"y2":0,"group":2,"y":106.76836951385988},{"x":71,"y2":0,"group":2,"y":110.06286156236393},{"x":72,"y2":0,"group":2,"y":109.88685994626938},{"x":73,"y2":0,"group":2,"y":112.55098178782217},{"x":74,"y2":0,"group":2,"y":114.07303570647427},{"x":75,"y2":0,"group":2,"y":113.55428122965438},{"x":76,"y2":0,"group":2,"y":116.83012081628749},{"x":77,"y2":0,"group":2,"y":119.12410650710446},{"x":78,"y2":0,"group":2,"y":118.93813378644354},{"x":79,"y2":0,"group":2,"y":120.17540213210881},{"x":80,"y2":0,"group":2,"y":124.40636982141221},{"x":81,"y2":0,"group":2,"y":124.44990914337959},{"x":82,"y2":0,"group":2,"y":124.88115555001386},{"x":83,"y2":0,"group":2,"y":124.88515959908439},{"x":84,"y2":0,"group":2,"y":128.98540062176556},{"x":85,"y2":0,"group":2,"y":129.89088154610207},{"x":86,"y2":0,"group":2,"y":133.02668234574597},{"x":87,"y2":0,"group":2,"y":133.94745458144092},{"x":88,"y2":0,"group":2,"y":134.42659505555756},{"x":89,"y2":0,"group":2,"y":134.9094695238586},{"x":90,"y2":0,"group":2,"y":138.85635910791504},{"x":91,"y2":0,"group":2,"y":138.13986278789577},{"x":92,"y2":0,"group":2,"y":142.05487993405137},{"x":93,"y2":0,"group":2,"y":141.16082214190453},{"x":94,"y2":0,"group":2,"y":143.40996538818},{"x":95,"y2":0,"group":2,"y":146.4727249582702},{"x":96,"y2":0,"group":2,"y":145.1238504449773},{"x":97,"y2":0,"group":2,"y":149.26984625418515},{"x":98,"y2":0,"group":2,"y":150.52203209488368},{"x":99,"y2":0,"group":2,"y":151.3308497817558},{"x":100,"y2":0,"group":2,"y":153.52865134160825}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"linear"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

	);
</script>
\endhtmlonly

### Adding Points


\htmlonly
<div id="lineplot2"></div>
<script type="text/javascript">
plot("lineplot2",
    {"name":"lineplot","height":450,"padding":"auto","marks":[{"marks":[{"properties":{"enter":{"stroke":{"field":"group","scale":"group"},"x":{"field":"x","scale":"x"},"strokeWidth":{"value":2},"y":{"field":"y","scale":"y"}}},"type":"line"},{"properties":{"enter":{"shape":{"value":"circle"},"stroke":{"field":"group","scale":"group"},"x":{"field":"x","scale":"x"},"strokeWidth":{"value":2},"size":{"value":10},"y":{"field":"y","scale":"y"}}},"type":"symbol"}],"from":{"data":"table","transform":[{"groupby":["group"],"type":"facet"}]},"type":"group"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":1,"y2":0,"group":1,"y":-0.21321369986580918},{"x":2,"y2":0,"group":1,"y":1.3985070732602853},{"x":3,"y2":0,"group":1,"y":2.8315969128222176},{"x":4,"y2":0,"group":1,"y":5.10847552856312},{"x":5,"y2":0,"group":1,"y":5.797644115914002},{"x":6,"y2":0,"group":1,"y":5.914610430797741},{"x":7,"y2":0,"group":1,"y":6.358133569529957},{"x":8,"y2":0,"group":1,"y":8.724427862853243},{"x":9,"y2":0,"group":1,"y":8.348623156839507},{"x":10,"y2":0,"group":1,"y":9.688147067106232},{"x":11,"y2":0,"group":1,"y":10.06112602165281},{"x":12,"y2":0,"group":1,"y":12.407836524004129},{"x":13,"y2":0,"group":1,"y":12.166519018329044},{"x":14,"y2":0,"group":1,"y":13.732125499038986},{"x":15,"y2":0,"group":1,"y":14.733586123242661},{"x":16,"y2":0,"group":1,"y":14.488612805533847},{"x":17,"y2":0,"group":1,"y":17.516473995333012},{"x":18,"y2":0,"group":1,"y":18.251977860211433},{"x":19,"y2":0,"group":1,"y":18.15792770870133},{"x":20,"y2":0,"group":1,"y":21.04063141312778},{"x":21,"y2":0,"group":1,"y":21.812152189304243},{"x":22,"y2":0,"group":1,"y":21.808301374717583},{"x":23,"y2":0,"group":1,"y":22.722977307050957},{"x":24,"y2":0,"group":1,"y":24.081096920490943},{"x":25,"y2":0,"group":1,"y":26.859041975538684},{"x":26,"y2":0,"group":1,"y":25.162163164847215},{"x":27,"y2":0,"group":1,"y":26.571685780072542},{"x":28,"y2":0,"group":1,"y":28.721574341634167},{"x":29,"y2":0,"group":1,"y":29.894064600438004},{"x":30,"y2":0,"group":1,"y":31.965721152914448},{"x":31,"y2":0,"group":1,"y":31.070567235019375},{"x":32,"y2":0,"group":1,"y":32.305612978826574},{"x":33,"y2":0,"group":1,"y":32.46357128268016},{"x":34,"y2":0,"group":1,"y":35.226342922540006},{"x":35,"y2":0,"group":1,"y":35.348539381967065},{"x":36,"y2":0,"group":1,"y":36.649149265710896},{"x":37,"y2":0,"group":1,"y":36.561535257826435},{"x":38,"y2":0,"group":1,"y":38.83016849547384},{"x":39,"y2":0,"group":1,"y":37.88531526555204},{"x":40,"y2":0,"group":1,"y":41.953830739664824},{"x":41,"y2":0,"group":1,"y":40.6704309868311},{"x":42,"y2":0,"group":1,"y":43.90959115734531},{"x":43,"y2":0,"group":1,"y":43.41137075670251},{"x":44,"y2":0,"group":1,"y":43.439708012187154},{"x":45,"y2":0,"group":1,"y":44.973493872253364},{"x":46,"y2":0,"group":1,"y":46.73305937810806},{"x":47,"y2":0,"group":1,"y":44.524780522197915},{"x":48,"y2":0,"group":1,"y":48.53010059957868},{"x":49,"y2":0,"group":1,"y":49.86780586391065},{"x":50,"y2":0,"group":1,"y":49.01557973572676},{"x":51,"y2":0,"group":1,"y":51.06983350683799},{"x":52,"y2":0,"group":1,"y":51.05888914249255},{"x":53,"y2":0,"group":1,"y":52.5115001831221},{"x":54,"y2":0,"group":1,"y":53.97393201664967},{"x":55,"y2":0,"group":1,"y":53.66053030674649},{"x":56,"y2":0,"group":1,"y":56.81971516458666},{"x":57,"y2":0,"group":1,"y":56.54959472971153},{"x":58,"y2":0,"group":1,"y":59.65139846648794},{"x":59,"y2":0,"group":1,"y":60.91139767808923},{"x":60,"y2":0,"group":1,"y":58.94542235843979},{"x":61,"y2":0,"group":1,"y":61.337909839945034},{"x":62,"y2":0,"group":1,"y":62.0909948309055},{"x":63,"y2":0,"group":1,"y":62.93246429097792},{"x":64,"y2":0,"group":1,"y":64.71022829267858},{"x":65,"y2":0,"group":1,"y":66.83522495852513},{"x":66,"y2":0,"group":1,"y":66.32649430944169},{"x":67,"y2":0,"group":1,"y":66.31951475210055},{"x":68,"y2":0,"group":1,"y":68.59769112425644},{"x":69,"y2":0,"group":1,"y":71.24129489433732},{"x":70,"y2":0,"group":1,"y":69.99823432374605},{"x":71,"y2":0,"group":1,"y":71.11322741154369},{"x":72,"y2":0,"group":1,"y":71.05344626161155},{"x":73,"y2":0,"group":1,"y":74.85440091977581},{"x":74,"y2":0,"group":1,"y":75.55317648562344},{"x":75,"y2":0,"group":1,"y":75.45503129141386},{"x":76,"y2":0,"group":1,"y":76.0223616001447},{"x":77,"y2":0,"group":1,"y":74.92783509731498},{"x":78,"y2":0,"group":1,"y":78.43358351371874},{"x":79,"y2":0,"group":1,"y":78.42153512627573},{"x":80,"y2":0,"group":1,"y":81.0553229766386},{"x":81,"y2":0,"group":1,"y":80.92601877652054},{"x":82,"y2":0,"group":1,"y":83.42152175231054},{"x":83,"y2":0,"group":1,"y":83.14269072793677},{"x":84,"y2":0,"group":1,"y":82.66049263182241},{"x":85,"y2":0,"group":1,"y":84.1226306401764},{"x":86,"y2":0,"group":1,"y":86.3227262400373},{"x":87,"y2":0,"group":1,"y":86.91795276393403},{"x":88,"y2":0,"group":1,"y":86.9567853292563},{"x":89,"y2":0,"group":1,"y":87.57613065700745},{"x":90,"y2":0,"group":1,"y":91.01575331700467},{"x":91,"y2":0,"group":1,"y":90.2519113725953},{"x":92,"y2":0,"group":1,"y":91.17616274054065},{"x":93,"y2":0,"group":1,"y":94.92983067686791},{"x":94,"y2":0,"group":1,"y":94.46040371685923},{"x":95,"y2":0,"group":1,"y":94.36177173493158},{"x":96,"y2":0,"group":1,"y":95.18760778064903},{"x":97,"y2":0,"group":1,"y":97.69778288965564},{"x":98,"y2":0,"group":1,"y":98.42700274410771},{"x":99,"y2":0,"group":1,"y":98.46411758202628},{"x":100,"y2":0,"group":1,"y":99.99237690251375},{"x":1,"y2":0,"group":2,"y":3.9685809095608855},{"x":2,"y2":0,"group":2,"y":6.149192141676936},{"x":3,"y2":0,"group":2,"y":8.593510164930834},{"x":4,"y2":0,"group":2,"y":9.98002741794392},{"x":5,"y2":0,"group":2,"y":11.451741017455385},{"x":6,"y2":0,"group":2,"y":10.779480801437428},{"x":7,"y2":0,"group":2,"y":12.673383443036764},{"x":8,"y2":0,"group":2,"y":15.779652415816079},{"x":9,"y2":0,"group":2,"y":16.07431024737684},{"x":10,"y2":0,"group":2,"y":19.718430791034823},{"x":11,"y2":0,"group":2,"y":18.44486731156199},{"x":12,"y2":0,"group":2,"y":20.36963341907325},{"x":13,"y2":0,"group":2,"y":21.538324403291618},{"x":14,"y2":0,"group":2,"y":24.216897481249482},{"x":15,"y2":0,"group":2,"y":25.44331378078537},{"x":16,"y2":0,"group":2,"y":28.17716062504715},{"x":17,"y2":0,"group":2,"y":29.324767259984746},{"x":18,"y2":0,"group":2,"y":28.590212431477553},{"x":19,"y2":0,"group":2,"y":33.647350741958796},{"x":20,"y2":0,"group":2,"y":31.94460195122258},{"x":21,"y2":0,"group":2,"y":32.10993058176156},{"x":22,"y2":0,"group":2,"y":37.176152378517315},{"x":23,"y2":0,"group":2,"y":37.107742736814956},{"x":24,"y2":0,"group":2,"y":39.40000330874773},{"x":25,"y2":0,"group":2,"y":40.669962925596174},{"x":26,"y2":0,"group":2,"y":42.08993134457572},{"x":27,"y2":0,"group":2,"y":44.30713225843759},{"x":28,"y2":0,"group":2,"y":42.288610524025245},{"x":29,"y2":0,"group":2,"y":48.53422810409889},{"x":30,"y2":0,"group":2,"y":48.990086221864914},{"x":31,"y2":0,"group":2,"y":49.43089313321655},{"x":32,"y2":0,"group":2,"y":52.70011370097463},{"x":33,"y2":0,"group":2,"y":51.59397651674797},{"x":34,"y2":0,"group":2,"y":53.325677936525764},{"x":35,"y2":0,"group":2,"y":55.8915802885221},{"x":36,"y2":0,"group":2,"y":56.70748947211033},{"x":37,"y2":0,"group":2,"y":58.749497717585434},{"x":38,"y2":0,"group":2,"y":59.527144181456336},{"x":39,"y2":0,"group":2,"y":60.348222325162354},{"x":40,"y2":0,"group":2,"y":63.54229414658719},{"x":41,"y2":0,"group":2,"y":64.67848723143688},{"x":42,"y2":0,"group":2,"y":64.92756798228612},{"x":43,"y2":0,"group":2,"y":67.76726519535438},{"x":44,"y2":0,"group":2,"y":71.21237589264443},{"x":45,"y2":0,"group":2,"y":69.77569015550937},{"x":46,"y2":0,"group":2,"y":73.1012721388432},{"x":47,"y2":0,"group":2,"y":73.59417127368785},{"x":48,"y2":0,"group":2,"y":75.84215611638032},{"x":49,"y2":0,"group":2,"y":76.73475947093556},{"x":50,"y2":0,"group":2,"y":77.46857405209},{"x":51,"y2":0,"group":2,"y":79.9144765767943},{"x":52,"y2":0,"group":2,"y":82.08105451776464},{"x":53,"y2":0,"group":2,"y":81.14068541228696},{"x":54,"y2":0,"group":2,"y":83.0548092512663},{"x":55,"y2":0,"group":2,"y":83.18705331674356},{"x":56,"y2":0,"group":2,"y":86.6682304790251},{"x":57,"y2":0,"group":2,"y":89.35823774633533},{"x":58,"y2":0,"group":2,"y":88.81169496810575},{"x":59,"y2":0,"group":2,"y":91.29064909938221},{"x":60,"y2":0,"group":2,"y":95.16679912340854},{"x":61,"y2":0,"group":2,"y":94.64503334447201},{"x":62,"y2":0,"group":2,"y":95.91747999563408},{"x":63,"y2":0,"group":2,"y":97.92024281683334},{"x":64,"y2":0,"group":2,"y":99.20470318694356},{"x":65,"y2":0,"group":2,"y":102.1070866223268},{"x":66,"y2":0,"group":2,"y":100.91658655349063},{"x":67,"y2":0,"group":2,"y":104.70489423099393},{"x":68,"y2":0,"group":2,"y":103.361897018721},{"x":69,"y2":0,"group":2,"y":105.24580823943818},{"x":70,"y2":0,"group":2,"y":106.76836951385988},{"x":71,"y2":0,"group":2,"y":110.06286156236393},{"x":72,"y2":0,"group":2,"y":109.88685994626938},{"x":73,"y2":0,"group":2,"y":112.55098178782217},{"x":74,"y2":0,"group":2,"y":114.07303570647427},{"x":75,"y2":0,"group":2,"y":113.55428122965438},{"x":76,"y2":0,"group":2,"y":116.83012081628749},{"x":77,"y2":0,"group":2,"y":119.12410650710446},{"x":78,"y2":0,"group":2,"y":118.93813378644354},{"x":79,"y2":0,"group":2,"y":120.17540213210881},{"x":80,"y2":0,"group":2,"y":124.40636982141221},{"x":81,"y2":0,"group":2,"y":124.44990914337959},{"x":82,"y2":0,"group":2,"y":124.88115555001386},{"x":83,"y2":0,"group":2,"y":124.88515959908439},{"x":84,"y2":0,"group":2,"y":128.98540062176556},{"x":85,"y2":0,"group":2,"y":129.89088154610207},{"x":86,"y2":0,"group":2,"y":133.02668234574597},{"x":87,"y2":0,"group":2,"y":133.94745458144092},{"x":88,"y2":0,"group":2,"y":134.42659505555756},{"x":89,"y2":0,"group":2,"y":134.9094695238586},{"x":90,"y2":0,"group":2,"y":138.85635910791504},{"x":91,"y2":0,"group":2,"y":138.13986278789577},{"x":92,"y2":0,"group":2,"y":142.05487993405137},{"x":93,"y2":0,"group":2,"y":141.16082214190453},{"x":94,"y2":0,"group":2,"y":143.40996538818},{"x":95,"y2":0,"group":2,"y":146.4727249582702},{"x":96,"y2":0,"group":2,"y":145.1238504449773},{"x":97,"y2":0,"group":2,"y":149.26984625418515},{"x":98,"y2":0,"group":2,"y":150.52203209488368},{"x":99,"y2":0,"group":2,"y":151.3308497817558},{"x":100,"y2":0,"group":2,"y":153.52865134160825}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"linear"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

	);
</script>
\endhtmlonly

### Making Points Larger


\htmlonly
<div id="lineplot3"></div>
<script type="text/javascript">
plot("lineplot3",
    {"name":"lineplot","height":450,"padding":"auto","marks":[{"marks":[{"properties":{"enter":{"stroke":{"field":"group","scale":"group"},"x":{"field":"x","scale":"x"},"strokeWidth":{"value":2},"y":{"field":"y","scale":"y"}}},"type":"line"},{"properties":{"enter":{"shape":{"value":"circle"},"stroke":{"field":"group","scale":"group"},"x":{"field":"x","scale":"x"},"strokeWidth":{"value":2},"size":{"value":20},"y":{"field":"y","scale":"y"}}},"type":"symbol"}],"from":{"data":"table","transform":[{"groupby":["group"],"type":"facet"}]},"type":"group"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":1,"y2":0,"group":1,"y":-0.21321369986580918},{"x":2,"y2":0,"group":1,"y":1.3985070732602853},{"x":3,"y2":0,"group":1,"y":2.8315969128222176},{"x":4,"y2":0,"group":1,"y":5.10847552856312},{"x":5,"y2":0,"group":1,"y":5.797644115914002},{"x":6,"y2":0,"group":1,"y":5.914610430797741},{"x":7,"y2":0,"group":1,"y":6.358133569529957},{"x":8,"y2":0,"group":1,"y":8.724427862853243},{"x":9,"y2":0,"group":1,"y":8.348623156839507},{"x":10,"y2":0,"group":1,"y":9.688147067106232},{"x":11,"y2":0,"group":1,"y":10.06112602165281},{"x":12,"y2":0,"group":1,"y":12.407836524004129},{"x":13,"y2":0,"group":1,"y":12.166519018329044},{"x":14,"y2":0,"group":1,"y":13.732125499038986},{"x":15,"y2":0,"group":1,"y":14.733586123242661},{"x":16,"y2":0,"group":1,"y":14.488612805533847},{"x":17,"y2":0,"group":1,"y":17.516473995333012},{"x":18,"y2":0,"group":1,"y":18.251977860211433},{"x":19,"y2":0,"group":1,"y":18.15792770870133},{"x":20,"y2":0,"group":1,"y":21.04063141312778},{"x":21,"y2":0,"group":1,"y":21.812152189304243},{"x":22,"y2":0,"group":1,"y":21.808301374717583},{"x":23,"y2":0,"group":1,"y":22.722977307050957},{"x":24,"y2":0,"group":1,"y":24.081096920490943},{"x":25,"y2":0,"group":1,"y":26.859041975538684},{"x":26,"y2":0,"group":1,"y":25.162163164847215},{"x":27,"y2":0,"group":1,"y":26.571685780072542},{"x":28,"y2":0,"group":1,"y":28.721574341634167},{"x":29,"y2":0,"group":1,"y":29.894064600438004},{"x":30,"y2":0,"group":1,"y":31.965721152914448},{"x":31,"y2":0,"group":1,"y":31.070567235019375},{"x":32,"y2":0,"group":1,"y":32.305612978826574},{"x":33,"y2":0,"group":1,"y":32.46357128268016},{"x":34,"y2":0,"group":1,"y":35.226342922540006},{"x":35,"y2":0,"group":1,"y":35.348539381967065},{"x":36,"y2":0,"group":1,"y":36.649149265710896},{"x":37,"y2":0,"group":1,"y":36.561535257826435},{"x":38,"y2":0,"group":1,"y":38.83016849547384},{"x":39,"y2":0,"group":1,"y":37.88531526555204},{"x":40,"y2":0,"group":1,"y":41.953830739664824},{"x":41,"y2":0,"group":1,"y":40.6704309868311},{"x":42,"y2":0,"group":1,"y":43.90959115734531},{"x":43,"y2":0,"group":1,"y":43.41137075670251},{"x":44,"y2":0,"group":1,"y":43.439708012187154},{"x":45,"y2":0,"group":1,"y":44.973493872253364},{"x":46,"y2":0,"group":1,"y":46.73305937810806},{"x":47,"y2":0,"group":1,"y":44.524780522197915},{"x":48,"y2":0,"group":1,"y":48.53010059957868},{"x":49,"y2":0,"group":1,"y":49.86780586391065},{"x":50,"y2":0,"group":1,"y":49.01557973572676},{"x":51,"y2":0,"group":1,"y":51.06983350683799},{"x":52,"y2":0,"group":1,"y":51.05888914249255},{"x":53,"y2":0,"group":1,"y":52.5115001831221},{"x":54,"y2":0,"group":1,"y":53.97393201664967},{"x":55,"y2":0,"group":1,"y":53.66053030674649},{"x":56,"y2":0,"group":1,"y":56.81971516458666},{"x":57,"y2":0,"group":1,"y":56.54959472971153},{"x":58,"y2":0,"group":1,"y":59.65139846648794},{"x":59,"y2":0,"group":1,"y":60.91139767808923},{"x":60,"y2":0,"group":1,"y":58.94542235843979},{"x":61,"y2":0,"group":1,"y":61.337909839945034},{"x":62,"y2":0,"group":1,"y":62.0909948309055},{"x":63,"y2":0,"group":1,"y":62.93246429097792},{"x":64,"y2":0,"group":1,"y":64.71022829267858},{"x":65,"y2":0,"group":1,"y":66.83522495852513},{"x":66,"y2":0,"group":1,"y":66.32649430944169},{"x":67,"y2":0,"group":1,"y":66.31951475210055},{"x":68,"y2":0,"group":1,"y":68.59769112425644},{"x":69,"y2":0,"group":1,"y":71.24129489433732},{"x":70,"y2":0,"group":1,"y":69.99823432374605},{"x":71,"y2":0,"group":1,"y":71.11322741154369},{"x":72,"y2":0,"group":1,"y":71.05344626161155},{"x":73,"y2":0,"group":1,"y":74.85440091977581},{"x":74,"y2":0,"group":1,"y":75.55317648562344},{"x":75,"y2":0,"group":1,"y":75.45503129141386},{"x":76,"y2":0,"group":1,"y":76.0223616001447},{"x":77,"y2":0,"group":1,"y":74.92783509731498},{"x":78,"y2":0,"group":1,"y":78.43358351371874},{"x":79,"y2":0,"group":1,"y":78.42153512627573},{"x":80,"y2":0,"group":1,"y":81.0553229766386},{"x":81,"y2":0,"group":1,"y":80.92601877652054},{"x":82,"y2":0,"group":1,"y":83.42152175231054},{"x":83,"y2":0,"group":1,"y":83.14269072793677},{"x":84,"y2":0,"group":1,"y":82.66049263182241},{"x":85,"y2":0,"group":1,"y":84.1226306401764},{"x":86,"y2":0,"group":1,"y":86.3227262400373},{"x":87,"y2":0,"group":1,"y":86.91795276393403},{"x":88,"y2":0,"group":1,"y":86.9567853292563},{"x":89,"y2":0,"group":1,"y":87.57613065700745},{"x":90,"y2":0,"group":1,"y":91.01575331700467},{"x":91,"y2":0,"group":1,"y":90.2519113725953},{"x":92,"y2":0,"group":1,"y":91.17616274054065},{"x":93,"y2":0,"group":1,"y":94.92983067686791},{"x":94,"y2":0,"group":1,"y":94.46040371685923},{"x":95,"y2":0,"group":1,"y":94.36177173493158},{"x":96,"y2":0,"group":1,"y":95.18760778064903},{"x":97,"y2":0,"group":1,"y":97.69778288965564},{"x":98,"y2":0,"group":1,"y":98.42700274410771},{"x":99,"y2":0,"group":1,"y":98.46411758202628},{"x":100,"y2":0,"group":1,"y":99.99237690251375},{"x":1,"y2":0,"group":2,"y":3.9685809095608855},{"x":2,"y2":0,"group":2,"y":6.149192141676936},{"x":3,"y2":0,"group":2,"y":8.593510164930834},{"x":4,"y2":0,"group":2,"y":9.98002741794392},{"x":5,"y2":0,"group":2,"y":11.451741017455385},{"x":6,"y2":0,"group":2,"y":10.779480801437428},{"x":7,"y2":0,"group":2,"y":12.673383443036764},{"x":8,"y2":0,"group":2,"y":15.779652415816079},{"x":9,"y2":0,"group":2,"y":16.07431024737684},{"x":10,"y2":0,"group":2,"y":19.718430791034823},{"x":11,"y2":0,"group":2,"y":18.44486731156199},{"x":12,"y2":0,"group":2,"y":20.36963341907325},{"x":13,"y2":0,"group":2,"y":21.538324403291618},{"x":14,"y2":0,"group":2,"y":24.216897481249482},{"x":15,"y2":0,"group":2,"y":25.44331378078537},{"x":16,"y2":0,"group":2,"y":28.17716062504715},{"x":17,"y2":0,"group":2,"y":29.324767259984746},{"x":18,"y2":0,"group":2,"y":28.590212431477553},{"x":19,"y2":0,"group":2,"y":33.647350741958796},{"x":20,"y2":0,"group":2,"y":31.94460195122258},{"x":21,"y2":0,"group":2,"y":32.10993058176156},{"x":22,"y2":0,"group":2,"y":37.176152378517315},{"x":23,"y2":0,"group":2,"y":37.107742736814956},{"x":24,"y2":0,"group":2,"y":39.40000330874773},{"x":25,"y2":0,"group":2,"y":40.669962925596174},{"x":26,"y2":0,"group":2,"y":42.08993134457572},{"x":27,"y2":0,"group":2,"y":44.30713225843759},{"x":28,"y2":0,"group":2,"y":42.288610524025245},{"x":29,"y2":0,"group":2,"y":48.53422810409889},{"x":30,"y2":0,"group":2,"y":48.990086221864914},{"x":31,"y2":0,"group":2,"y":49.43089313321655},{"x":32,"y2":0,"group":2,"y":52.70011370097463},{"x":33,"y2":0,"group":2,"y":51.59397651674797},{"x":34,"y2":0,"group":2,"y":53.325677936525764},{"x":35,"y2":0,"group":2,"y":55.8915802885221},{"x":36,"y2":0,"group":2,"y":56.70748947211033},{"x":37,"y2":0,"group":2,"y":58.749497717585434},{"x":38,"y2":0,"group":2,"y":59.527144181456336},{"x":39,"y2":0,"group":2,"y":60.348222325162354},{"x":40,"y2":0,"group":2,"y":63.54229414658719},{"x":41,"y2":0,"group":2,"y":64.67848723143688},{"x":42,"y2":0,"group":2,"y":64.92756798228612},{"x":43,"y2":0,"group":2,"y":67.76726519535438},{"x":44,"y2":0,"group":2,"y":71.21237589264443},{"x":45,"y2":0,"group":2,"y":69.77569015550937},{"x":46,"y2":0,"group":2,"y":73.1012721388432},{"x":47,"y2":0,"group":2,"y":73.59417127368785},{"x":48,"y2":0,"group":2,"y":75.84215611638032},{"x":49,"y2":0,"group":2,"y":76.73475947093556},{"x":50,"y2":0,"group":2,"y":77.46857405209},{"x":51,"y2":0,"group":2,"y":79.9144765767943},{"x":52,"y2":0,"group":2,"y":82.08105451776464},{"x":53,"y2":0,"group":2,"y":81.14068541228696},{"x":54,"y2":0,"group":2,"y":83.0548092512663},{"x":55,"y2":0,"group":2,"y":83.18705331674356},{"x":56,"y2":0,"group":2,"y":86.6682304790251},{"x":57,"y2":0,"group":2,"y":89.35823774633533},{"x":58,"y2":0,"group":2,"y":88.81169496810575},{"x":59,"y2":0,"group":2,"y":91.29064909938221},{"x":60,"y2":0,"group":2,"y":95.16679912340854},{"x":61,"y2":0,"group":2,"y":94.64503334447201},{"x":62,"y2":0,"group":2,"y":95.91747999563408},{"x":63,"y2":0,"group":2,"y":97.92024281683334},{"x":64,"y2":0,"group":2,"y":99.20470318694356},{"x":65,"y2":0,"group":2,"y":102.1070866223268},{"x":66,"y2":0,"group":2,"y":100.91658655349063},{"x":67,"y2":0,"group":2,"y":104.70489423099393},{"x":68,"y2":0,"group":2,"y":103.361897018721},{"x":69,"y2":0,"group":2,"y":105.24580823943818},{"x":70,"y2":0,"group":2,"y":106.76836951385988},{"x":71,"y2":0,"group":2,"y":110.06286156236393},{"x":72,"y2":0,"group":2,"y":109.88685994626938},{"x":73,"y2":0,"group":2,"y":112.55098178782217},{"x":74,"y2":0,"group":2,"y":114.07303570647427},{"x":75,"y2":0,"group":2,"y":113.55428122965438},{"x":76,"y2":0,"group":2,"y":116.83012081628749},{"x":77,"y2":0,"group":2,"y":119.12410650710446},{"x":78,"y2":0,"group":2,"y":118.93813378644354},{"x":79,"y2":0,"group":2,"y":120.17540213210881},{"x":80,"y2":0,"group":2,"y":124.40636982141221},{"x":81,"y2":0,"group":2,"y":124.44990914337959},{"x":82,"y2":0,"group":2,"y":124.88115555001386},{"x":83,"y2":0,"group":2,"y":124.88515959908439},{"x":84,"y2":0,"group":2,"y":128.98540062176556},{"x":85,"y2":0,"group":2,"y":129.89088154610207},{"x":86,"y2":0,"group":2,"y":133.02668234574597},{"x":87,"y2":0,"group":2,"y":133.94745458144092},{"x":88,"y2":0,"group":2,"y":134.42659505555756},{"x":89,"y2":0,"group":2,"y":134.9094695238586},{"x":90,"y2":0,"group":2,"y":138.85635910791504},{"x":91,"y2":0,"group":2,"y":138.13986278789577},{"x":92,"y2":0,"group":2,"y":142.05487993405137},{"x":93,"y2":0,"group":2,"y":141.16082214190453},{"x":94,"y2":0,"group":2,"y":143.40996538818},{"x":95,"y2":0,"group":2,"y":146.4727249582702},{"x":96,"y2":0,"group":2,"y":145.1238504449773},{"x":97,"y2":0,"group":2,"y":149.26984625418515},{"x":98,"y2":0,"group":2,"y":150.52203209488368},{"x":99,"y2":0,"group":2,"y":151.3308497817558},{"x":100,"y2":0,"group":2,"y":153.52865134160825}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"linear"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

	);
</script>
\endhtmlonly


# Pie/Donut Chart

### Pie Chart


\htmlonly
<div id="pie"></div>
<script type="text/javascript">
plot("pie",
    {"name":"Vega Visualization","height":250,"padding":"auto","marks":[{"properties":{"enter":{"innerRadius":{"value":0},"stroke":{"value":"white"},"startAngle":{"field":"layout_start"},"x":{"mult":0.5,"group":"width"},"outerRadius":{"value":250},"endAngle":{"field":"layout_end"},"fill":{"field":"x","scale":"color"},"y":{"mult":0.5,"group":"height"}}},"from":{"data":"table","transform":[{"field":"y","type":"pie"}]},"type":"arc"}],"data":[{"name":"table","values":[{"x":"peaches","y2":0,"group":1,"y":100},{"x":"plums","y2":0,"group":1,"y":32},{"x":"blueberries","y2":0,"group":1,"y":180},{"x":"strawberries","y2":0,"group":1,"y":46},{"x":"bananas","y2":0,"group":1,"y":21}]}],"scales":[{"name":"color","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"x"},"type":"ordinal"}],"width":250,"legends":[{"title":"group","fill":"color"}]}
    );

</script>
\endhtmlonly

### Donut Chart


\htmlonly
<div id="donut"></div>
<script type="text/javascript">
plot("donut",
    {"name":"Vega Visualization","height":250,"padding":"auto","marks":[{"properties":{"enter":{"innerRadius":{"value":175},"stroke":{"value":"white"},"startAngle":{"field":"layout_start"},"x":{"mult":0.5,"group":"width"},"outerRadius":{"value":250},"endAngle":{"field":"layout_end"},"fill":{"field":"x","scale":"color"},"y":{"mult":0.5,"group":"height"}}},"from":{"data":"table","transform":[{"field":"y","type":"pie"}]},"type":"arc"}],"data":[{"name":"table","values":[{"x":"peaches","y2":0,"group":1,"y":100},{"x":"plums","y2":0,"group":1,"y":32},{"x":"blueberries","y2":0,"group":1,"y":180},{"x":"strawberries","y2":0,"group":1,"y":46},{"x":"bananas","y2":0,"group":1,"y":21}]}],"scales":[{"name":"color","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"x"},"type":"ordinal"}],"width":250,"legends":[{"title":"group","fill":"color"}]});
</script>
\endhtmlonly


# Scatter Plot


### Scatterplot By Group


\htmlonly
<div id="scatter"></div>
<script type="text/javascript">
plot("scatter",
    {"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"shape":{"value":"circle"},"x":{"field":"x","scale":"x"},"size":{"value":50},"fill":{"field":"group","scale":"group"},"y":{"field":"y","scale":"y"}}},"from":{"data":"table"},"type":"symbol"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":-0.4642751223803044,"y2":0,"group":1,"y":-0.2503646637277523},{"x":-1.8716779950817728,"y2":0,"group":1,"y":-1.578974916760044},{"x":1.2834839567369596,"y2":0,"group":1,"y":0.45017279932751375},{"x":0.25070669742711027,"y2":0,"group":1,"y":0.43198048486221907},{"x":-0.027747960670703786,"y2":0,"group":1,"y":-1.2068137023478662},{"x":0.5948085930807692,"y2":0,"group":1,"y":0.298621844834038},{"x":0.15126693570634694,"y2":0,"group":1,"y":-0.1525110388208707},{"x":1.4413771675590537,"y2":0,"group":1,"y":0.8926914927224905},{"x":-0.24507439715855223,"y2":0,"group":1,"y":-0.7126330344668806},{"x":-0.0411712726402745,"y2":0,"group":1,"y":-0.3187698518619378},{"x":-0.6326175683078022,"y2":0,"group":1,"y":-0.8356461669371509},{"x":0.025198973847480495,"y2":0,"group":1,"y":0.49482407098521286},{"x":-0.42535105066768003,"y2":0,"group":1,"y":-0.5263249345551149},{"x":-0.23399273472735466,"y2":0,"group":1,"y":-0.0736105423894936},{"x":1.079669882823539,"y2":0,"group":1,"y":0.860796550824055},{"x":1.2282904625122837,"y2":0,"group":1,"y":2.455758648411483},{"x":-1.1181920084701893,"y2":0,"group":1,"y":-0.8809188577372269},{"x":-1.0836489551707837,"y2":0,"group":1,"y":-0.6811735436677737},{"x":-1.0793075393638392,"y2":0,"group":1,"y":-0.6728331975175583},{"x":-0.012480239919086597,"y2":0,"group":1,"y":0.27667251778079865},{"x":-1.6938414885783226,"y2":0,"group":1,"y":-1.536367370019656},{"x":-0.6012211936842374,"y2":0,"group":1,"y":-0.4701421117217889},{"x":0.7722545574035417,"y2":0,"group":1,"y":1.047687813792098},{"x":1.027388084862333,"y2":0,"group":1,"y":1.0779043720319914},{"x":0.5289679352044544,"y2":0,"group":1,"y":1.3672637698959598},{"x":0.45243075349816264,"y2":0,"group":1,"y":-0.07561000534155227},{"x":-0.41155974530228795,"y2":0,"group":1,"y":-0.6331778409832457},{"x":2.1912722342329753,"y2":0,"group":1,"y":2.3903513940137446},{"x":-1.6259448281238138,"y2":0,"group":1,"y":-1.33888277832192},{"x":0.7081851149878344,"y2":0,"group":1,"y":1.3200511985668215},{"x":0.3501296225034444,"y2":0,"group":1,"y":0.37663181323165085},{"x":-1.2196822136309629,"y2":0,"group":1,"y":-0.7472736732651672},{"x":-0.8808754436246625,"y2":0,"group":1,"y":-0.28347441761156444},{"x":-0.43706500759648437,"y2":0,"group":1,"y":-0.0624608704662713},{"x":-1.3368086355424411,"y2":0,"group":1,"y":-1.7845778854734666},{"x":-0.9770763926701824,"y2":0,"group":1,"y":-1.3136695513355563},{"x":0.0892730334545257,"y2":0,"group":1,"y":-0.03598785044064019},{"x":-0.024066679633158457,"y2":0,"group":1,"y":-0.358288685761583},{"x":1.894851464816175,"y2":0,"group":1,"y":2.154605736812719},{"x":-0.7737136449212324,"y2":0,"group":1,"y":-0.6762541679511268},{"x":1.1478273377534143,"y2":0,"group":1,"y":1.0847823745278686},{"x":-0.0778428325582762,"y2":0,"group":1,"y":-0.4988854122939156},{"x":-0.031730560221279416,"y2":0,"group":1,"y":-0.0028442803020883277},{"x":-1.017742981703117,"y2":0,"group":1,"y":0.10363754696240102},{"x":0.9470928573090313,"y2":0,"group":1,"y":0.17174855648971443},{"x":-1.4800085242276497,"y2":0,"group":1,"y":-0.46149121610987975},{"x":-0.5565548499327778,"y2":0,"group":1,"y":-0.7482391547451313},{"x":-1.3577498993719774,"y2":0,"group":1,"y":-1.6898279980200575},{"x":-0.8070024424114242,"y2":0,"group":1,"y":-0.8654809368572007},{"x":-0.5052278715996145,"y2":0,"group":1,"y":-0.4797362068002291},{"x":-0.5939834201767955,"y2":0,"group":1,"y":-1.1230824633444891},{"x":1.1829131844554541,"y2":0,"group":1,"y":1.0153087734360406},{"x":0.8067189301314597,"y2":0,"group":1,"y":1.541880186588144},{"x":-0.7950293112315797,"y2":0,"group":1,"y":-0.7479342468327473},{"x":0.8833221153279999,"y2":0,"group":1,"y":1.089242714609463},{"x":1.1976846748796277,"y2":0,"group":1,"y":0.9229250879183313},{"x":0.3059771711012314,"y2":0,"group":1,"y":0.49803115858787883},{"x":0.7267165600253392,"y2":0,"group":1,"y":0.706132016427299},{"x":1.2701696125984547,"y2":0,"group":1,"y":1.0805752875623702},{"x":0.2330337695101241,"y2":0,"group":1,"y":0.6806131955202175},{"x":-0.24481554013187648,"y2":0,"group":1,"y":-0.5249903420062976},{"x":-0.6323504840333269,"y2":0,"group":1,"y":-0.37185191645102594},{"x":0.4301755481758339,"y2":0,"group":1,"y":0.469505513962583},{"x":0.32701177767668155,"y2":0,"group":1,"y":0.1292129673314419},{"x":-1.4983376894576,"y2":0,"group":1,"y":-1.3734869474360631},{"x":0.0831464520480676,"y2":0,"group":1,"y":0.14590643794741207},{"x":0.34388962322447886,"y2":0,"group":1,"y":0.43063545322110947},{"x":-0.854277474860592,"y2":0,"group":1,"y":-1.0432015907729457},{"x":-0.7970120918821237,"y2":0,"group":1,"y":-0.8879197773531774},{"x":0.13666193683078445,"y2":0,"group":1,"y":-0.2435861655663465},{"x":0.2288056737810377,"y2":0,"group":1,"y":1.1615858246272812},{"x":0.6357198651024563,"y2":0,"group":1,"y":1.299821318086118},{"x":0.4543736039405554,"y2":0,"group":1,"y":0.2718083717507066},{"x":1.0488368302116753,"y2":0,"group":1,"y":0.9849741963465878},{"x":-1.2425358077241544,"y2":0,"group":1,"y":-1.2061918730266652},{"x":-1.0862649725842746,"y2":0,"group":1,"y":-0.5552653683450488},{"x":0.3546480018631953,"y2":0,"group":1,"y":0.6868309707803013},{"x":1.3311656203854092,"y2":0,"group":1,"y":0.8321136456552167},{"x":0.31552994709375864,"y2":0,"group":1,"y":0.4061129848273907},{"x":-0.4547140098714807,"y2":0,"group":1,"y":-0.34239373124397987},{"x":0.945578059530549,"y2":0,"group":1,"y":1.2360513571229714},{"x":0.15868122676606333,"y2":0,"group":1,"y":0.3678005036194633},{"x":1.9024740333377592,"y2":0,"group":1,"y":2.3525342275923635},{"x":-1.301492032772107,"y2":0,"group":1,"y":-0.6888353265917465},{"x":0.5051290613950771,"y2":0,"group":1,"y":0.07475572810373943},{"x":0.2703808413508291,"y2":0,"group":1,"y":0.8587188120513003},{"x":0.9253718008038574,"y2":0,"group":1,"y":1.0452451470784525},{"x":-1.9445436240267517,"y2":0,"group":1,"y":-2.197222036418865},{"x":1.0160313243193715,"y2":0,"group":1,"y":1.007978712662309},{"x":-1.0503303447357553,"y2":0,"group":1,"y":-1.4502203342617688},{"x":1.2049290038706832,"y2":0,"group":1,"y":1.526758876416946},{"x":-1.3277511653902963,"y2":0,"group":1,"y":-0.7642901199899212},{"x":0.3784034247347551,"y2":0,"group":1,"y":-0.10327813597405439},{"x":0.4677655582526878,"y2":0,"group":1,"y":0.2913528004188365},{"x":-0.686025126487273,"y2":0,"group":1,"y":-0.3672613736562226},{"x":-0.2447083790887922,"y2":0,"group":1,"y":-0.19989640606982906},{"x":1.0733260321904665,"y2":0,"group":1,"y":0.47371077397412087},{"x":0.26312757305513274,"y2":0,"group":1,"y":0.3781148434984239},{"x":-1.2905749354161364,"y2":0,"group":1,"y":-0.9999524572838662},{"x":-0.07665865043390155,"y2":0,"group":1,"y":-0.3503055018318615},{"x":-0.7835623452501583,"y2":0,"group":1,"y":-0.9273143224559369},{"x":1.3571838001018273,"y2":0,"group":1,"y":1.3212043813382015},{"x":0.6272124614308094,"y2":0,"group":1,"y":0.9322877149845732},{"x":0.8322917737051914,"y2":0,"group":1,"y":0.4995225056219081},{"x":1.0820607374451878,"y2":0,"group":1,"y":1.255392792977815},{"x":-0.5347075865276188,"y2":0,"group":1,"y":-0.05832091820766527},{"x":2.025038549700351,"y2":0,"group":1,"y":2.2929857468809662},{"x":-1.332440890594508,"y2":0,"group":1,"y":-0.26294482599575586},{"x":-1.5625513256874515,"y2":0,"group":1,"y":-1.2644585219513025},{"x":-0.22332119762366068,"y2":0,"group":1,"y":-1.008661355702416},{"x":0.21055590689779952,"y2":0,"group":1,"y":-0.34866872698383533},{"x":2.449375338493735,"y2":0,"group":1,"y":1.4868279364582955},{"x":1.5309119595686054,"y2":0,"group":1,"y":1.1389007404774683},{"x":0.3736741934427466,"y2":0,"group":1,"y":-0.07460238156397819},{"x":-0.818074000872076,"y2":0,"group":1,"y":-0.9649216149285887},{"x":-1.3649337331620568,"y2":0,"group":1,"y":-1.4636027815029233},{"x":-0.018547443320833277,"y2":0,"group":1,"y":-0.2595562399317709},{"x":-0.6680916043733529,"y2":0,"group":1,"y":-1.4708770893983036},{"x":0.26587281318165024,"y2":0,"group":1,"y":-0.014657702442744724},{"x":-0.2201667447043086,"y2":0,"group":1,"y":0.5932384402703627},{"x":1.1697458681306991,"y2":0,"group":1,"y":1.444698518530278},{"x":1.6626326600124706,"y2":0,"group":1,"y":1.2407309938042894},{"x":-1.3044605619833396,"y2":0,"group":1,"y":-1.7292987875286943},{"x":-0.5601225061783428,"y2":0,"group":1,"y":-0.6282947860065923},{"x":-1.56107345154556,"y2":0,"group":1,"y":-0.8757348653701684},{"x":-0.17617838333227193,"y2":0,"group":1,"y":-0.3835810419427087},{"x":0.5188112802652413,"y2":0,"group":1,"y":1.1999763289142489},{"x":0.2544013300250459,"y2":0,"group":1,"y":-0.6789884221790008},{"x":-1.4470259102667693,"y2":0,"group":1,"y":-1.6881344684547543},{"x":-0.7936896126777311,"y2":0,"group":1,"y":-0.10407738692524815},{"x":-1.1072244981119361,"y2":0,"group":1,"y":-1.076428862884904},{"x":0.5058495158225965,"y2":0,"group":1,"y":0.3356410466621853},{"x":-0.5855076535115261,"y2":0,"group":1,"y":-1.2833273670973537},{"x":0.6387404562742743,"y2":0,"group":1,"y":0.854174824411448},{"x":-0.8445413318667256,"y2":0,"group":1,"y":-1.2194581757132341},{"x":-1.4161532634546363,"y2":0,"group":1,"y":-1.623055791690466},{"x":0.23022116650273755,"y2":0,"group":1,"y":-0.280683984907212},{"x":-0.5225008795005956,"y2":0,"group":1,"y":-0.7874377472086563},{"x":-1.6167130289391929,"y2":0,"group":1,"y":-1.0038809443029577},{"x":-0.04718330065902109,"y2":0,"group":1,"y":0.17935082547542178},{"x":0.6577411418500864,"y2":0,"group":1,"y":1.4105198774441765},{"x":1.133447024306916,"y2":0,"group":1,"y":0.5331759801744188},{"x":-0.8133840617659658,"y2":0,"group":1,"y":-1.198672427838679},{"x":-0.8138888698405233,"y2":0,"group":1,"y":-1.0469839279769935},{"x":-0.24682027172595264,"y2":0,"group":1,"y":-0.31334706431813},{"x":1.6247226425973016,"y2":0,"group":1,"y":1.9701068116455867},{"x":-0.6925201266905849,"y2":0,"group":1,"y":-1.1664272730035694},{"x":-2.559305042005615,"y2":0,"group":1,"y":-2.0833567327131024},{"x":0.03142246080406075,"y2":0,"group":1,"y":-0.4250959666129085},{"x":-0.9216803373048422,"y2":0,"group":1,"y":-0.4003754412203084},{"x":0.2609002977455508,"y2":0,"group":1,"y":-0.47593458347773687},{"x":-0.16956570351023506,"y2":0,"group":1,"y":-0.5735235115308627},{"x":-0.29614763358607327,"y2":0,"group":1,"y":0.6542425779612345},{"x":0.4054774247001145,"y2":0,"group":1,"y":-0.5452901543770758},{"x":-0.11939916762377127,"y2":0,"group":1,"y":-0.28155454840299465},{"x":-0.14499031233669124,"y2":0,"group":1,"y":-0.3326699646636201},{"x":0.18122859891544163,"y2":0,"group":1,"y":-0.2663254200857251},{"x":-0.34560980268214353,"y2":0,"group":1,"y":-0.6885252157192869},{"x":0.9600686913582076,"y2":0,"group":1,"y":1.462180587388889},{"x":0.17616080650753085,"y2":0,"group":1,"y":-0.32028955332510134},{"x":0.9433248414804098,"y2":0,"group":1,"y":0.5002755958283518},{"x":0.06877975346423694,"y2":0,"group":1,"y":0.024719264915629875},{"x":0.7007780581210232,"y2":0,"group":1,"y":0.27457378759512724},{"x":-0.0004427873836226274,"y2":0,"group":1,"y":0.8004619033683343},{"x":0.5814281940142395,"y2":0,"group":1,"y":0.8491090728252868},{"x":-1.0810321600593642,"y2":0,"group":1,"y":-0.28604093047607915},{"x":-0.3706019820775156,"y2":0,"group":1,"y":-0.7550826534058462},{"x":0.8783862193960696,"y2":0,"group":1,"y":1.2969067633060436},{"x":1.2120052784499387,"y2":0,"group":1,"y":0.551543088094536},{"x":0.6113120047023273,"y2":0,"group":1,"y":-0.22541095069483985},{"x":1.1702332972236202,"y2":0,"group":1,"y":1.762647044685428},{"x":-1.298371228045187,"y2":0,"group":1,"y":-1.335447252770259},{"x":0.12849166390607109,"y2":0,"group":1,"y":0.22159437822989644},{"x":-1.379139161090028,"y2":0,"group":1,"y":-2.1529302386901574},{"x":-0.7849203106600593,"y2":0,"group":1,"y":-0.6875181598431633},{"x":0.8781914044621542,"y2":0,"group":1,"y":1.1621878353031003},{"x":-0.5564150453233977,"y2":0,"group":1,"y":-0.9751592409287186},{"x":0.6548025126026146,"y2":0,"group":1,"y":0.5697652808680084},{"x":1.3543633607092018,"y2":0,"group":1,"y":2.0162291985541105},{"x":0.38170708116491275,"y2":0,"group":1,"y":0.38691247706366927},{"x":1.1405153812319395,"y2":0,"group":1,"y":1.433863160989929},{"x":0.6092185068514575,"y2":0,"group":1,"y":0.22443100688022993},{"x":0.5391143977570663,"y2":0,"group":1,"y":0.2479886950462346},{"x":-1.2475172709006248,"y2":0,"group":1,"y":-1.1741924852505006},{"x":1.9791364212898366,"y2":0,"group":1,"y":1.299974023179697},{"x":0.35548282914228146,"y2":0,"group":1,"y":0.36990683506624483},{"x":0.23559134571385137,"y2":0,"group":1,"y":-0.22309968748851594},{"x":0.2270250192554848,"y2":0,"group":1,"y":0.3144926799594547},{"x":-1.0338622444976904,"y2":0,"group":1,"y":-0.8708719173006817},{"x":1.2076031316514135,"y2":0,"group":1,"y":1.163820853196874},{"x":-0.5836755616001961,"y2":0,"group":1,"y":-0.3760914358625505},{"x":0.7950970280226061,"y2":0,"group":1,"y":1.0927183933464995},{"x":0.9723204856419131,"y2":0,"group":1,"y":1.172741362664619},{"x":-1.8076532524572289,"y2":0,"group":1,"y":-1.6453769130088425},{"x":-0.9301731466555879,"y2":0,"group":1,"y":-0.5512328144303963},{"x":-1.432182087954551,"y2":0,"group":1,"y":-1.3943757874959797},{"x":0.6104778600749536,"y2":0,"group":1,"y":0.7133967030974208},{"x":-0.9196737973184611,"y2":0,"group":1,"y":-1.7058805225438625},{"x":0.7013104125438486,"y2":0,"group":1,"y":0.2799568183261442},{"x":0.12686858247773855,"y2":0,"group":1,"y":0.1916132366779168},{"x":-0.5876211801820221,"y2":0,"group":1,"y":-0.38476224359119765},{"x":-0.9681610637038954,"y2":0,"group":1,"y":-0.5450575324167555},{"x":-0.13251923784023475,"y2":0,"group":1,"y":-0.2351921049138417},{"x":-0.4602788796618786,"y2":0,"group":1,"y":-0.3917283737219601},{"x":-0.3779707961506874,"y2":0,"group":1,"y":0.44282590056074755},{"x":-0.4811975031265405,"y2":0,"group":1,"y":-0.9467673138555178},{"x":-0.16224184591711407,"y2":0,"group":1,"y":0.2373622665396336},{"x":0.5812634914531922,"y2":0,"group":1,"y":0.81219807575343},{"x":-1.4905062893575676,"y2":0,"group":1,"y":-2.0764726175838835},{"x":0.7733633670706315,"y2":0,"group":1,"y":1.6106789898158762},{"x":0.19443677368101123,"y2":0,"group":1,"y":0.4419730971875962},{"x":-3.387761478552098,"y2":0,"group":1,"y":-2.8068315708990483},{"x":0.08276227734380114,"y2":0,"group":1,"y":-0.3530252551440165},{"x":-1.0091434115473852,"y2":0,"group":1,"y":-0.538758828530685},{"x":0.694115606791009,"y2":0,"group":1,"y":0.6122731227436851},{"x":0.23493125631008016,"y2":0,"group":1,"y":0.23135506295380412},{"x":-1.95406779735253,"y2":0,"group":1,"y":-1.7086970519951892},{"x":-0.5842636302977398,"y2":0,"group":1,"y":-0.11370341248543686},{"x":0.22334982792623753,"y2":0,"group":1,"y":1.2339159434375004},{"x":-1.0536094087528693,"y2":0,"group":1,"y":-1.088016918463308},{"x":-0.879107975122606,"y2":0,"group":1,"y":-0.7919965139722042},{"x":2.6393446239565117,"y2":0,"group":1,"y":1.7240064693046306},{"x":0.08305395355595128,"y2":0,"group":1,"y":-0.3535826569606498},{"x":-1.15904986211766,"y2":0,"group":1,"y":-1.2169161168055604},{"x":1.2599764343192694,"y2":0,"group":1,"y":0.5479308587706725},{"x":-1.0181304613080038,"y2":0,"group":1,"y":-1.4059295231498286},{"x":1.9378371880406835,"y2":0,"group":1,"y":1.6678846690426463},{"x":1.1247795714589006,"y2":0,"group":1,"y":0.9115342018247045},{"x":0.2810510812902331,"y2":0,"group":1,"y":0.5250608230682647},{"x":-0.4613030431781608,"y2":0,"group":1,"y":0.48567993458970343},{"x":-0.37682970572151,"y2":0,"group":1,"y":-0.07738793575959246},{"x":0.18998298936577332,"y2":0,"group":1,"y":0.08537315059790167},{"x":0.02900806120247077,"y2":0,"group":1,"y":0.20614194602655814},{"x":0.46655785645535136,"y2":0,"group":1,"y":0.041387599675757614},{"x":-0.4888496312893456,"y2":0,"group":1,"y":-0.17358396736528264},{"x":-0.294582315308181,"y2":0,"group":1,"y":-1.1370437441994936},{"x":-1.365589091866473,"y2":0,"group":1,"y":-0.9483357465657988},{"x":-0.4728827549893476,"y2":0,"group":1,"y":-0.7095317142197644},{"x":0.40451650982047044,"y2":0,"group":1,"y":0.343910513476048},{"x":-0.05360813022937411,"y2":0,"group":1,"y":-0.7208577645105546},{"x":-0.4628661875310697,"y2":0,"group":1,"y":0.14046189572960704},{"x":-0.7447890270890134,"y2":0,"group":1,"y":-0.9639145843858818},{"x":0.21482098444743136,"y2":0,"group":1,"y":0.9012344561411878},{"x":1.161974461119622,"y2":0,"group":1,"y":1.1747607371972464},{"x":0.14772108534792935,"y2":0,"group":1,"y":0.22888795658412908},{"x":-0.4121795579983426,"y2":0,"group":1,"y":-0.02857799520698573},{"x":-1.8112142632506107,"y2":0,"group":1,"y":-1.8435978968554605},{"x":1.285597166407998,"y2":0,"group":1,"y":1.1163793359832657},{"x":2.4656613392501976,"y2":0,"group":1,"y":1.6611346207144906},{"x":-0.4330551916276194,"y2":0,"group":1,"y":-0.4704573559634955},{"x":0.9038638424182783,"y2":0,"group":1,"y":1.181109576254857},{"x":-0.5741197523142149,"y2":0,"group":1,"y":-0.4053311162325396},{"x":-1.2791722985161684,"y2":0,"group":1,"y":-0.18803751178331396},{"x":1.2927536065264706,"y2":0,"group":1,"y":1.5932714964792305},{"x":-0.45810993174206854,"y2":0,"group":1,"y":-0.7558578715359399},{"x":0.7396057492978035,"y2":0,"group":1,"y":0.5228069805879527},{"x":-0.7934674962472009,"y2":0,"group":1,"y":-0.27963355392717987},{"x":-0.11589038083734927,"y2":0,"group":1,"y":-0.07850597538430651},{"x":-2.0258712322574564,"y2":0,"group":1,"y":-0.7639466537824404},{"x":-0.013543761705711997,"y2":0,"group":1,"y":-0.3384347894954335},{"x":1.0276120051929962,"y2":0,"group":1,"y":0.9094591464795775},{"x":0.2890992958433101,"y2":0,"group":1,"y":-0.844789660527433},{"x":0.4269324372652057,"y2":0,"group":1,"y":0.2049349784706948},{"x":1.1167312035415844,"y2":0,"group":1,"y":1.528244457672059},{"x":0.9757308889601648,"y2":0,"group":1,"y":0.8229500231514469},{"x":0.47436925541408587,"y2":0,"group":1,"y":0.46054787900246835},{"x":-0.36901506052027583,"y2":0,"group":1,"y":-0.07013410275429621},{"x":0.835043545771125,"y2":0,"group":1,"y":0.06280700650453552},{"x":0.7662154543410132,"y2":0,"group":1,"y":0.14766971555649},{"x":-0.7532657719865447,"y2":0,"group":1,"y":-0.9826156408011958},{"x":0.5816315822793359,"y2":0,"group":1,"y":0.11411921527334717},{"x":-0.16368989368125264,"y2":0,"group":1,"y":0.6281414214162055},{"x":0.45289714094749206,"y2":0,"group":1,"y":0.2155248647962249},{"x":-0.018680674283439967,"y2":0,"group":1,"y":-0.21600868134312068},{"x":-0.15026902349097693,"y2":0,"group":1,"y":-0.20553908737795384},{"x":-0.18440653191128784,"y2":0,"group":1,"y":-0.1609681781769558},{"x":-0.1622887346361862,"y2":0,"group":1,"y":-0.07160612705302262},{"x":0.02491415986813111,"y2":0,"group":1,"y":0.29696932079598615},{"x":-0.4410485606674079,"y2":0,"group":1,"y":-0.13224594661856337},{"x":0.37795981478644797,"y2":0,"group":1,"y":0.1119626245183126},{"x":-2.8801054462578937,"y2":0,"group":1,"y":-2.477435128767076},{"x":-0.5715146856213696,"y2":0,"group":1,"y":-0.4335375228972676},{"x":0.25339213236515173,"y2":0,"group":1,"y":-0.1908434450050761},{"x":-0.5791255359489107,"y2":0,"group":1,"y":-0.564856774515934},{"x":-1.116556200448016,"y2":0,"group":1,"y":-1.3925678915712418},{"x":-0.20927915161381352,"y2":0,"group":1,"y":-0.2168225383125326},{"x":-0.510586851965152,"y2":0,"group":1,"y":0.6176349216879455},{"x":0.41949754090802316,"y2":0,"group":1,"y":0.40976990655477485},{"x":-0.37458926276322263,"y2":0,"group":1,"y":-0.9676147573733828},{"x":1.3520146490693914,"y2":0,"group":1,"y":1.0442247439700205},{"x":1.2313928614631993,"y2":0,"group":1,"y":1.8036945354505705},{"x":0.4108686784832596,"y2":0,"group":1,"y":0.6380467291468314},{"x":-0.7384214083391758,"y2":0,"group":1,"y":-0.4968389851940948},{"x":-0.9624092410941198,"y2":0,"group":1,"y":-1.515689600062865},{"x":0.9043676773971286,"y2":0,"group":1,"y":0.6201098547279178},{"x":-0.9630512635693861,"y2":0,"group":1,"y":-1.423364565734439},{"x":0.5110213055094236,"y2":0,"group":1,"y":1.3199725580185269},{"x":-0.40310949642735566,"y2":0,"group":1,"y":-1.0608510966365445},{"x":-0.8431859273029803,"y2":0,"group":1,"y":-0.353337032985411},{"x":-0.47257607526298623,"y2":0,"group":1,"y":-0.47597368279100766},{"x":-0.7068024664189811,"y2":0,"group":1,"y":-0.07376638222456988},{"x":1.0985430160691734,"y2":0,"group":1,"y":0.9495151550800726},{"x":-0.2744598690424316,"y2":0,"group":1,"y":-0.20385948126525022},{"x":0.42935791646600263,"y2":0,"group":1,"y":-0.6633913570834256},{"x":-0.25369836886250563,"y2":0,"group":1,"y":-1.179926938656976},{"x":1.185589174533815,"y2":0,"group":1,"y":1.3155558801682812},{"x":-0.38920834829957573,"y2":0,"group":1,"y":-0.6287554824634924},{"x":0.6433546120143125,"y2":0,"group":1,"y":0.16480809862701884},{"x":1.0664509070285786,"y2":0,"group":1,"y":1.0596229526517813},{"x":-0.29136514674895797,"y2":0,"group":1,"y":0.1685413922312397},{"x":1.8739589265933374,"y2":0,"group":1,"y":2.1904679141985324},{"x":1.547961641156074,"y2":0,"group":1,"y":1.8077103139615565},{"x":0.01921426330088001,"y2":0,"group":1,"y":0.2595182954803071},{"x":-0.14955840992720926,"y2":0,"group":1,"y":0.4089658514420349},{"x":0.6193544334890574,"y2":0,"group":1,"y":0.2513341422300505},{"x":0.42135994420053224,"y2":0,"group":1,"y":1.033748854692885},{"x":0.7074995545428782,"y2":0,"group":1,"y":0.07111154237794026},{"x":-0.8366857201892469,"y2":0,"group":1,"y":0.03025703376175748},{"x":-1.3524874602039278,"y2":0,"group":1,"y":-1.4273322373584603},{"x":0.106293714125376,"y2":0,"group":1,"y":0.2517777150560297},{"x":-0.427919690004838,"y2":0,"group":1,"y":-0.6989251570106179},{"x":-0.868808812935365,"y2":0,"group":1,"y":-0.9119732683146815},{"x":-0.010577581459119748,"y2":0,"group":1,"y":0.6226166544980168},{"x":-0.9607589783512616,"y2":0,"group":1,"y":-0.6583444795892798},{"x":0.9951884035023676,"y2":0,"group":1,"y":0.5994859174789646},{"x":2.163185564982633,"y2":0,"group":1,"y":2.097476006835532},{"x":-1.9963555739866456,"y2":0,"group":1,"y":-2.1869706920934204},{"x":1.7576422269261935,"y2":0,"group":1,"y":1.0833483677817428},{"x":-1.318289979772834,"y2":0,"group":1,"y":-0.5576178088704403},{"x":-0.08458009858753648,"y2":0,"group":1,"y":-0.014268855232085091},{"x":1.0833087070185592,"y2":0,"group":1,"y":1.3437086083235044},{"x":-0.09989262618906043,"y2":0,"group":1,"y":-0.18892149387393392},{"x":-0.21228614409092125,"y2":0,"group":1,"y":0.27378429348409195},{"x":0.20560606005827442,"y2":0,"group":1,"y":-0.5224927146502807},{"x":0.2508410685882439,"y2":0,"group":1,"y":0.016766387381061926},{"x":-1.1848105871773513,"y2":0,"group":1,"y":-0.44900344161442984},{"x":-0.4598856945952439,"y2":0,"group":1,"y":-0.3325414746004599},{"x":-1.529500376966329,"y2":0,"group":1,"y":-1.8053888557468338},{"x":1.5535281850642688,"y2":0,"group":1,"y":1.579180331535708},{"x":-0.15723383111301842,"y2":0,"group":1,"y":0.7222994688369413},{"x":-0.5616785701948857,"y2":0,"group":1,"y":-0.6311663002327789},{"x":-0.6252560822939206,"y2":0,"group":1,"y":-1.395079369364339},{"x":-0.5715151356485417,"y2":0,"group":1,"y":-0.27367212132770513},{"x":0.3830739250251319,"y2":0,"group":1,"y":0.33831418119700934},{"x":0.6069995830120937,"y2":0,"group":1,"y":-0.054987815724173994},{"x":-0.2707845131115933,"y2":0,"group":1,"y":-0.14607273520710334},{"x":1.0189108960959221,"y2":0,"group":1,"y":0.6437299786305328},{"x":-0.6286623644411963,"y2":0,"group":1,"y":-1.5878131871436032},{"x":-0.565387655251262,"y2":0,"group":1,"y":-0.1377370032574629},{"x":0.3255024320478121,"y2":0,"group":1,"y":-0.19630886099336153},{"x":1.209138999330144,"y2":0,"group":1,"y":1.5606090497630716},{"x":-2.407639349850734,"y2":0,"group":1,"y":-1.65807912067576},{"x":0.15536070312157657,"y2":0,"group":1,"y":0.5941705909027415},{"x":0.0061397313100841925,"y2":0,"group":1,"y":-0.3202502107932207},{"x":-1.9169545055741015,"y2":0,"group":1,"y":-1.3845799300998172},{"x":0.5864486507838146,"y2":0,"group":1,"y":0.5211717984154155},{"x":1.603883767963099,"y2":0,"group":1,"y":1.5541488645107957},{"x":1.018933013331811,"y2":0,"group":1,"y":0.36856584009421156},{"x":-2.304187038773564,"y2":0,"group":1,"y":-2.4119135325426604},{"x":0.49918988407301446,"y2":0,"group":1,"y":0.7290239685644169},{"x":-0.7506802336361874,"y2":0,"group":1,"y":-0.7489464326145739},{"x":0.9958883066416747,"y2":0,"group":1,"y":0.8174031818775394},{"x":-0.004522123816362057,"y2":0,"group":1,"y":0.6102894703842444},{"x":-0.8720016443357524,"y2":0,"group":1,"y":-0.8469985929219719},{"x":-0.6173362472894119,"y2":0,"group":1,"y":-0.5499542863070408},{"x":-0.5639697559236223,"y2":0,"group":1,"y":-0.9947635847074808},{"x":0.4686552071087219,"y2":0,"group":1,"y":0.11920555451753101},{"x":-1.4592792050704844,"y2":0,"group":1,"y":-1.2074355567810864},{"x":0.47932200941541026,"y2":0,"group":1,"y":0.47360861476264815},{"x":0.673190322142144,"y2":0,"group":1,"y":0.3940820324551849},{"x":0.46171884951498504,"y2":0,"group":1,"y":-0.3597780852269587},{"x":-0.9186128443501392,"y2":0,"group":1,"y":-0.8112958044795745},{"x":-0.8460007208546324,"y2":0,"group":1,"y":-0.803567040240877},{"x":-0.6221458310731804,"y2":0,"group":1,"y":-0.8643593043974016},{"x":0.40536935467917207,"y2":0,"group":1,"y":0.43051550813829226},{"x":1.0030406625053068,"y2":0,"group":1,"y":-0.24074795405476956},{"x":-0.43437165152906765,"y2":0,"group":1,"y":-0.46756702935972116},{"x":-0.16396679693898478,"y2":0,"group":1,"y":-0.31571804452470453},{"x":-0.3232622558976258,"y2":0,"group":1,"y":-0.7618696911592642},{"x":0.01000253687998966,"y2":0,"group":1,"y":-0.37583860864848395},{"x":-1.689895130125453,"y2":0,"group":1,"y":-1.0033569164343732},{"x":0.2971387976747972,"y2":0,"group":1,"y":0.12181219406785962},{"x":-0.5909036366996571,"y2":0,"group":1,"y":-0.4215343185620187},{"x":0.3611806367843093,"y2":0,"group":1,"y":0.8733862669077085},{"x":1.3946030319074252,"y2":0,"group":1,"y":1.3127885084154864},{"x":1.0496586258504714,"y2":0,"group":1,"y":1.8088163098416756},{"x":-0.8896414442003704,"y2":0,"group":1,"y":-1.056983706033035},{"x":0.6313889646381993,"y2":0,"group":1,"y":0.20860320656865688},{"x":0.8030460616043853,"y2":0,"group":1,"y":0.7712070749703305},{"x":-0.8594059601306504,"y2":0,"group":1,"y":-0.10348111880556965},{"x":-0.8457392488082475,"y2":0,"group":1,"y":-1.3511707872189025},{"x":0.06697380532300319,"y2":0,"group":1,"y":-0.7929640591329654},{"x":-0.5549300719569005,"y2":0,"group":1,"y":0.09691976860459683},{"x":-0.42984472516085864,"y2":0,"group":1,"y":0.8982486795581406},{"x":-1.1089996603607668,"y2":0,"group":1,"y":-1.2075303691582695},{"x":-0.2473035385922431,"y2":0,"group":1,"y":0.5089781134428286},{"x":0.9976424552054433,"y2":0,"group":1,"y":0.9539002284836018},{"x":-1.9424027741691026,"y2":0,"group":1,"y":-2.025934990139439},{"x":0.5554790912936795,"y2":0,"group":1,"y":0.7592058924176781},{"x":0.35017868112140743,"y2":0,"group":1,"y":0.6487230582956272},{"x":-1.0568316910282416,"y2":0,"group":1,"y":-0.6589765210601775},{"x":-0.9814491442649049,"y2":0,"group":1,"y":-1.083175685638102},{"x":0.8401933227256253,"y2":0,"group":1,"y":1.2219700469674923},{"x":0.976747988523205,"y2":0,"group":1,"y":0.8254423071264081},{"x":-0.6274867659868636,"y2":0,"group":1,"y":-1.3633452957476904},{"x":-0.17299344713630838,"y2":0,"group":1,"y":-0.07289671817226764},{"x":0.18160777969205982,"y2":0,"group":1,"y":-0.09358745762962148},{"x":0.14470622649992446,"y2":0,"group":1,"y":-0.22710040609971258},{"x":0.09378286932064145,"y2":0,"group":1,"y":-0.06272729102262299},{"x":-0.44887552376316947,"y2":0,"group":1,"y":-0.8791333007423883},{"x":-0.8084851029339722,"y2":0,"group":1,"y":-0.42946323328753644},{"x":-0.7608121617774012,"y2":0,"group":1,"y":-0.7614872604136458},{"x":-0.1236943541552662,"y2":0,"group":1,"y":0.17331641952515303},{"x":-0.2413087950888839,"y2":0,"group":1,"y":0.024880637217480878},{"x":-0.6999095805430399,"y2":0,"group":1,"y":-0.9038000000903148},{"x":2.3994513975245586,"y2":0,"group":1,"y":2.0913652808388123},{"x":0.4204101798913259,"y2":0,"group":1,"y":0.5109868886573947},{"x":0.6379996518795081,"y2":0,"group":1,"y":1.1891897458136098},{"x":-0.383868151422154,"y2":0,"group":1,"y":0.047448899077796425},{"x":0.13106308958641447,"y2":0,"group":1,"y":0.13820869325525412},{"x":-0.6671169477359883,"y2":0,"group":1,"y":-1.1126246683031136},{"x":-1.096354908954894,"y2":0,"group":1,"y":-0.3708290462902758},{"x":0.5349790629257657,"y2":0,"group":1,"y":0.5351758106231821},{"x":-0.5243457456760333,"y2":0,"group":1,"y":-0.8397662651095357},{"x":-1.2860517072795934,"y2":0,"group":1,"y":-1.6978926700955241},{"x":-0.7755514112412,"y2":0,"group":1,"y":-0.6629156370616861},{"x":1.0023749140887765,"y2":0,"group":1,"y":1.1789176901159},{"x":1.1170640271216696,"y2":0,"group":1,"y":0.8463762839563004},{"x":1.6225055823912415,"y2":0,"group":1,"y":1.2333778061859078},{"x":-0.9607763677431196,"y2":0,"group":1,"y":-0.8599654391091083},{"x":-0.6174620572293988,"y2":0,"group":1,"y":0.40382041585126716},{"x":0.9747307813789556,"y2":0,"group":1,"y":1.190693301717515},{"x":2.6987956782040263,"y2":0,"group":1,"y":2.38183300626265},{"x":-0.2507460250318298,"y2":0,"group":1,"y":-0.1859987990585459},{"x":-2.0621301549881665,"y2":0,"group":1,"y":-2.3573932276422265},{"x":-2.0564195437611676,"y2":0,"group":1,"y":-2.2788867939891033},{"x":-1.5237836188277636,"y2":0,"group":1,"y":-1.1396190352657465},{"x":1.237284412430426,"y2":0,"group":1,"y":1.3926624877625828},{"x":-0.28538717976810696,"y2":0,"group":1,"y":0.5747037952836559},{"x":0.8512081388288842,"y2":0,"group":1,"y":0.49740221995381695},{"x":-0.3630977951948654,"y2":0,"group":1,"y":-0.5693276555374962},{"x":-0.9531759121321604,"y2":0,"group":1,"y":-1.2812702706389838},{"x":1.261926587007966,"y2":0,"group":1,"y":0.9186558264726773},{"x":-0.8283692915830672,"y2":0,"group":1,"y":-0.9139241974964365},{"x":1.3812415185432365,"y2":0,"group":1,"y":1.004944901841335},{"x":0.54403290481004,"y2":0,"group":1,"y":0.8407547059001967},{"x":0.7533168676707179,"y2":0,"group":1,"y":-0.07533183637531649},{"x":0.3283783552294662,"y2":0,"group":1,"y":0.30085152792908826},{"x":0.3847334360147473,"y2":0,"group":1,"y":0.9138301324369184},{"x":0.3602542242949424,"y2":0,"group":1,"y":0.12256608199751479},{"x":0.24899139069542878,"y2":0,"group":1,"y":-0.2065445533725995},{"x":-0.14770184207207013,"y2":0,"group":1,"y":-0.45958272223702656},{"x":1.7988534375655578,"y2":0,"group":1,"y":1.6316487195802847},{"x":-0.07269979252343765,"y2":0,"group":1,"y":0.312991314839247},{"x":-0.5843747688826676,"y2":0,"group":1,"y":-0.5231105398092489},{"x":-0.37890657709704,"y2":0,"group":1,"y":-0.8365461564066368},{"x":0.2730654708181987,"y2":0,"group":1,"y":-0.017237699392495948},{"x":-2.4855047589861163,"y2":0,"group":1,"y":-2.6231144215193325},{"x":1.251326011749461,"y2":0,"group":1,"y":0.809008118041841},{"x":0.6826638836368242,"y2":0,"group":1,"y":0.5542581335778988},{"x":-0.4318494249123775,"y2":0,"group":1,"y":-0.3017707328395239},{"x":1.3030122472463581,"y2":0,"group":1,"y":0.3349651812226001},{"x":0.05492695376978474,"y2":0,"group":1,"y":-0.46031631159945186},{"x":-1.6928253178115504,"y2":0,"group":1,"y":-1.310465577836911},{"x":-0.3867205991503991,"y2":0,"group":1,"y":-0.17505932649013953},{"x":0.2885970979462741,"y2":0,"group":1,"y":0.49926255673847075},{"x":0.2617265372896306,"y2":0,"group":1,"y":0.17645372957339742},{"x":-0.8930297552503925,"y2":0,"group":1,"y":-1.1121157353714033},{"x":-0.5322171520263655,"y2":0,"group":1,"y":-0.3342400511657494},{"x":-0.36794836960276317,"y2":0,"group":1,"y":-0.12471749926429249},{"x":0.24588643052976256,"y2":0,"group":1,"y":0.26903112240287275},{"x":-0.43887240987201226,"y2":0,"group":1,"y":-0.5432318566623826},{"x":0.6785446273259035,"y2":0,"group":1,"y":0.41786669179124375},{"x":-0.49950773579699076,"y2":0,"group":1,"y":-0.9271539235938874},{"x":0.7025959695049913,"y2":0,"group":1,"y":0.1753147989106243},{"x":0.3091137562945079,"y2":0,"group":1,"y":0.5194810409387891},{"x":0.3536448266588141,"y2":0,"group":1,"y":0.7368451918781167},{"x":1.040267353920778,"y2":0,"group":1,"y":1.3125857569156254},{"x":-1.3205971103326752,"y2":0,"group":1,"y":-0.76195283657152},{"x":-0.004237034002175757,"y2":0,"group":1,"y":0.9006222338907308},{"x":-0.5816451911243227,"y2":0,"group":1,"y":-0.5939207325488},{"x":0.04154314738558499,"y2":0,"group":1,"y":-0.5705643077799009},{"x":-1.2623662656833412,"y2":0,"group":1,"y":-1.677101630616836},{"x":-0.2044002806157135,"y2":0,"group":1,"y":-0.683385589594361},{"x":1.5152115693310844,"y2":0,"group":1,"y":0.9080808173658146},{"x":-0.5453507875421753,"y2":0,"group":1,"y":-0.30387357438685103},{"x":-0.8317075693216338,"y2":0,"group":1,"y":-0.6125850254233826},{"x":-0.5142170214590056,"y2":0,"group":1,"y":-0.9621081345360293},{"x":2.667478342385033,"y2":0,"group":1,"y":2.583223528424184},{"x":1.1398847983837974,"y2":0,"group":1,"y":0.5706512497980738},{"x":-0.6220417475484582,"y2":0,"group":1,"y":-0.27833608520111663},{"x":-0.17699023183776683,"y2":0,"group":1,"y":-1.2913799510950934},{"x":-1.433941151295398,"y2":0,"group":1,"y":-1.3549102656945666},{"x":2.5430746818254573,"y2":0,"group":1,"y":2.0705305067481774},{"x":1.2593918726310216,"y2":0,"group":1,"y":0.2652634633644999},{"x":1.2317756307646073,"y2":0,"group":1,"y":1.3801375662440898},{"x":-0.7653662770692836,"y2":0,"group":1,"y":-0.6922621129949205},{"x":0.13521826819193816,"y2":0,"group":1,"y":0.6680908446051057},{"x":-1.2401801588571704,"y2":0,"group":1,"y":-0.5940925619977482},{"x":-0.2452435652730705,"y2":0,"group":1,"y":0.288302204422342},{"x":12.567982528884755,"y2":0,"group":2,"y":11.259806729507616},{"x":9.558837613909096,"y2":0,"group":2,"y":9.966127441218578},{"x":9.16493755314591,"y2":0,"group":2,"y":9.584968118516244},{"x":9.714795827986455,"y2":0,"group":2,"y":7.772348277066891},{"x":11.902185145870703,"y2":0,"group":2,"y":12.863286134639836},{"x":12.627930371879131,"y2":0,"group":2,"y":12.124409963698781},{"x":14.492472640414388,"y2":0,"group":2,"y":10.216194301941371},{"x":12.082102841031261,"y2":0,"group":2,"y":10.53549460940412},{"x":6.198323405957716,"y2":0,"group":2,"y":11.39073546536276},{"x":11.265169658274525,"y2":0,"group":2,"y":10.321680073481293},{"x":13.176532861549054,"y2":0,"group":2,"y":12.931521635463929},{"x":7.240035687609234,"y2":0,"group":2,"y":12.107809712215884},{"x":10.450458848228255,"y2":0,"group":2,"y":11.774390532723055},{"x":7.746448318960705,"y2":0,"group":2,"y":6.914422535696325},{"x":10.11665725530434,"y2":0,"group":2,"y":3.8037140753378056},{"x":5.691454205960061,"y2":0,"group":2,"y":10.641300033033914},{"x":9.21614469728824,"y2":0,"group":2,"y":11.186489933575135},{"x":6.167894244109046,"y2":0,"group":2,"y":12.139572274965879},{"x":8.523026576496484,"y2":0,"group":2,"y":8.112341042925362},{"x":11.992846469337666,"y2":0,"group":2,"y":9.65463300358621},{"x":4.539928585828674,"y2":0,"group":2,"y":8.798984981816037},{"x":8.65117483892952,"y2":0,"group":2,"y":8.930114075786427},{"x":13.866606438866405,"y2":0,"group":2,"y":10.028918702797805},{"x":9.494070698474198,"y2":0,"group":2,"y":7.439571224063313},{"x":12.561703887843183,"y2":0,"group":2,"y":9.657678026381669},{"x":8.760737213747145,"y2":0,"group":2,"y":11.262655595324151},{"x":12.407271171882146,"y2":0,"group":2,"y":8.36613991757231},{"x":12.09942895666859,"y2":0,"group":2,"y":11.336751949242515},{"x":10.682722664112074,"y2":0,"group":2,"y":6.568743268230341},{"x":7.484069761821429,"y2":0,"group":2,"y":9.609330769136463},{"x":7.178074915730024,"y2":0,"group":2,"y":5.58654134801327},{"x":9.283672462248648,"y2":0,"group":2,"y":10.201764017062295},{"x":12.753843034797724,"y2":0,"group":2,"y":7.981334374770913},{"x":7.502707974550856,"y2":0,"group":2,"y":11.544981234207235},{"x":12.333415955882693,"y2":0,"group":2,"y":14.003892599789495},{"x":8.396122769149136,"y2":0,"group":2,"y":7.392324715125962},{"x":7.797532597724905,"y2":0,"group":2,"y":12.595038652610828},{"x":12.671590483906007,"y2":0,"group":2,"y":5.9704102324292},{"x":9.396938963929887,"y2":0,"group":2,"y":11.182983605677899},{"x":13.192854544614455,"y2":0,"group":2,"y":11.350965199408183},{"x":6.090896992434676,"y2":0,"group":2,"y":5.092359959687193},{"x":7.63005247161964,"y2":0,"group":2,"y":12.129620925637727},{"x":9.132448851504998,"y2":0,"group":2,"y":8.247024974844862},{"x":7.262531106819576,"y2":0,"group":2,"y":9.380406770328063},{"x":13.820409915262818,"y2":0,"group":2,"y":11.693771203152005},{"x":8.979676692530976,"y2":0,"group":2,"y":11.999281481584335},{"x":10.453492851548138,"y2":0,"group":2,"y":12.670102404273347},{"x":6.7550030262495335,"y2":0,"group":2,"y":7.570389824781017},{"x":12.7835593109792,"y2":0,"group":2,"y":8.700872498075734},{"x":9.761761490840339,"y2":0,"group":2,"y":8.492925904620645},{"x":12.089909464133534,"y2":0,"group":2,"y":11.814292767888144},{"x":12.918972767610308,"y2":0,"group":2,"y":10.919130101077545},{"x":9.482073526276492,"y2":0,"group":2,"y":9.572576988156321},{"x":9.533591243981064,"y2":0,"group":2,"y":11.014905445071756},{"x":12.316469439288438,"y2":0,"group":2,"y":9.603818687205775},{"x":11.604138820426984,"y2":0,"group":2,"y":8.514457644215728},{"x":9.798226064754141,"y2":0,"group":2,"y":7.960938572637625},{"x":10.48885497947393,"y2":0,"group":2,"y":11.47752493936563},{"x":9.85294536548584,"y2":0,"group":2,"y":9.394745853462656},{"x":15.41886104811185,"y2":0,"group":2,"y":10.038091208770885},{"x":10.423043110270099,"y2":0,"group":2,"y":8.113097256415376},{"x":8.051077275284007,"y2":0,"group":2,"y":8.691556492656893},{"x":10.270623083160555,"y2":0,"group":2,"y":9.366240327570582},{"x":10.16509640229795,"y2":0,"group":2,"y":12.108721109247915},{"x":11.99789621332668,"y2":0,"group":2,"y":9.808377648813916},{"x":7.54929421438293,"y2":0,"group":2,"y":11.507576433375384},{"x":11.3827996542822,"y2":0,"group":2,"y":14.659294142878913},{"x":12.413698493186406,"y2":0,"group":2,"y":13.00273352942795},{"x":10.467951106400447,"y2":0,"group":2,"y":10.950501819005748},{"x":9.62869220511176,"y2":0,"group":2,"y":11.833342982192056},{"x":8.269899866997163,"y2":0,"group":2,"y":6.219141814882676},{"x":11.206488697371125,"y2":0,"group":2,"y":8.041198307753264},{"x":12.262566535784392,"y2":0,"group":2,"y":12.368524127991103},{"x":9.40520148065432,"y2":0,"group":2,"y":11.492825642994687},{"x":8.570813305256925,"y2":0,"group":2,"y":11.288772009265958},{"x":12.200931319289397,"y2":0,"group":2,"y":7.223114526855179},{"x":12.920142454757617,"y2":0,"group":2,"y":8.166817231753704},{"x":7.647056639499429,"y2":0,"group":2,"y":7.45154874609164},{"x":12.14639224922301,"y2":0,"group":2,"y":11.344969395326252},{"x":11.904944517850888,"y2":0,"group":2,"y":12.31213465227167},{"x":9.756787778759008,"y2":0,"group":2,"y":10.611692391362388},{"x":12.676381292032534,"y2":0,"group":2,"y":8.346507397057845},{"x":9.707536536250359,"y2":0,"group":2,"y":11.59534108943979},{"x":12.402031611648745,"y2":0,"group":2,"y":7.757209395882088},{"x":11.11027916876263,"y2":0,"group":2,"y":12.436169356953467},{"x":9.654469651619344,"y2":0,"group":2,"y":11.282325117341575},{"x":8.290404375384115,"y2":0,"group":2,"y":7.961025469972111},{"x":14.102466086101673,"y2":0,"group":2,"y":9.890976323716023},{"x":9.4695142011551,"y2":0,"group":2,"y":9.70885014548244},{"x":10.020279563911036,"y2":0,"group":2,"y":9.606183633303326},{"x":8.866299544799709,"y2":0,"group":2,"y":8.948540746439008},{"x":11.483234750205154,"y2":0,"group":2,"y":10.283756800397573},{"x":14.942447328531268,"y2":0,"group":2,"y":9.959405191039265},{"x":8.02085125319586,"y2":0,"group":2,"y":10.730736873689635},{"x":10.122024810736049,"y2":0,"group":2,"y":11.261216575441852},{"x":9.749285719597376,"y2":0,"group":2,"y":8.536809312983895},{"x":9.758066402738699,"y2":0,"group":2,"y":11.140146661325268},{"x":9.022666737782362,"y2":0,"group":2,"y":11.294123952997479},{"x":11.604541787755693,"y2":0,"group":2,"y":9.18596270329937},{"x":11.204589257104184,"y2":0,"group":2,"y":8.690953513644574},{"x":10.097697191767706,"y2":0,"group":2,"y":10.476941753431104},{"x":8.516744879848787,"y2":0,"group":2,"y":11.49013001754144},{"x":8.721577537082254,"y2":0,"group":2,"y":8.318719117089667},{"x":9.684729465822993,"y2":0,"group":2,"y":10.004770088800267},{"x":8.616422427427725,"y2":0,"group":2,"y":11.173679022145961},{"x":9.526146450855075,"y2":0,"group":2,"y":8.78811324017114},{"x":9.415645345549805,"y2":0,"group":2,"y":11.603361120068406},{"x":13.278493100696492,"y2":0,"group":2,"y":12.819200071055546},{"x":11.798050278784025,"y2":0,"group":2,"y":6.613889027130549},{"x":10.365279168206063,"y2":0,"group":2,"y":10.762972370737474},{"x":9.647001453325387,"y2":0,"group":2,"y":10.640685411700291},{"x":8.522931253050885,"y2":0,"group":2,"y":7.999376155113081},{"x":12.305452226671692,"y2":0,"group":2,"y":12.148847515363896},{"x":9.45187948738585,"y2":0,"group":2,"y":10.07498971780662},{"x":8.198765158563774,"y2":0,"group":2,"y":13.015249319293718},{"x":10.750795757547378,"y2":0,"group":2,"y":11.111226521363218},{"x":9.275130873590395,"y2":0,"group":2,"y":10.451821982090788},{"x":7.9581894307608785,"y2":0,"group":2,"y":11.515859389921886},{"x":12.846921865631385,"y2":0,"group":2,"y":12.89301427511103},{"x":9.151508538351276,"y2":0,"group":2,"y":7.176462703715501},{"x":10.041756373705212,"y2":0,"group":2,"y":9.8395020239026},{"x":11.582204876803056,"y2":0,"group":2,"y":11.19997638142877},{"x":8.425739412412694,"y2":0,"group":2,"y":8.934912009889695},{"x":12.416041264182855,"y2":0,"group":2,"y":9.917932925620693},{"x":8.471462809132861,"y2":0,"group":2,"y":9.614899535272427},{"x":10.64664745367625,"y2":0,"group":2,"y":11.2930085846136},{"x":14.261228682474789,"y2":0,"group":2,"y":14.360614364198618},{"x":9.194088828254083,"y2":0,"group":2,"y":12.941026570924127},{"x":11.872825614584466,"y2":0,"group":2,"y":9.358934200478615},{"x":9.945729743562518,"y2":0,"group":2,"y":7.884402045655124},{"x":9.19300576719681,"y2":0,"group":2,"y":10.403648607754704},{"x":9.897822303120021,"y2":0,"group":2,"y":10.931177417302132},{"x":9.132182792853545,"y2":0,"group":2,"y":11.086292315801291},{"x":12.11171914234772,"y2":0,"group":2,"y":8.43843394639742},{"x":12.11613435483569,"y2":0,"group":2,"y":6.162664235090794},{"x":8.557513940539003,"y2":0,"group":2,"y":9.235865779444008},{"x":8.41911004448361,"y2":0,"group":2,"y":10.910629318597627},{"x":12.397270819359274,"y2":0,"group":2,"y":9.602950980109553},{"x":7.627396384073735,"y2":0,"group":2,"y":7.318248311485464},{"x":8.456887925407655,"y2":0,"group":2,"y":8.85261442306047},{"x":11.579344872913618,"y2":0,"group":2,"y":10.969619564593408},{"x":11.328682123934538,"y2":0,"group":2,"y":8.061353067364738},{"x":13.485134804112885,"y2":0,"group":2,"y":5.619483184164487},{"x":8.18278772101433,"y2":0,"group":2,"y":11.270030790397476},{"x":12.6972401493606,"y2":0,"group":2,"y":12.573790611214442},{"x":7.9935973231888315,"y2":0,"group":2,"y":6.1624700889430075},{"x":7.929657341797293,"y2":0,"group":2,"y":10.006775511205264},{"x":14.22102732216014,"y2":0,"group":2,"y":12.413693844151418},{"x":9.609042125717261,"y2":0,"group":2,"y":9.670838887112716},{"x":8.92648728426921,"y2":0,"group":2,"y":7.523173066506629},{"x":10.056612933129678,"y2":0,"group":2,"y":8.865636409288205},{"x":8.311416596144793,"y2":0,"group":2,"y":9.034610606776374},{"x":7.824415468981219,"y2":0,"group":2,"y":7.093007943995323},{"x":8.4323100804861,"y2":0,"group":2,"y":9.077492866071594},{"x":7.25952620281171,"y2":0,"group":2,"y":9.140050670164339},{"x":5.277636539498409,"y2":0,"group":2,"y":9.27297183610186},{"x":8.945615551372125,"y2":0,"group":2,"y":10.06439841199471},{"x":8.869410264014808,"y2":0,"group":2,"y":10.589761705472876},{"x":7.3686523962101,"y2":0,"group":2,"y":9.709201431237803},{"x":14.097269589254351,"y2":0,"group":2,"y":9.095983885353938},{"x":10.559136268381865,"y2":0,"group":2,"y":11.827270341351442},{"x":14.062150032749631,"y2":0,"group":2,"y":10.722513022140946},{"x":9.663621884056099,"y2":0,"group":2,"y":7.964736681168738},{"x":12.277074975971503,"y2":0,"group":2,"y":7.306402060998762},{"x":8.811763066797607,"y2":0,"group":2,"y":11.505344377514833},{"x":9.46196811930837,"y2":0,"group":2,"y":9.281758533563295},{"x":10.583276468903902,"y2":0,"group":2,"y":11.231361315943738},{"x":8.07626991900894,"y2":0,"group":2,"y":9.075537419875152},{"x":13.064873808007047,"y2":0,"group":2,"y":10.03348488329632},{"x":10.887434878330735,"y2":0,"group":2,"y":8.0798434488962},{"x":8.206157089194386,"y2":0,"group":2,"y":8.906964136610956},{"x":9.474312778768745,"y2":0,"group":2,"y":8.502441004872885},{"x":10.018559677623438,"y2":0,"group":2,"y":9.620921279615613},{"x":10.709460764417319,"y2":0,"group":2,"y":11.147484272008729},{"x":10.56768460823817,"y2":0,"group":2,"y":12.06947297278763},{"x":9.371801226153558,"y2":0,"group":2,"y":11.139663991518095},{"x":11.051388488131515,"y2":0,"group":2,"y":10.877722274172518},{"x":7.088509808461701,"y2":0,"group":2,"y":7.150175937150017},{"x":9.601206102798525,"y2":0,"group":2,"y":9.974535263822554},{"x":9.139211611409525,"y2":0,"group":2,"y":10.404381113378204},{"x":7.808848801661074,"y2":0,"group":2,"y":12.228551832282976},{"x":10.105689009903932,"y2":0,"group":2,"y":8.164225964589576},{"x":11.167875496598342,"y2":0,"group":2,"y":5.964460247001401},{"x":11.369966288262665,"y2":0,"group":2,"y":9.700427645496232},{"x":10.368999467646056,"y2":0,"group":2,"y":11.197358904147686},{"x":7.199410663855123,"y2":0,"group":2,"y":11.190480610771317},{"x":7.196517115385102,"y2":0,"group":2,"y":7.846147061601225},{"x":6.935798151074582,"y2":0,"group":2,"y":8.679170506435675},{"x":7.950964881658557,"y2":0,"group":2,"y":11.768340897196564},{"x":8.100965832616545,"y2":0,"group":2,"y":8.160628877322939},{"x":8.21504050579258,"y2":0,"group":2,"y":11.552697939409247},{"x":12.13009592538382,"y2":0,"group":2,"y":4.6319695672993335},{"x":8.19016267191374,"y2":0,"group":2,"y":7.523657260339334},{"x":8.454348619139632,"y2":0,"group":2,"y":11.181169331552272},{"x":17.150373360650093,"y2":0,"group":2,"y":15.126036947615088},{"x":8.16845012771054,"y2":0,"group":2,"y":11.59636207534798},{"x":9.699907427268538,"y2":0,"group":2,"y":9.077176137857652},{"x":8.952243252918686,"y2":0,"group":2,"y":12.620829811258332},{"x":8.880645635291026,"y2":0,"group":2,"y":12.322680211168173},{"x":10.060183048901909,"y2":0,"group":2,"y":9.336169384516339},{"x":8.874980079068932,"y2":0,"group":2,"y":8.364302818535053},{"x":11.37403383041595,"y2":0,"group":2,"y":7.662530358475847},{"x":12.67923401448012,"y2":0,"group":2,"y":10.786612624092566},{"x":7.423622720372606,"y2":0,"group":2,"y":6.1625094848069235},{"x":9.142293298833556,"y2":0,"group":2,"y":7.359113026785105},{"x":11.796713109132194,"y2":0,"group":2,"y":10.885719294882453},{"x":9.228876092890271,"y2":0,"group":2,"y":8.028658353092002},{"x":11.157694847154591,"y2":0,"group":2,"y":7.47536642717778},{"x":8.674853206185073,"y2":0,"group":2,"y":11.304857563317288},{"x":10.12308427003257,"y2":0,"group":2,"y":9.85050042657454},{"x":10.019288597376802,"y2":0,"group":2,"y":10.152922694762944},{"x":8.878984402306132,"y2":0,"group":2,"y":8.951462619652405},{"x":9.357371660108326,"y2":0,"group":2,"y":9.431151401608037},{"x":7.74531696330007,"y2":0,"group":2,"y":8.933944628752949},{"x":10.583615643167237,"y2":0,"group":2,"y":11.095678882767208},{"x":6.710108632730988,"y2":0,"group":2,"y":11.47054958743022},{"x":6.744050051188539,"y2":0,"group":2,"y":9.428280084030984},{"x":9.802600645397455,"y2":0,"group":2,"y":8.593804188868166},{"x":5.777229631251143,"y2":0,"group":2,"y":12.820642056108218},{"x":9.74920934154041,"y2":0,"group":2,"y":11.324463671068397},{"x":8.763476206123565,"y2":0,"group":2,"y":8.171200604050421},{"x":7.289023921486955,"y2":0,"group":2,"y":10.665616450795618},{"x":4.323755573465341,"y2":0,"group":2,"y":10.235699316724183},{"x":7.524462618299276,"y2":0,"group":2,"y":12.005977879862225},{"x":13.334101429002631,"y2":0,"group":2,"y":12.64535592764129},{"x":13.575582014434746,"y2":0,"group":2,"y":11.685830491622117},{"x":11.35304377180999,"y2":0,"group":2,"y":8.71250366113882},{"x":8.513211032271517,"y2":0,"group":2,"y":9.785720029425685},{"x":11.836139668641488,"y2":0,"group":2,"y":8.577656631807127},{"x":11.204555709356066,"y2":0,"group":2,"y":7.879669854945858},{"x":8.877461908234439,"y2":0,"group":2,"y":6.7736814496407565},{"x":5.778854154658547,"y2":0,"group":2,"y":10.576895361714437},{"x":6.8789577561367485,"y2":0,"group":2,"y":10.699792319155845},{"x":9.397918889308661,"y2":0,"group":2,"y":11.394283513669807},{"x":14.075179525175383,"y2":0,"group":2,"y":10.675616839565759},{"x":7.955699190967019,"y2":0,"group":2,"y":8.576930577723488},{"x":12.251890855282955,"y2":0,"group":2,"y":8.191205253001403},{"x":9.386229858240416,"y2":0,"group":2,"y":10.186920758937147},{"x":8.06920550905748,"y2":0,"group":2,"y":9.393648553331918},{"x":10.827927726301791,"y2":0,"group":2,"y":9.520108614863387},{"x":10.49727571269202,"y2":0,"group":2,"y":12.103688662612738},{"x":10.465613315130032,"y2":0,"group":2,"y":12.635358192779456},{"x":10.754470081817306,"y2":0,"group":2,"y":7.245038649002398},{"x":10.475072458426876,"y2":0,"group":2,"y":8.112788679765092},{"x":10.515375915854046,"y2":0,"group":2,"y":9.485008200245245},{"x":6.37750396142091,"y2":0,"group":2,"y":7.5778315540125964},{"x":12.169792158209631,"y2":0,"group":2,"y":7.605312667930901},{"x":11.439608194953356,"y2":0,"group":2,"y":9.571701319086472},{"x":11.302112209432142,"y2":0,"group":2,"y":11.05984684891876},{"x":6.24465745267163,"y2":0,"group":2,"y":10.567745306362449},{"x":10.410433729944799,"y2":0,"group":2,"y":11.589893761219791},{"x":4.990763420816055,"y2":0,"group":2,"y":9.306135785410172},{"x":12.656476555709508,"y2":0,"group":2,"y":7.7807147851241485},{"x":13.17576667490453,"y2":0,"group":2,"y":10.211042003492897},{"x":12.536888352607683,"y2":0,"group":2,"y":5.185694664566822},{"x":11.007971382564284,"y2":0,"group":2,"y":8.423414202374286},{"x":10.808109596571635,"y2":0,"group":2,"y":13.968255362092947},{"x":12.400019925540269,"y2":0,"group":2,"y":7.319519738229749},{"x":12.624274633424664,"y2":0,"group":2,"y":9.410628956375172},{"x":11.40878561591058,"y2":0,"group":2,"y":9.220436699856158},{"x":9.70871912463968,"y2":0,"group":2,"y":8.149316573118448},{"x":11.851556399387128,"y2":0,"group":2,"y":10.402486413441085},{"x":9.53132728375537,"y2":0,"group":2,"y":7.091603449829687},{"x":9.381813911493794,"y2":0,"group":2,"y":11.013268084848693},{"x":10.222760969079927,"y2":0,"group":2,"y":10.695966418342609},{"x":10.73948917663458,"y2":0,"group":2,"y":10.647077950284402},{"x":7.159771747679388,"y2":0,"group":2,"y":8.84570068579757},{"x":8.645537505791886,"y2":0,"group":2,"y":11.15210703814395},{"x":8.594599687422438,"y2":0,"group":2,"y":9.659162180051057},{"x":8.9569543692811,"y2":0,"group":2,"y":6.35460084915314},{"x":7.2380710302705635,"y2":0,"group":2,"y":11.217214139530082},{"x":10.320573846725898,"y2":0,"group":2,"y":8.284690129757223},{"x":5.006511679721071,"y2":0,"group":2,"y":6.518503354254622},{"x":8.711731011453992,"y2":0,"group":2,"y":9.169507547296275},{"x":13.398768748941485,"y2":0,"group":2,"y":11.65666942415059},{"x":7.7959814044557705,"y2":0,"group":2,"y":7.224942411887561},{"x":6.701284357838185,"y2":0,"group":2,"y":8.942251069140859},{"x":11.290315113876813,"y2":0,"group":2,"y":8.95099678254282},{"x":8.62712412117434,"y2":0,"group":2,"y":5.769884140093999},{"x":10.982389985161763,"y2":0,"group":2,"y":13.488036446995421},{"x":10.362771560168781,"y2":0,"group":2,"y":12.362526689499528},{"x":8.539550671765314,"y2":0,"group":2,"y":5.1417673008838385},{"x":9.928863571924127,"y2":0,"group":2,"y":8.415002564270246},{"x":9.694407791933896,"y2":0,"group":2,"y":12.451276392423724},{"x":11.532250608408791,"y2":0,"group":2,"y":8.757350624021974},{"x":10.664193243898747,"y2":0,"group":2,"y":11.147737729174194},{"x":9.506741162203113,"y2":0,"group":2,"y":11.08619921078704},{"x":9.788698727932289,"y2":0,"group":2,"y":7.264352092547269},{"x":10.663330216957752,"y2":0,"group":2,"y":11.295687127269646},{"x":4.8164358247987344,"y2":0,"group":2,"y":10.240053642693763},{"x":10.98849241273648,"y2":0,"group":2,"y":10.74386108695398},{"x":13.264475301425168,"y2":0,"group":2,"y":11.470905874836383},{"x":14.99661987642778,"y2":0,"group":2,"y":11.668117940975483},{"x":11.944890705492666,"y2":0,"group":2,"y":8.323120020081832},{"x":11.683611947179775,"y2":0,"group":2,"y":7.935877904653326},{"x":10.89443810501655,"y2":0,"group":2,"y":12.54150292202097},{"x":9.027247232311511,"y2":0,"group":2,"y":13.876518009024045},{"x":9.305113924943415,"y2":0,"group":2,"y":12.259117220171756},{"x":9.025273411587328,"y2":0,"group":2,"y":10.648818909175027},{"x":10.999292084290227,"y2":0,"group":2,"y":10.376810928763799},{"x":11.723193581028525,"y2":0,"group":2,"y":8.379136364006225},{"x":12.10143361253332,"y2":0,"group":2,"y":11.156734201730796},{"x":11.451628115781245,"y2":0,"group":2,"y":9.434265387939483},{"x":8.09529350421791,"y2":0,"group":2,"y":8.124581850968625},{"x":9.917365993109192,"y2":0,"group":2,"y":8.936130595097477},{"x":9.374047815477146,"y2":0,"group":2,"y":9.298190342738478},{"x":9.984145108417335,"y2":0,"group":2,"y":8.45709646108046},{"x":8.91918433005399,"y2":0,"group":2,"y":10.737380717251101},{"x":13.089674109353428,"y2":0,"group":2,"y":8.200870229980978},{"x":12.146198330295835,"y2":0,"group":2,"y":9.707819816136144},{"x":9.883295948532393,"y2":0,"group":2,"y":6.869249908630483},{"x":9.483544675628172,"y2":0,"group":2,"y":10.850052121781568},{"x":9.907586816063795,"y2":0,"group":2,"y":10.841919510650674},{"x":9.357299816663925,"y2":0,"group":2,"y":10.635800816983613},{"x":8.961018376684336,"y2":0,"group":2,"y":9.148731664139483},{"x":10.778841099423872,"y2":0,"group":2,"y":7.771550965130951},{"x":11.590075460632763,"y2":0,"group":2,"y":6.639884460093876},{"x":8.177859251043593,"y2":0,"group":2,"y":5.286751853529044},{"x":6.076195196045311,"y2":0,"group":2,"y":10.986312154909438},{"x":9.300851687614678,"y2":0,"group":2,"y":7.224361977434887},{"x":11.362091568378506,"y2":0,"group":2,"y":6.7312579975263445},{"x":11.695212308145425,"y2":0,"group":2,"y":9.801105143509746},{"x":12.164062526861114,"y2":0,"group":2,"y":9.879623098710002},{"x":4.837271474996942,"y2":0,"group":2,"y":8.48715341192315},{"x":11.28090432951089,"y2":0,"group":2,"y":10.715235145126707},{"x":9.59605209109403,"y2":0,"group":2,"y":7.764719322539964},{"x":10.032214208885915,"y2":0,"group":2,"y":10.108660388273297},{"x":7.151185642752398,"y2":0,"group":2,"y":10.79050121777921},{"x":7.830015790446171,"y2":0,"group":2,"y":10.024465905994731},{"x":8.666120308334122,"y2":0,"group":2,"y":10.776341525479593},{"x":7.980715154330591,"y2":0,"group":2,"y":9.829313588271145},{"x":12.284753736355956,"y2":0,"group":2,"y":8.323770112518709},{"x":8.079117873396408,"y2":0,"group":2,"y":13.028676709072734},{"x":9.710889132995357,"y2":0,"group":2,"y":10.232542978609928},{"x":14.61948672188145,"y2":0,"group":2,"y":9.650937784600224},{"x":12.466089303077558,"y2":0,"group":2,"y":11.1762762400509},{"x":8.104748340088767,"y2":0,"group":2,"y":10.612868116874209},{"x":8.35285426615497,"y2":0,"group":2,"y":10.843168626044351},{"x":11.128651548932858,"y2":0,"group":2,"y":10.335236781143996},{"x":10.58242377029514,"y2":0,"group":2,"y":9.836812916290846},{"x":7.498307012339627,"y2":0,"group":2,"y":12.66851943294618},{"x":10.322350309933235,"y2":0,"group":2,"y":7.09918887084423},{"x":13.573420675805924,"y2":0,"group":2,"y":10.207844558535163},{"x":10.173008692943988,"y2":0,"group":2,"y":11.578133601295354},{"x":6.42416854920862,"y2":0,"group":2,"y":9.756334753599782},{"x":9.735933632268127,"y2":0,"group":2,"y":8.534543504797107},{"x":10.210043604246346,"y2":0,"group":2,"y":12.275614064890643},{"x":9.801774454037098,"y2":0,"group":2,"y":12.922219260498132},{"x":8.895307085157887,"y2":0,"group":2,"y":9.268047149634409},{"x":8.263644736205649,"y2":0,"group":2,"y":7.715513820179683},{"x":8.044419442773476,"y2":0,"group":2,"y":8.229939687696088},{"x":7.716725570670268,"y2":0,"group":2,"y":10.584079501765718},{"x":9.747581735758226,"y2":0,"group":2,"y":9.802743740338222},{"x":11.051603281156412,"y2":0,"group":2,"y":11.365205481680086},{"x":10.900584099915717,"y2":0,"group":2,"y":8.341654240684488},{"x":11.805569107997282,"y2":0,"group":2,"y":11.608266527714934},{"x":13.19449136982748,"y2":0,"group":2,"y":9.962840810025261},{"x":9.219764486599267,"y2":0,"group":2,"y":9.71860756184702},{"x":7.038251315657041,"y2":0,"group":2,"y":8.819057256984825},{"x":7.075877929814889,"y2":0,"group":2,"y":10.51568726461552},{"x":6.120801332907114,"y2":0,"group":2,"y":12.257286129966408},{"x":10.989775731492584,"y2":0,"group":2,"y":13.204339621465282},{"x":7.257622077761505,"y2":0,"group":2,"y":7.9371711274379395},{"x":10.44011401178841,"y2":0,"group":2,"y":11.876620387680504},{"x":12.45480053644408,"y2":0,"group":2,"y":7.983259932762697},{"x":6.091122969486063,"y2":0,"group":2,"y":11.742080270920049},{"x":10.89867104399394,"y2":0,"group":2,"y":12.156728340384234},{"x":10.15670360783506,"y2":0,"group":2,"y":11.580086574734},{"x":9.49854284308009,"y2":0,"group":2,"y":11.465594653765757},{"x":11.321176929727438,"y2":0,"group":2,"y":9.17985109815986},{"x":8.190522622048178,"y2":0,"group":2,"y":9.942608757600588},{"x":7.380016696544928,"y2":0,"group":2,"y":9.45557242482508},{"x":7.907820065004726,"y2":0,"group":2,"y":8.01274861506549},{"x":8.266388680992078,"y2":0,"group":2,"y":9.280834185488178},{"x":9.67015536643224,"y2":0,"group":2,"y":7.601892970065066},{"x":10.076270720755044,"y2":0,"group":2,"y":11.22906277722995},{"x":10.475565582726691,"y2":0,"group":2,"y":7.045202731822711},{"x":11.726781703375883,"y2":0,"group":2,"y":9.746173752690822},{"x":12.082737097072846,"y2":0,"group":2,"y":9.16880379867454},{"x":7.00579809407559,"y2":0,"group":2,"y":12.122742044389001},{"x":11.46976988404827,"y2":0,"group":2,"y":9.617745415019687},{"x":11.525712962016797,"y2":0,"group":2,"y":8.58790508178444},{"x":11.623915060882117,"y2":0,"group":2,"y":10.49808000813507},{"x":10.910880346318358,"y2":0,"group":2,"y":9.96330899055117},{"x":9.88515988263269,"y2":0,"group":2,"y":10.167553781162189},{"x":8.268917704654218,"y2":0,"group":2,"y":11.842001437183226},{"x":9.009970845302439,"y2":0,"group":2,"y":8.971461580383707},{"x":9.830202700871814,"y2":0,"group":2,"y":10.435712150404939},{"x":10.671874321330664,"y2":0,"group":2,"y":9.16160663688501},{"x":10.439819098041186,"y2":0,"group":2,"y":6.721799207533361},{"x":8.824849103207589,"y2":0,"group":2,"y":7.682502889142748},{"x":7.801960987820805,"y2":0,"group":2,"y":8.581295752005222},{"x":10.291613483170005,"y2":0,"group":2,"y":10.69638673514656},{"x":10.809212603317158,"y2":0,"group":2,"y":13.6091422992967},{"x":11.64298749045429,"y2":0,"group":2,"y":13.202934363791776},{"x":10.637605548300838,"y2":0,"group":2,"y":7.266538085599765},{"x":11.623916119006786,"y2":0,"group":2,"y":10.218439941430766},{"x":11.000310067243316,"y2":0,"group":2,"y":11.654924292503162},{"x":11.828233001661765,"y2":0,"group":2,"y":9.543576255371152},{"x":6.562640897691444,"y2":0,"group":2,"y":11.557797296510268},{"x":13.375528814408074,"y2":0,"group":2,"y":7.640785664739948},{"x":8.591973518210992,"y2":0,"group":2,"y":12.38977067241806},{"x":8.70504771784905,"y2":0,"group":2,"y":7.406466225918085},{"x":12.591203389523516,"y2":0,"group":2,"y":10.771738109569608},{"x":9.73678988631471,"y2":0,"group":2,"y":9.009618678340031},{"x":12.524243986715742,"y2":0,"group":2,"y":9.772475781066884},{"x":7.063805719604673,"y2":0,"group":2,"y":6.671452894844183},{"x":9.470191993057592,"y2":0,"group":2,"y":7.973497832220939},{"x":11.505694257679963,"y2":0,"group":2,"y":12.696015145760331},{"x":10.148051582687941,"y2":0,"group":2,"y":9.187267360683169},{"x":10.004720059904018,"y2":0,"group":2,"y":9.28173982641246},{"x":13.249444250736339,"y2":0,"group":2,"y":13.172880309967546},{"x":9.020296706100225,"y2":0,"group":2,"y":10.334016382857484},{"x":13.4443520137651,"y2":0,"group":2,"y":14.039497417650928},{"x":10.076118866659264,"y2":0,"group":2,"y":12.060005932303946},{"x":10.14609435626815,"y2":0,"group":2,"y":7.4192789943711155},{"x":10.592912711315677,"y2":0,"group":2,"y":10.81112837587957},{"x":10.935247632167725,"y2":0,"group":2,"y":9.158625436905858},{"x":7.876612182365346,"y2":0,"group":2,"y":9.391096022209469},{"x":7.837546102448034,"y2":0,"group":2,"y":11.697632308642554},{"x":12.376317863741333,"y2":0,"group":2,"y":12.335020593105149},{"x":7.698078656402304,"y2":0,"group":2,"y":11.11240711808078},{"x":12.277827116206463,"y2":0,"group":2,"y":7.220070686707189},{"x":11.896054873648554,"y2":0,"group":2,"y":11.685657834797208},{"x":8.608679279507085,"y2":0,"group":2,"y":10.5189853357055},{"x":10.27264026149118,"y2":0,"group":2,"y":10.809689609857859},{"x":6.136086987280659,"y2":0,"group":2,"y":10.256606421199464},{"x":10.971250767702172,"y2":0,"group":2,"y":10.377316049157285},{"x":10.789271559022533,"y2":0,"group":2,"y":11.590826428801268},{"x":10.440589453261312,"y2":0,"group":2,"y":8.391595171387491},{"x":10.179437610637564,"y2":0,"group":2,"y":11.84406454700951},{"x":13.736571968559975,"y2":0,"group":2,"y":9.473970849827701},{"x":7.649788800637406,"y2":0,"group":2,"y":10.575388481216539},{"x":11.603359550946266,"y2":0,"group":2,"y":8.208586369494505},{"x":7.220429495660339,"y2":0,"group":2,"y":9.873047564823533},{"x":10.860663765971994,"y2":0,"group":2,"y":14.607935138596735},{"x":10.53951752980134,"y2":0,"group":2,"y":11.65846678515175},{"x":11.698201263750148,"y2":0,"group":2,"y":9.851407311862344},{"x":13.530039971998924,"y2":0,"group":2,"y":7.93809350537878},{"x":10.274430301353089,"y2":0,"group":2,"y":13.495197813664333},{"x":9.907460768797979,"y2":0,"group":2,"y":8.77798104913764},{"x":8.052061373761012,"y2":0,"group":2,"y":12.521081198268867},{"x":8.780642360218264,"y2":0,"group":2,"y":11.415125347714364},{"x":9.62655690680504,"y2":0,"group":2,"y":6.986887861040477},{"x":8.603266375045704,"y2":0,"group":2,"y":9.369337308434854},{"x":7.250869415436392,"y2":0,"group":2,"y":8.293788346879374},{"x":6.927421663363873,"y2":0,"group":2,"y":10.07132638207779},{"x":10.608655399379817,"y2":0,"group":2,"y":9.59155969720007},{"x":10.097662278576506,"y2":0,"group":2,"y":11.153994098863373},{"x":8.538351214264184,"y2":0,"group":2,"y":10.293028065928759},{"x":11.295437239362826,"y2":0,"group":2,"y":5.913031399527121},{"x":9.246410770751773,"y2":0,"group":2,"y":9.330448330120152},{"x":12.16927220910308,"y2":0,"group":2,"y":10.45902485607195},{"x":10.587617470520316,"y2":0,"group":2,"y":7.368573948206264},{"x":12.793190632953948,"y2":0,"group":2,"y":8.686441963533694},{"x":8.405216925578864,"y2":0,"group":2,"y":9.028843637362769},{"x":8.701438827744541,"y2":0,"group":2,"y":7.103402503115783},{"x":11.023970627708895,"y2":0,"group":2,"y":12.103426890871628},{"x":7.291298137785729,"y2":0,"group":2,"y":9.044945829953816},{"x":9.992486686286114,"y2":0,"group":2,"y":10.426666367654486},{"x":12.27059541711776,"y2":0,"group":2,"y":13.069031852872484},{"x":9.07991289557589,"y2":0,"group":2,"y":8.96341638307924},{"x":10.141588597737396,"y2":0,"group":2,"y":8.153788177311947},{"x":10.428228776345211,"y2":0,"group":2,"y":9.954993536101119},{"x":11.826854828468393,"y2":0,"group":2,"y":8.50119763198247},{"x":11.123313313791947,"y2":0,"group":2,"y":12.537044681071636},{"x":12.19570031668492,"y2":0,"group":2,"y":11.802384598415198},{"x":12.354069267659904,"y2":0,"group":2,"y":11.315852560320877},{"x":10.400545405411469,"y2":0,"group":2,"y":7.4669432650386796},{"x":8.754797503392929,"y2":0,"group":2,"y":12.992357136782832},{"x":8.802004864959407,"y2":0,"group":2,"y":8.21860843274932},{"x":12.39601701936552,"y2":0,"group":2,"y":11.292094935444474},{"x":4.4486707868706015,"y2":0,"group":2,"y":10.362420267973679},{"x":8.939791105132366,"y2":0,"group":2,"y":10.86561920386125},{"x":8.864822553053534,"y2":0,"group":2,"y":9.077803697421697},{"x":10.392576523671993,"y2":0,"group":2,"y":10.229717658684397},{"x":13.299952716071386,"y2":0,"group":2,"y":9.739991096085681},{"x":9.281685374176515,"y2":0,"group":2,"y":11.507032593703327},{"x":7.682901313175039,"y2":0,"group":2,"y":8.59866896439994},{"x":8.62537852927741,"y2":0,"group":2,"y":9.098309307764483},{"x":9.753550993544804,"y2":0,"group":2,"y":8.456608650220101},{"x":7.677162132197728,"y2":0,"group":2,"y":8.869546330918558},{"x":13.900273665732751,"y2":0,"group":2,"y":9.557893723653319},{"x":10.85577183766057,"y2":0,"group":2,"y":10.19021038627685},{"x":10.598989644781618,"y2":0,"group":2,"y":10.105589330019573},{"x":12.019030880082486,"y2":0,"group":2,"y":8.592855473797878},{"x":10.268130033070685,"y2":0,"group":2,"y":13.091766243142327},{"x":9.464427786555188,"y2":0,"group":2,"y":9.542358661044679},{"x":11.053722522361388,"y2":0,"group":2,"y":11.119305370804033},{"x":10.336445988081797,"y2":0,"group":2,"y":7.284080231116049},{"x":9.460145729880988,"y2":0,"group":2,"y":8.267136340609746},{"x":11.593638886923221,"y2":0,"group":2,"y":9.515560254392987},{"x":10.369610958665534,"y2":0,"group":2,"y":13.801675447810762},{"x":10.462412915168576,"y2":0,"group":2,"y":10.774029077142865},{"x":10.902955120831242,"y2":0,"group":2,"y":11.521815973210227},{"x":10.240703105461009,"y2":0,"group":2,"y":11.658282265128415},{"x":14.489433114247564,"y2":0,"group":2,"y":7.953360050666465},{"x":9.34076030979578,"y2":0,"group":2,"y":9.903003839387301},{"x":10.978655270772846,"y2":0,"group":2,"y":13.276747899230356},{"x":11.865284403573002,"y2":0,"group":2,"y":9.242006895925455}]}],"scales":[{"name":"x","range":"width","domain":{"data":"table","field":"x"},"type":"linear"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

);
</script>
\endhtmlonly


# Area Plot


### Area Plot - Regular


\htmlonly
<div id="area"></div>
<script type="text/javascript">
plot("area",
	{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"properties":{"enter":{"interpolate":{"value":"monotone"},"x":{"field":"x","scale":"x"},"y2":{"field":"y2","scale":"y"},"fill":{"field":"group","scale":"group"},"y":{"field":"y","scale":"y"}}},"from":{"data":"table"},"type":"area"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":-0.0429278945154396,"y2":0,"group":1,"y":0.0003784595866943574},{"x":-0.042285982213274856,"y2":0,"group":1,"y":0.0003813186190221307},{"x":-0.04164406991111011,"y2":0,"group":1,"y":0.00038441218330032},{"x":-0.041002157608945364,"y2":0,"group":1,"y":0.0003877419166323781},{"x":-0.04036024530678061,"y2":0,"group":1,"y":0.0003913095632702701},{"x":-0.039718333004615865,"y2":0,"group":1,"y":0.00039511697521432687},{"x":-0.03907642070245112,"y2":0,"group":1,"y":0.00039916611284385173},{"x":-0.03843450840028637,"y2":0,"group":1,"y":0.00040345904557259615},{"x":-0.03779259609812163,"y2":0,"group":1,"y":0.00040799795253620985},{"x":-0.03715068379595688,"y2":0,"group":1,"y":0.0004127851233068913},{"x":-0.03650877149379213,"y2":0,"group":1,"y":0.00041782295863568297},{"x":-0.03586685919162738,"y2":0,"group":1,"y":0.00042311397122629657},{"x":-0.035224946889462636,"y2":0,"group":1,"y":0.00042866078653325257},{"x":-0.03458303458729789,"y2":0,"group":1,"y":0.0004344661435951025},{"x":-0.03394112228513314,"y2":0,"group":1,"y":0.0004405328958900778},{"x":-0.0332992099829684,"y2":0,"group":1,"y":0.00044686401222493455},{"x":-0.03265729768080365,"y2":0,"group":1,"y":0.0004534625776509982},{"x":-0.0320153853786389,"y2":0,"group":1,"y":0.00046033179441096195},{"x":-0.03137347307647415,"y2":0,"group":1,"y":0.0004674749829119973},{"x":-0.030731560774309406,"y2":0,"group":1,"y":0.00047489558272983956},{"x":-0.03008964847214466,"y2":0,"group":1,"y":0.0004825971536436269},{"x":-0.029447736169979914,"y2":0,"group":1,"y":0.0004905833766968293},{"x":-0.028805823867815165,"y2":0,"group":1,"y":0.0004988580552884869},{"x":-0.02816391156565042,"y2":0,"group":1,"y":0.0005074251162964227},{"x":-0.02752199926348567,"y2":0,"group":1,"y":0.0005162886112251019},{"x":-0.026880086961320923,"y2":0,"group":1,"y":0.0005254527173879087},{"x":-0.026238174659156177,"y2":0,"group":1,"y":0.0005349217391150685},{"x":-0.02559626235699143,"y2":0,"group":1,"y":0.0005447001089932124},{"x":-0.02495435005482668,"y2":0,"group":1,"y":0.0005547923891345841},{"x":-0.024312437752661935,"y2":0,"group":1,"y":0.0005652032724736689},{"x":-0.02367052545049719,"y2":0,"group":1,"y":0.0005759375840963532},{"x":-0.02302861314833244,"y2":0,"group":1,"y":0.0005870002825979492},{"x":-0.022386700846167694,"y2":0,"group":1,"y":0.0005983964614674903},{"x":-0.021744788544002948,"y2":0,"group":1,"y":0.0006101313505094269},{"x":-0.0211028762418382,"y2":0,"group":1,"y":0.0006222103172858484},{"x":-0.020460963939673452,"y2":0,"group":1,"y":0.0006346388685953414},{"x":-0.019819051637508706,"y2":0,"group":1,"y":0.0006474226519799509},{"x":-0.019177139335343957,"y2":0,"group":1,"y":0.0006605674572597586},{"x":-0.01853522703317921,"y2":0,"group":1,"y":0.0006740792180997823},{"x":-0.017893314731014465,"y2":0,"group":1,"y":0.0006879640136055326},{"x":-0.01725140242884972,"y2":0,"group":1,"y":0.0007022280699489764},{"x":-0.01660949012668497,"y2":0,"group":1,"y":0.0007168777620251005},{"x":-0.015967577824520223,"y2":0,"group":1,"y":0.0007319196151362312},{"x":-0.015325665522355477,"y2":0,"group":1,"y":0.000747360306707287},{"x":-0.014683753220190728,"y2":0,"group":1,"y":0.0007632066680318822},{"x":-0.014041840918025981,"y2":0,"group":1,"y":0.0007794656860480309},{"x":-0.013399928615861235,"y2":0,"group":1,"y":0.0007961445051419269},{"x":-0.012758016313696486,"y2":0,"group":1,"y":0.0008132504289830311},{"x":-0.01211610401153174,"y2":0,"group":1,"y":0.0008307909223896071},{"x":-0.01147419170936699,"y2":0,"group":1,"y":0.0008487736132222201},{"x":-0.010832279407202244,"y2":0,"group":1,"y":0.0008672062943091002},{"x":-0.010190367105037498,"y2":0,"group":1,"y":0.0008860969253986217},{"x":-0.009548454802872752,"y2":0,"group":1,"y":0.0009054536351450204},{"x":-0.008906542500708006,"y2":0,"group":1,"y":0.0009252847231197014},{"x":-0.00826463019854326,"y2":0,"group":1,"y":0.0009455986618548962},{"x":-0.007622717896378507,"y2":0,"group":1,"y":0.0009664040989177808},{"x":-0.006980805594213761,"y2":0,"group":1,"y":0.0009877098590105743},{"x":-0.006338893292049015,"y2":0,"group":1,"y":0.0010095249461020833},{"x":-0.005696980989884269,"y2":0,"group":1,"y":0.0010318585455900836},{"x":-0.005055068687719523,"y2":0,"group":1,"y":0.001054720026489972},{"x":-0.004413156385554777,"y2":0,"group":1,"y":0.0010781189436552124},{"x":-0.003771244083390024,"y2":0,"group":1,"y":0.001102065040025052},{"x":-0.003129331781225278,"y2":0,"group":1,"y":0.0011265682489031148},{"x":-0.002487419479060532,"y2":0,"group":1,"y":0.001151638696262891},{"x":-0.001845507176895786,"y2":0,"group":1,"y":0.0011772867030838685},{"x":-0.0012035948747310399,"y2":0,"group":1,"y":0.0012035227877153498},{"x":-0.0005616825725662938,"y2":0,"group":1,"y":0.0012303576682695383},{"x":8.02297295984522e-5,"y2":0,"group":1,"y":0.0012578022650417964},{"x":0.0007221420317632052,"y2":0,"group":1,"y":0.0012858677029627674},{"x":0.0013640543339279512,"y2":0,"group":1,"y":0.0013145653140734237},{"x":0.0020059666360926973,"y2":0,"group":1,"y":0.001343906640032172},{"x":0.0026478789382574433,"y2":0,"group":1,"y":0.0013739034346490486},{"x":0.0032897912404221893,"y2":0,"group":1,"y":0.0014045676664475593},{"x":0.003931703542586935,"y2":0,"group":1,"y":0.0014359115212537188},{"x":0.004573615844751688,"y2":0,"group":1,"y":0.001467947404812514},{"x":0.005215528146916434,"y2":0,"group":1,"y":0.0015006879454324829},{"x":0.0058574404490811804,"y2":0,"group":1,"y":0.0015341459966556337},{"x":0.0064993527512459265,"y2":0,"group":1,"y":0.001568334639957536},{"x":0.0071412650534106725,"y2":0,"group":1,"y":0.0016032671874697535},{"x":0.0077831773555754186,"y2":0,"group":1,"y":0.0016389571847324202},{"x":0.008425089657740165,"y2":0,"group":1,"y":0.001675418413470603},{"x":0.009067001959904918,"y2":0,"group":1,"y":0.0017126648943977252},{"x":0.009708914262069664,"y2":0,"group":1,"y":0.001750710890044499},{"x":0.01035082656423441,"y2":0,"group":1,"y":0.0017895709076141408},{"x":0.010992738866399156,"y2":0,"group":1,"y":0.0018292597018609857},{"x":0.011634651168563902,"y2":0,"group":1,"y":0.001869792277995025},{"x":0.012276563470728648,"y2":0,"group":1,"y":0.0019111838946115345},{"x":0.0129184757728934,"y2":0,"group":1,"y":0.00195345006664685},{"x":0.013560388075058147,"y2":0,"group":1,"y":0.0019966065683504064},{"x":0.014202300377222893,"y2":0,"group":1,"y":0.0020406694362933298},{"x":0.014844212679387639,"y2":0,"group":1,"y":0.0020856549723866635},{"x":0.015486124981552385,"y2":0,"group":1,"y":0.002131579746933232},{"x":0.01612803728371713,"y2":0,"group":1,"y":0.0021784606016983232},{"x":0.016769949585881884,"y2":0,"group":1,"y":0.0022263146530001887},{"x":0.01741186188804663,"y2":0,"group":1,"y":0.0022751592948272725},{"x":0.018053774190211376,"y2":0,"group":1,"y":0.002325012201973342},{"x":0.018695686492376122,"y2":0,"group":1,"y":0.0023758913331968223},{"x":0.019337598794540868,"y2":0,"group":1,"y":0.0024278149343984756},{"x":0.01997951109670562,"y2":0,"group":1,"y":0.0024808015418205376},{"x":0.020621423398870367,"y2":0,"group":1,"y":0.002534869985265642},{"x":0.021263335701035113,"y2":0,"group":1,"y":0.002590039391336285},{"x":0.02190524800319986,"y2":0,"group":1,"y":0.002646329186691748},{"x":0.022547160305364605,"y2":0,"group":1,"y":0.002703759101324421},{"x":0.02318907260752935,"y2":0,"group":1,"y":0.0027623491718538895},{"x":0.023830984909694097,"y2":0,"group":1,"y":0.002822119744840035},{"x":0.024472897211858843,"y2":0,"group":1,"y":0.0028830914801107888},{"x":0.02511480951402359,"y2":0,"group":1,"y":0.0029452853541096224},{"x":0.025756721816188335,"y2":0,"group":1,"y":0.0030087226632553588},{"x":0.02639863411835308,"y2":0,"group":1,"y":0.003073425027320442},{"x":0.02704054642051784,"y2":0,"group":1,"y":0.0031394143928245855},{"x":0.027682458722682587,"y2":0,"group":1,"y":0.003206713036439157},{"x":0.028324371024847333,"y2":0,"group":1,"y":0.0032753435684087506},{"x":0.02896628332701208,"y2":0,"group":1,"y":0.003345328935985936},{"x":0.029608195629176826,"y2":0,"group":1,"y":0.0034166924268750898},{"x":0.03025010793134157,"y2":0,"group":1,"y":0.003489457672693208},{"x":0.030892020233506318,"y2":0,"group":1,"y":0.0035636486524380506},{"x":0.031533932535671064,"y2":0,"group":1,"y":0.0036392896959676624},{"x":0.03217584483783581,"y2":0,"group":1,"y":0.003716405487492358},{"x":0.032817757140000556,"y2":0,"group":1,"y":0.0037950210690721187},{"x":0.0334596694421653,"y2":0,"group":1,"y":0.0038751618441265345},{"x":0.03410158174433005,"y2":0,"group":1,"y":0.003956853580948189},{"x":0.034743494046494794,"y2":0,"group":1,"y":0.004040122416227365},{"x":0.035385406348659554,"y2":0,"group":1,"y":0.004124994858581005},{"x":0.0360273186508243,"y2":0,"group":1,"y":0.004211497792086266},{"x":0.036669230952989046,"y2":0,"group":1,"y":0.004299658479820945},{"x":0.03731114325515379,"y2":0,"group":1,"y":0.004389504567407443},{"x":0.03795305555731854,"y2":0,"group":1,"y":0.0044810640865575735},{"x":0.038594967859483284,"y2":0,"group":1,"y":0.004574365458624219},{"x":0.03923688016164803,"y2":0,"group":1,"y":0.00466943749814866},{"x":0.039878792463812776,"y2":0,"group":1,"y":0.004766309416414566},{"x":0.04052070476597752,"y2":0,"group":1,"y":0.004865010824996352},{"x":0.04116261706814227,"y2":0,"group":1,"y":0.004965571739309921},{"x":0.041804529370307014,"y2":0,"group":1,"y":0.005068022582160686},{"x":0.04244644167247176,"y2":0,"group":1,"y":0.005172394187286183},{"x":0.043088353974636506,"y2":0,"group":1,"y":0.005278717802898264},{"x":0.043730266276801266,"y2":0,"group":1,"y":0.005387025095217823},{"x":0.04437217857896601,"y2":0,"group":1,"y":0.005497348152003284},{"x":0.04501409088113076,"y2":0,"group":1,"y":0.005609719486075665},{"x":0.045656003183295504,"y2":0,"group":1,"y":0.005724172038830522},{"x":0.04629791548546025,"y2":0,"group":1,"y":0.00584073918374578},{"x":0.046939827787624996,"y2":0,"group":1,"y":0.005959454729874664},{"x":0.04758174008978974,"y2":0,"group":1,"y":0.006080352925333626},{"x":0.04822365239195449,"y2":0,"group":1,"y":0.006203468460771244},{"x":0.048865564694119235,"y2":0,"group":1,"y":0.006328836472828836},{"x":0.04950747699628398,"y2":0,"group":1,"y":0.0064564925475840695},{"x":0.05014938929844873,"y2":0,"group":1,"y":0.00658647272398033},{"x":0.05079130160061347,"y2":0,"group":1,"y":0.006718813497239268},{"x":0.05143321390277822,"y2":0,"group":1,"y":0.0068535518222534175},{"x":0.05207512620494298,"y2":0,"group":1,"y":0.0069907251169616935},{"x":0.052717038507107725,"y2":0,"group":1,"y":0.007130371265706398},{"x":0.05335895080927247,"y2":0,"group":1,"y":0.007272528622562252},{"x":0.05400086311143722,"y2":0,"group":1,"y":0.007417236014650219},{"x":0.05464277541360196,"y2":0,"group":1,"y":0.00756453274542182},{"x":0.05528468771576671,"y2":0,"group":1,"y":0.007714458597920998},{"x":0.055926600017931455,"y2":0,"group":1,"y":0.007867053838018223},{"x":0.0565685123200962,"y2":0,"group":1,"y":0.008022359217616348},{"x":0.05721042462226095,"y2":0,"group":1,"y":0.008180415977828315},{"x":0.05785233692442569,"y2":0,"group":1,"y":0.008341265852122526},{"x":0.05849424922659044,"y2":0,"group":1,"y":0.008504951069439048},{"x":0.059136161528755185,"y2":0,"group":1,"y":0.008671514357268456},{"x":0.05977807383091993,"y2":0,"group":1,"y":0.008840998944699086},{"x":0.06041998613308469,"y2":0,"group":1,"y":0.00901344856542774},{"x":0.06106189843524944,"y2":0,"group":1,"y":0.009188907460730478},{"x":0.06170381073741418,"y2":0,"group":1,"y":0.009367420382397074},{"x":0.06234572303957893,"y2":0,"group":1,"y":0.009549032595623452},{"x":0.06298763534174368,"y2":0,"group":1,"y":0.009733789881862243},{"x":0.06362954764390842,"y2":0,"group":1,"y":0.009921738541629754},{"x":0.06427145994607317,"y2":0,"group":1,"y":0.010112925397267936},{"x":0.06491337224823791,"y2":0,"group":1,"y":0.010307397795661255},{"x":0.06555528455040266,"y2":0,"group":1,"y":0.01050520361090257},{"x":0.0661971968525674,"y2":0,"group":1,"y":0.010706391246910807},{"x":0.06683910915473215,"y2":0,"group":1,"y":0.010911009639998415},{"x":0.0674810214568969,"y2":0,"group":1,"y":0.011119108261383903},{"x":0.06812293375906166,"y2":0,"group":1,"y":0.011330737119651824},{"x":0.0687648460612264,"y2":0,"group":1,"y":0.011545946763154957},{"x":0.06940675836339115,"y2":0,"group":1,"y":0.011764788282360494},{"x":0.0700486706655559,"y2":0,"group":1,"y":0.011987313312134318},{"x":0.07069058296772064,"y2":0,"group":1,"y":0.012213574033967889},{"x":0.07133249526988539,"y2":0,"group":1,"y":0.012443623178138796},{"x":0.07197440757205013,"y2":0,"group":1,"y":0.01267751402580869},{"x":0.07261631987421488,"y2":0,"group":1,"y":0.01291530041105532},{"x":0.07325823217637963,"y2":0,"group":1,"y":0.013157036722836246},{"x":0.07390014447854437,"y2":0,"group":1,"y":0.013402777906881908},{"x":0.07454205678070912,"y2":0,"group":1,"y":0.013652579467519799},{"x":0.07518396908287386,"y2":0,"group":1,"y":0.01390649746942288},{"x":0.07582588138503861,"y2":0,"group":1,"y":0.01416458853928576},{"x":0.07646779368720337,"y2":0,"group":1,"y":0.0144269098674214},{"x":0.07710970598936812,"y2":0,"group":1,"y":0.014693519209281886},{"x":0.07775161829153286,"y2":0,"group":1,"y":0.014964474886897589},{"x":0.07839353059369761,"y2":0,"group":1,"y":0.015239835790234718},{"x":0.07903544289586235,"y2":0,"group":1,"y":0.015519661378468885},{"x":0.0796773551980271,"y2":0,"group":1,"y":0.01580401168117304},{"x":0.08031926750019185,"y2":0,"group":1,"y":0.016092947299416904},{"x":0.08096117980235659,"y2":0,"group":1,"y":0.01638652940678037},{"x":0.08160309210452134,"y2":0,"group":1,"y":0.016684819750269142},{"x":0.08224500440668608,"y2":0,"group":1,"y":0.016987880651145648},{"x":0.08288691670885084,"y2":0,"group":1,"y":0.017295775005657266},{"x":0.08352882901101558,"y2":0,"group":1,"y":0.0176085662856722},{"x":0.08417074131318034,"y2":0,"group":1,"y":0.017926318539215058},{"x":0.08481265361534507,"y2":0,"group":1,"y":0.018249096390901942},{"x":0.08545456591750983,"y2":0,"group":1,"y":0.018576965042272575},{"x":0.08609647821967456,"y2":0,"group":1,"y":0.01890999027201906},{"x":0.08673839052183932,"y2":0,"group":1,"y":0.0192482384361069},{"x":0.08738030282400405,"y2":0,"group":1,"y":0.019591776467787037},{"x":0.08802221512616881,"y2":0,"group":1,"y":0.01994067187750062},{"x":0.08866412742833357,"y2":0,"group":1,"y":0.02029499275266916},{"x":0.0893060397304983,"y2":0,"group":1,"y":0.02065480775737008},{"x":0.08994795203266306,"y2":0,"group":1,"y":0.021020186131897683},{"x":0.0905898643348278,"y2":0,"group":1,"y":0.021391197692206765},{"x":0.09123177663699256,"y2":0,"group":1,"y":0.021767912829232017},{"x":0.09187368893915729,"y2":0,"group":1,"y":0.022150402508093275},{"x":0.09251560124132205,"y2":0,"group":1,"y":0.022538738267167513},{"x":0.09315751354348678,"y2":0,"group":1,"y":0.022932992217042952},{"x":0.09379942584565154,"y2":0,"group":1,"y":0.023333237039341037},{"x":0.09444133814781627,"y2":0,"group":1,"y":0.02373954598541081},{"x":0.09508325044998103,"y2":0,"group":1,"y":0.024151992874890005},{"x":0.09572516275214576,"y2":0,"group":1,"y":0.024570652094134365},{"x":0.09636707505431052,"y2":0,"group":1,"y":0.024995598594511254},{"x":0.09700898735647528,"y2":0,"group":1,"y":0.025426907890554185},{"x":0.09765089965864002,"y2":0,"group":1,"y":0.025864656057979718},{"x":0.09829281196080478,"y2":0,"group":1,"y":0.026308919731563435},{"x":0.09893472426296951,"y2":0,"group":1,"y":0.02675977610287178},{"x":0.09957663656513427,"y2":0,"group":1,"y":0.02721730291784827},{"x":0.100218548867299,"y2":0,"group":1,"y":0.027681578474255508},{"x":0.10086046116946376,"y2":0,"group":1,"y":0.028152681618965614},{"x":0.1015023734716285,"y2":0,"group":1,"y":0.02863069174510066},{"x":0.10214428577379325,"y2":0,"group":1,"y":0.02911568878902257},{"x":0.10278619807595799,"y2":0,"group":1,"y":0.029607753227165245},{"x":0.10342811037812275,"y2":0,"group":1,"y":0.030106966072713348},{"x":0.10407002268028748,"y2":0,"group":1,"y":0.030613408872122383},{"x":0.10471193498245224,"y2":0,"group":1,"y":0.03112716370147675},{"x":0.105353847284617,"y2":0,"group":1,"y":0.031648313162689734},{"x":0.10599575958678173,"y2":0,"group":1,"y":0.03217694037953617},{"x":0.10663767188894649,"y2":0,"group":1,"y":0.03271312899352388},{"x":0.10727958419111122,"y2":0,"group":1,"y":0.03325696315959409},{"x":0.10792149649327598,"y2":0,"group":1,"y":0.03380852754165625},{"x":0.10856340879544071,"y2":0,"group":1,"y":0.03436790730794956},{"x":0.10920532109760547,"y2":0,"group":1,"y":0.034935188126236055},{"x":0.1098472333997702,"y2":0,"group":1,"y":0.03551045615881421},{"x":0.11048914570193497,"y2":0,"group":1,"y":0.03609379805736432},{"x":0.1111310580040997,"y2":0,"group":1,"y":0.036685300957608497},{"x":0.11177297030626446,"y2":0,"group":1,"y":0.03728505247379818},{"x":0.11241488260842919,"y2":0,"group":1,"y":0.037893140693019106},{"x":0.11305679491059395,"y2":0,"group":1,"y":0.03850965416931254},{"x":0.11369870721275871,"y2":0,"group":1,"y":0.039134681917615405},{"x":0.11434061951492344,"y2":0,"group":1,"y":0.039768313407513106},{"x":0.1149825318170882,"y2":0,"group":1,"y":0.040410638556808515},{"x":0.11562444411925293,"y2":0,"group":1,"y":0.04106174772489926},{"x":0.1162663564214177,"y2":0,"group":1,"y":0.04172173170596816},{"x":0.11690826872358243,"y2":0,"group":1,"y":0.04239068172198188},{"x":0.11755018102574719,"y2":0,"group":1,"y":0.04306868941549594},{"x":0.11819209332791192,"y2":0,"group":1,"y":0.043755846842268104},{"x":0.11883400563007668,"y2":0,"group":1,"y":0.044452246463674794},{"x":0.11947591793224141,"y2":0,"group":1,"y":0.0451579811389321},{"x":0.12011783023440617,"y2":0,"group":1,"y":0.04587314411711907},{"x":0.1207597425365709,"y2":0,"group":1,"y":0.04659782902899978},{"x":0.12140165483873566,"y2":0,"group":1,"y":0.04733212987865082},{"x":0.12204356714090042,"y2":0,"group":1,"y":0.048076141034880476},{"x":0.12268547944306515,"y2":0,"group":1,"y":0.04882995722245076},{"x":0.12332739174522991,"y2":0,"group":1,"y":0.049593673513093295},{"x":0.12396930404739465,"y2":0,"group":1,"y":0.05036738531632154},{"x":0.1246112163495594,"y2":0,"group":1,"y":0.0511511883700371},{"x":0.12525312865172414,"y2":0,"group":1,"y":0.05194517873093005},{"x":0.1258950409538889,"y2":0,"group":1,"y":0.052749452764670385},{"x":0.12653695325605363,"y2":0,"group":1,"y":0.053564107135892686},{"x":0.1271788655582184,"y2":0,"group":1,"y":0.05438923879797053},{"x":0.12782077786038312,"y2":0,"group":1,"y":0.055224944982580904},{"x":0.12846269016254788,"y2":0,"group":1,"y":0.05607132318905886},{"x":0.12910460246471261,"y2":0,"group":1,"y":0.05692847117353772},{"x":0.12974651476687737,"y2":0,"group":1,"y":0.05779648693788278},{"x":0.13038842706904213,"y2":0,"group":1,"y":0.05867546871840479},{"x":0.13103033937120687,"y2":0,"group":1,"y":0.059565514974365796},{"x":0.13167225167337163,"y2":0,"group":1,"y":0.060466724376269376},{"x":0.13231416397553636,"y2":0,"group":1,"y":0.061379195793935},{"x":0.13295607627770112,"y2":0,"group":1,"y":0.06230302828436057},{"x":0.13359798857986585,"y2":0,"group":1,"y":0.06323832107936592},{"x":0.1342399008820306,"y2":0,"group":1,"y":0.0641851735730248},{"x":0.13488181318419534,"y2":0,"group":1,"y":0.06514368530887876},{"x":0.1355237254863601,"y2":0,"group":1,"y":0.06611395596693567},{"x":0.13616563778852483,"y2":0,"group":1,"y":0.0670960853504527},{"x":0.1368075500906896,"y2":0,"group":1,"y":0.06809017337250067},{"x":0.13744946239285433,"y2":0,"group":1,"y":0.06909632004231686},{"x":0.1380913746950191,"y2":0,"group":1,"y":0.0701146254514364},{"x":0.13873328699718385,"y2":0,"group":1,"y":0.07114518975961232},{"x":0.13937519929934858,"y2":0,"group":1,"y":0.07218811318051532},{"x":0.14001711160151334,"y2":0,"group":1,"y":0.07324349596721982},{"x":0.14065902390367807,"y2":0,"group":1,"y":0.07431143839747775},{"x":0.14130093620584283,"y2":0,"group":1,"y":0.07539204075876896},{"x":0.14194284850800756,"y2":0,"group":1,"y":0.07648540333314499},{"x":0.14258476081017232,"y2":0,"group":1,"y":0.07759162638185532},{"x":0.14322667311233706,"y2":0,"group":1,"y":0.07871081012975795},{"x":0.14386858541450182,"y2":0,"group":1,"y":0.07984305474952055},{"x":0.14451049771666655,"y2":0,"group":1,"y":0.0809884603456058},{"x":0.1451524100188313,"y2":0,"group":1,"y":0.08214712693804663},{"x":0.14579432232099604,"y2":0,"group":1,"y":0.08331915444601098},{"x":0.1464362346231608,"y2":0,"group":1,"y":0.08450464267115279},{"x":0.14707814692532556,"y2":0,"group":1,"y":0.08570369128075864},{"x":0.1477200592274903,"y2":0,"group":1,"y":0.08691639979068255},{"x":0.14836197152965505,"y2":0,"group":1,"y":0.08814286754807515},{"x":0.14900388383181978,"y2":0,"group":1,"y":0.08938319371390599},{"x":0.14964579613398454,"y2":0,"group":1,"y":0.09063747724528096},{"x":0.15028770843614928,"y2":0,"group":1,"y":0.0919058168775581},{"x":0.15092962073831404,"y2":0,"group":1,"y":0.09318831110625656},{"x":0.15157153304047877,"y2":0,"group":1,"y":0.09448505816877105},{"x":0.15221344534264353,"y2":0,"group":1,"y":0.09579615602588004},{"x":0.15285535764480826,"y2":0,"group":1,"y":0.09712170234306328},{"x":0.15349726994697302,"y2":0,"group":1,"y":0.0984617944716172},{"x":0.15413918224913775,"y2":0,"group":1,"y":0.09981652942957962},{"x":0.1547810945513025,"y2":0,"group":1,"y":0.10118600388246085},{"x":0.15542300685346727,"y2":0,"group":1,"y":0.10257031412378233},{"x":0.156064919155632,"y2":0,"group":1,"y":0.10396955605543162},{"x":0.15670683145779676,"y2":0,"group":1,"y":0.105383825167824},{"x":0.1573487437599615,"y2":0,"group":1,"y":0.10681321651988424},{"x":0.15799065606212626,"y2":0,"group":1,"y":0.10825782471884557},{"x":0.158632568364291,"y2":0,"group":1,"y":0.10971774389986616},{"x":0.15927448066645575,"y2":0,"group":1,"y":0.11119306770546963},{"x":0.15991639296862048,"y2":0,"group":1,"y":0.11268388926481084},{"x":0.16055830527078524,"y2":0,"group":1,"y":0.11419030117276727},{"x":0.16120021757294997,"y2":0,"group":1,"y":0.11571239546886176},{"x":0.16184212987511473,"y2":0,"group":1,"y":0.11725026361601563},{"x":0.16248404217727946,"y2":0,"group":1,"y":0.11880399647913942},{"x":0.16312595447944422,"y2":0,"group":1,"y":0.12037368430356096},{"x":0.16376786678160898,"y2":0,"group":1,"y":0.12195941669329273},{"x":0.16440977908377372,"y2":0,"group":1,"y":0.12356128258914723},{"x":0.16505169138593848,"y2":0,"group":1,"y":0.12517937024669326},{"x":0.1656936036881032,"y2":0,"group":1,"y":0.1268137672140694},{"x":0.16633551599026797,"y2":0,"group":1,"y":0.12846456030964576},{"x":0.1669774282924327,"y2":0,"group":1,"y":0.1301318355995452},{"x":0.16761934059459746,"y2":0,"group":1,"y":0.1318156783750235},{"x":0.1682612528967622,"y2":0,"group":1,"y":0.13351617312971462},{"x":0.16890316519892695,"y2":0,"group":1,"y":0.13523340353674196},{"x":0.16954507750109168,"y2":0,"group":1,"y":0.13696745242570077},{"x":0.17018698980325644,"y2":0,"group":1,"y":0.13871840175951594},{"x":0.1708289021054212,"y2":0,"group":1,"y":0.14048633261117827},{"x":0.17147081440758594,"y2":0,"group":1,"y":0.1422713251403623},{"x":0.1721127267097507,"y2":0,"group":1,"y":0.1440734585699322},{"x":0.17275463901191543,"y2":0,"group":1,"y":0.14589281116233538},{"x":0.1733965513140802,"y2":0,"group":1,"y":0.14772946019589367},{"x":0.17403846361624492,"y2":0,"group":1,"y":0.1495834819409922},{"x":0.17468037591840968,"y2":0,"group":1,"y":0.15145495163617084},{"x":0.1753222882205744,"y2":0,"group":1,"y":0.15334394346412447},{"x":0.17596420052273917,"y2":0,"group":1,"y":0.15525053052761284},{"x":0.1766061128249039,"y2":0,"group":1,"y":0.15717478482529115},{"x":0.17724802512706866,"y2":0,"group":1,"y":0.15911677722745876},{"x":0.1778899374292334,"y2":0,"group":1,"y":0.16107657745173262},{"x":0.17853184973139816,"y2":0,"group":1,"y":0.1630542540386565},{"x":0.17917376203356292,"y2":0,"group":1,"y":0.1650498743272397},{"x":0.17981567433572765,"y2":0,"group":1,"y":0.16706350443044074},{"x":0.1804575866378924,"y2":0,"group":1,"y":0.16909520921059537},{"x":0.18109949894005714,"y2":0,"group":1,"y":0.17114505225479365},{"x":0.1817414112422219,"y2":0,"group":1,"y":0.17321309585021502},{"x":0.18238332354438663,"y2":0,"group":1,"y":0.17529940095942326},{"x":0.1830252358465514,"y2":0,"group":1,"y":0.17740402719562773},{"x":0.18366714814871613,"y2":0,"group":1,"y":0.17952703279791715},{"x":0.18430906045088088,"y2":0,"group":1,"y":0.18166847460646973},{"x":0.18495097275304562,"y2":0,"group":1,"y":0.1838284080377473},{"x":0.18559288505521038,"y2":0,"group":1,"y":0.18600688705967605},{"x":0.1862347973573751,"y2":0,"group":1,"y":0.18820396416682256},{"x":0.18687670965953987,"y2":0,"group":1,"y":0.19041969035556955},{"x":0.18751862196170463,"y2":0,"group":1,"y":0.19265411509929753},{"x":0.18816053426386936,"y2":0,"group":1,"y":0.19490728632357618},{"x":0.18880244656603412,"y2":0,"group":1,"y":0.1971792503813768},{"x":0.18944435886819885,"y2":0,"group":1,"y":0.19947005202830267},{"x":0.1900862711703636,"y2":0,"group":1,"y":0.2017797343978554},{"x":0.19072818347252835,"y2":0,"group":1,"y":0.20410833897673253},{"x":0.1913700957746931,"y2":0,"group":1,"y":0.20645590558016735},{"x":0.19201200807685784,"y2":0,"group":1,"y":0.20882247232732032},{"x":0.1926539203790226,"y2":0,"group":1,"y":0.2112080756167199},{"x":0.19329583268118733,"y2":0,"group":1,"y":0.2136127501017694},{"x":0.1939377449833521,"y2":0,"group":1,"y":0.21603652866631617},{"x":0.19457965728551682,"y2":0,"group":1,"y":0.2184794424002995},{"x":0.19522156958768158,"y2":0,"group":1,"y":0.2209415205754749},{"x":0.19586348188984634,"y2":0,"group":1,"y":0.22342279062122708},{"x":0.19650539419201107,"y2":0,"group":1,"y":0.22592327810047866},{"x":0.19714730649417583,"y2":0,"group":1,"y":0.2284430066856929},{"x":0.19778921879634057,"y2":0,"group":1,"y":0.23098199813499237},{"x":0.19843113109850533,"y2":0,"group":1,"y":0.23354027226838292},{"x":0.19907304340067006,"y2":0,"group":1,"y":0.23611784694410498},{"x":0.19971495570283482,"y2":0,"group":1,"y":0.23871473803510831},{"x":0.20035686800499955,"y2":0,"group":1,"y":0.24133095940566326},{"x":0.2009987803071643,"y2":0,"group":1,"y":0.24396652288811393},{"x":0.20164069260932904,"y2":0,"group":1,"y":0.24662143825977795},{"x":0.2022826049114938,"y2":0,"group":1,"y":0.24929571322000435},{"x":0.20292451721365853,"y2":0,"group":1,"y":0.25198935336739126},{"x":0.2035664295158233,"y2":0,"group":1,"y":0.25470236217717723},{"x":0.20420834181798805,"y2":0,"group":1,"y":0.25743474097880575},{"x":0.20485025412015279,"y2":0,"group":1,"y":0.26018648893367624},{"x":0.20549216642231755,"y2":0,"group":1,"y":0.26295760301308524},{"x":0.20613407872448228,"y2":0,"group":1,"y":0.26574807797636824},{"x":0.20677599102664704,"y2":0,"group":1,"y":0.26855790634924454},{"x":0.20741790332881177,"y2":0,"group":1,"y":0.2713870784023772},{"x":0.20805981563097653,"y2":0,"group":1,"y":0.27423558213015287},{"x":0.2087017279331413,"y2":0,"group":1,"y":0.2771034032296906},{"x":0.209343640235306,"y2":0,"group":1,"y":0.27999052508008304},{"x":0.20998555253747075,"y2":0,"group":1,"y":0.2828969287218831},{"x":0.21062746483963551,"y2":0,"group":1,"y":0.28582259283683953},{"x":0.21126937714180027,"y2":0,"group":1,"y":0.2887674937278893},{"x":0.21191128944396503,"y2":0,"group":1,"y":0.291731605299415},{"x":0.21255320174612974,"y2":0,"group":1,"y":0.2947148990377735},{"x":0.2131951140482945,"y2":0,"group":1,"y":0.2977173439921065},{"x":0.21383702635045926,"y2":0,"group":1,"y":0.30073890675543524},{"x":0.21447893865262402,"y2":0,"group":1,"y":0.30377955144605057},{"x":0.21512085095478872,"y2":0,"group":1,"y":0.30683923968920535},{"x":0.21576276325695348,"y2":0,"group":1,"y":0.309917930599115},{"x":0.21640467555911824,"y2":0,"group":1,"y":0.3130155807612745},{"x":0.217046587861283,"y2":0,"group":1,"y":0.31613214421510016},{"x":0.2176885001634477,"y2":0,"group":1,"y":0.3192675724369017},{"x":0.21833041246561247,"y2":0,"group":1,"y":0.32242181432319333},{"x":0.21897232476777723,"y2":0,"group":1,"y":0.3255948161743504},{"x":0.219614237069942,"y2":0,"group":1,"y":0.32878652167862005},{"x":0.22025614937210675,"y2":0,"group":1,"y":0.33199687189649174},{"x":0.22089806167427145,"y2":0,"group":1,"y":0.33522580524543505},{"x":0.2215399739764362,"y2":0,"group":1,"y":0.3384732574850141},{"x":0.22218188627860097,"y2":0,"group":1,"y":0.3417391617023824},{"x":0.22282379858076573,"y2":0,"group":1,"y":0.34502344829816955},{"x":0.22346571088293044,"y2":0,"group":1,"y":0.3483260449727613},{"x":0.2241076231850952,"y2":0,"group":1,"y":0.35164687671298683},{"x":0.22474953548725995,"y2":0,"group":1,"y":0.3549858657792137},{"x":0.22539144778942471,"y2":0,"group":1,"y":0.35834293169286335},{"x":0.22603336009158942,"y2":0,"group":1,"y":0.3617179912243496},{"x":0.22667527239375418,"y2":0,"group":1,"y":0.36511095838144947},{"x":0.22731718469591894,"y2":0,"group":1,"y":0.368521744398114},{"x":0.2279590969980837,"y2":0,"group":1,"y":0.37195025772372187},{"x":0.22860100930024846,"y2":0,"group":1,"y":0.3753964040127884},{"x":0.22924292160241316,"y2":0,"group":1,"y":0.378860086115131},{"x":0.22988483390457792,"y2":0,"group":1,"y":0.3823412040665033},{"x":0.23052674620674268,"y2":0,"group":1,"y":0.3858396550796984},{"x":0.23116865850890744,"y2":0,"group":1,"y":0.3893553335361346},{"x":0.23181057081107215,"y2":0,"group":1,"y":0.3928881309779227},{"x":0.2324524831132369,"y2":0,"group":1,"y":0.3964379361004293},{"x":0.23309439541540167,"y2":0,"group":1,"y":0.4000046347453356},{"x":0.23373630771756643,"y2":0,"group":1,"y":0.40358810989420085},{"x":0.23437822001973113,"y2":0,"group":1,"y":0.4071882416625385},{"x":0.2350201323218959,"y2":0,"group":1,"y":0.41080490729440683},{"x":0.23566204462406065,"y2":0,"group":1,"y":0.41443798115752434},{"x":0.2363039569262254,"y2":0,"group":1,"y":0.41808733473891313},{"x":0.23694586922839017,"y2":0,"group":1,"y":0.4217528366410764},{"x":0.23758778153055488,"y2":0,"group":1,"y":0.4254343525787184},{"x":0.23822969383271964,"y2":0,"group":1,"y":0.4291317453760089},{"x":0.2388716061348844,"y2":0,"group":1,"y":0.4328448749644005},{"x":0.23951351843704916,"y2":0,"group":1,"y":0.4365735983810013},{"x":0.24015543073921386,"y2":0,"group":1,"y":0.44031776976751436},{"x":0.24079734304137862,"y2":0,"group":1,"y":0.44407724036974033},{"x":0.24143925534354338,"y2":0,"group":1,"y":0.4478518585376572},{"x":0.24208116764570814,"y2":0,"group":1,"y":0.45164146972607566},{"x":0.24272307994787284,"y2":0,"group":1,"y":0.45544591649588},{"x":0.2433649922500376,"y2":0,"group":1,"y":0.4592650385158552},{"x":0.24400690455220236,"y2":0,"group":1,"y":0.46309867256510884},{"x":0.24464881685436712,"y2":0,"group":1,"y":0.4669466525360887},{"x":0.24529072915653188,"y2":0,"group":1,"y":0.470808809438204},{"x":0.2459326414586966,"y2":0,"group":1,"y":0.47468497140205224},{"x":0.24657455376086135,"y2":0,"group":1,"y":0.4785749636842581},{"x":0.2472164660630261,"y2":0,"group":1,"y":0.48247860867292436},{"x":0.24785837836519087,"y2":0,"group":1,"y":0.48639572589370594},{"x":0.24850029066735557,"y2":0,"group":1,"y":0.4903261320165033},{"x":0.24914220296952033,"y2":0,"group":1,"y":0.4942696408627843},{"x":0.2497841152716851,"y2":0,"group":1,"y":0.4982260634135357},{"x":0.25042602757384985,"y2":0,"group":1,"y":0.5021952078178493},{"x":0.25106793987601456,"y2":0,"group":1,"y":0.5061768794021432},{"x":0.2517098521781793,"y2":0,"group":1,"y":0.5101708806800255},{"x":0.2523517644803441,"y2":0,"group":1,"y":0.5141770113628008},{"x":0.25299367678250884,"y2":0,"group":1,"y":0.5181950683706223},{"x":0.2536355890846736,"y2":0,"group":1,"y":0.522224845844294},{"x":0.2542775013868383,"y2":0,"group":1,"y":0.5262661351577234},{"x":0.25491941368900306,"y2":0,"group":1,"y":0.530318724931031},{"x":0.2555613259911678,"y2":0,"group":1,"y":0.5343824010443128},{"x":0.2562032382933326,"y2":0,"group":1,"y":0.5384569466520639},{"x":0.2568451505954973,"y2":0,"group":1,"y":0.5425421421982632},{"x":0.25748706289766204,"y2":0,"group":1,"y":0.5466377654321191},{"x":0.2581289751998268,"y2":0,"group":1,"y":0.5507435914244796},{"x":0.25877088750199156,"y2":0,"group":1,"y":0.5548593925849106},{"x":0.25941279980415627,"y2":0,"group":1,"y":0.5589849386794392},{"x":0.26005471210632103,"y2":0,"group":1,"y":0.5631199968489643},{"x":0.2606966244084858,"y2":0,"group":1,"y":0.5672643316283421},{"x":0.26133853671065055,"y2":0,"group":1,"y":0.5714177049661342},{"x":0.2619804490128153,"y2":0,"group":1,"y":0.5755798762450344},{"x":0.26262236131498,"y2":0,"group":1,"y":0.579750602302961},{"x":0.2632642736171448,"y2":0,"group":1,"y":0.5839296374548251},{"x":0.26390618591930953,"y2":0,"group":1,"y":0.58811673351497},{"x":0.2645480982214743,"y2":0,"group":1,"y":0.592311639820281},{"x":0.265190010523639,"y2":0,"group":1,"y":0.5965141032539696},{"x":0.26583192282580376,"y2":0,"group":1,"y":0.6007238682700291},{"x":0.2664738351279685,"y2":0,"group":1,"y":0.6049406769183592},{"x":0.2671157474301333,"y2":0,"group":1,"y":0.6091642688705639},{"x":0.267757659732298,"y2":0,"group":1,"y":0.6133943814464171},{"x":0.26839957203446274,"y2":0,"group":1,"y":0.6176307496409974},{"x":0.2690414843366275,"y2":0,"group":1,"y":0.6218731061524904},{"x":0.26968339663879226,"y2":0,"group":1,"y":0.626121181410656},{"x":0.270325308940957,"y2":0,"group":1,"y":0.6303747036059612},{"x":0.2709672212431217,"y2":0,"group":1,"y":0.6346333987193744},{"x":0.2716091335452865,"y2":0,"group":1,"y":0.6388969905528203},{"x":0.27225104584745125,"y2":0,"group":1,"y":0.6431652007602934},{"x":0.272892958149616,"y2":0,"group":1,"y":0.6474377488796264},{"x":0.2735348704517807,"y2":0,"group":1,"y":0.6517143523649155},{"x":0.27417678275394547,"y2":0,"group":1,"y":0.6559947266195902},{"x":0.27481869505611023,"y2":0,"group":1,"y":0.6602785850301371},{"x":0.275460607358275,"y2":0,"group":1,"y":0.6645656390004643},{"x":0.2761025196604397,"y2":0,"group":1,"y":0.6688555979869079},{"x":0.27674443196260445,"y2":0,"group":1,"y":0.6731481695338787},{"x":0.2773863442647692,"y2":0,"group":1,"y":0.6774430593101388},{"x":0.278028256566934,"y2":0,"group":1,"y":0.681739971145713},{"x":0.27867016886909873,"y2":0,"group":1,"y":0.6860386070694232},{"x":0.27931208117126344,"y2":0,"group":1,"y":0.6903386673470445},{"x":0.2799539934734282,"y2":0,"group":1,"y":0.6946398505200825},{"x":0.28059590577559296,"y2":0,"group":1,"y":0.6989418534451586},{"x":0.2812378180777577,"y2":0,"group":1,"y":0.7032443713340084},{"x":0.2818797303799224,"y2":0,"group":1,"y":0.7075470977940772},{"x":0.2825216426820872,"y2":0,"group":1,"y":0.7118497248697194},{"x":0.28316355498425194,"y2":0,"group":1,"y":0.7161519430839858},{"x":0.2838054672864167,"y2":0,"group":1,"y":0.7204534414810004},{"x":0.2844473795885814,"y2":0,"group":1,"y":0.7247539076689161},{"x":0.28508929189074617,"y2":0,"group":1,"y":0.7290530278634473},{"x":0.2857312041929109,"y2":0,"group":1,"y":0.7333504869319711},{"x":0.2863731164950757,"y2":0,"group":1,"y":0.7376459684381903},{"x":0.28701502879724045,"y2":0,"group":1,"y":0.7419391546873523},{"x":0.28765694109940515,"y2":0,"group":1,"y":0.7462297267720197},{"x":0.2882988534015699,"y2":0,"group":1,"y":0.7505173646183796},{"x":0.28894076570373467,"y2":0,"group":1,"y":0.7548017470330926},{"x":0.28958267800589943,"y2":0,"group":1,"y":0.7590825517506661},{"x":0.29022459030806413,"y2":0,"group":1,"y":0.7633594554813505},{"x":0.2908665026102289,"y2":0,"group":1,"y":0.7676321339595478},{"x":0.29150841491239365,"y2":0,"group":1,"y":0.7719002619927262},{"x":0.2921503272145584,"y2":0,"group":1,"y":0.7761635135108307},{"x":0.2927922395167231,"y2":0,"group":1,"y":0.7804215616161838},{"x":0.2934341518188879,"y2":0,"group":1,"y":0.7846740786338671},{"x":0.29407606412105264,"y2":0,"group":1,"y":0.7889207361625751},{"x":0.2947179764232174,"y2":0,"group":1,"y":0.7931612051259318},{"x":0.29535988872538216,"y2":0,"group":1,"y":0.7973951558242635},{"x":0.29600180102754686,"y2":0,"group":1,"y":0.8016222579868159},{"x":0.2966437133297116,"y2":0,"group":1,"y":0.8058421808244098},{"x":0.2972856256318764,"y2":0,"group":1,"y":0.810054593082522},{"x":0.29792753793404114,"y2":0,"group":1,"y":0.8142591630947853},{"x":0.29856945023620585,"y2":0,"group":1,"y":0.8184555588368954},{"x":0.2992113625383706,"y2":0,"group":1,"y":0.822643447980918},{"x":0.29985327484053537,"y2":0,"group":1,"y":0.8268224979499816},{"x":0.3004951871427001,"y2":0,"group":1,"y":0.830992375973351},{"x":0.30113709944486483,"y2":0,"group":1,"y":0.8351527491418662},{"x":0.3017790117470296,"y2":0,"group":1,"y":0.8393032844637407},{"x":0.30242092404919435,"y2":0,"group":1,"y":0.8434436489207047},{"x":0.3030628363513591,"y2":0,"group":1,"y":0.8475735095244871},{"x":0.30370474865352387,"y2":0,"group":1,"y":0.8516925333736184},{"x":0.3043466609556886,"y2":0,"group":1,"y":0.8558003877105516},{"x":0.30498857325785333,"y2":0,"group":1,"y":0.8598967399790848},{"x":0.3056304855600181,"y2":0,"group":1,"y":0.8639812578820734},{"x":0.30627239786218285,"y2":0,"group":1,"y":0.8680536094394257},{"x":0.30691431016434756,"y2":0,"group":1,"y":0.872113463046365},{"x":0.3075562224665123,"y2":0,"group":1,"y":0.8761604875319486},{"x":0.3081981347686771,"y2":0,"group":1,"y":0.880194352217833},{"x":0.30884004707084184,"y2":0,"group":1,"y":0.8842147269772707},{"x":0.30948195937300654,"y2":0,"group":1,"y":0.8882212822943293},{"x":0.3101238716751713,"y2":0,"group":1,"y":0.8922136893233185},{"x":0.31076578397733606,"y2":0,"group":1,"y":0.8961916199484137},{"x":0.3114076962795008,"y2":0,"group":1,"y":0.9001547468434647},{"x":0.3120496085816656,"y2":0,"group":1,"y":0.904102743531974},{"x":0.3126915208838303,"y2":0,"group":1,"y":0.9080352844472348},{"x":0.31333343318599505,"y2":0,"group":1,"y":0.911952044992617},{"x":0.3139753454881598,"y2":0,"group":1,"y":0.9158527016019824},{"x":0.31461725779032457,"y2":0,"group":1,"y":0.9197369318002256},{"x":0.31525917009248927,"y2":0,"group":1,"y":0.9236044142639162},{"x":0.31590108239465403,"y2":0,"group":1,"y":0.9274548288820404},{"x":0.3165429946968188,"y2":0,"group":1,"y":0.931287856816821},{"x":0.31718490699898355,"y2":0,"group":1,"y":0.9351031805646044},{"x":0.31782681930114826,"y2":0,"group":1,"y":0.9389004840168049},{"x":0.318468731603313,"y2":0,"group":1,"y":0.9426794525208861},{"x":0.3191106439054778,"y2":0,"group":1,"y":0.9464397729413736},{"x":0.31975255620764254,"y2":0,"group":1,"y":0.9501811337208779},{"x":0.3203944685098073,"y2":0,"group":1,"y":0.953903224941119},{"x":0.321036380811972,"y2":0,"group":1,"y":0.9576057383839383},{"x":0.32167829311413676,"y2":0,"group":1,"y":0.9612883675922808},{"x":0.3223202054163015,"y2":0,"group":1,"y":0.9649508079311389},{"x":0.3229621177184663,"y2":0,"group":1,"y":0.9685927566484382},{"x":0.323604030020631,"y2":0,"group":1,"y":0.972213912935857},{"x":0.32424594232279574,"y2":0,"group":1,"y":0.9758139779895596},{"x":0.3248878546249605,"y2":0,"group":1,"y":0.9793926550708372},{"x":0.32552976692712526,"y2":0,"group":1,"y":0.9829496495666318},{"x":0.32617167922928997,"y2":0,"group":1,"y":0.9864846690499404},{"x":0.32681359153145473,"y2":0,"group":1,"y":0.9899974233400772},{"x":0.3274555038336195,"y2":0,"group":1,"y":0.9934876245627822},{"x":0.32809741613578425,"y2":0,"group":1,"y":0.9969549872101636},{"x":0.328739328437949,"y2":0,"group":1,"y":1.000399228200458},{"x":0.3293812407401137,"y2":0,"group":1,"y":1.003820066937593},{"x":0.33002315304227847,"y2":0,"group":1,"y":1.0072172253705447},{"x":0.33066506534444323,"y2":0,"group":1,"y":1.010590428052463},{"x":0.331306977646608,"y2":0,"group":1,"y":1.013939402199567},{"x":0.3319488899487727,"y2":0,"group":1,"y":1.0172638777497802},{"x":0.33259080225093746,"y2":0,"group":1,"y":1.0205635874211056},{"x":0.3332327145531022,"y2":0,"group":1,"y":1.0238382667697143},{"x":0.333874626855267,"y2":0,"group":1,"y":1.0270876542477412},{"x":0.3345165391574317,"y2":0,"group":1,"y":1.0303114912607725},{"x":0.33515845145959644,"y2":0,"group":1,"y":1.0335095222250075},{"x":0.3358003637617612,"y2":0,"group":1,"y":1.0366814946240837},{"x":0.33644227606392596,"y2":0,"group":1,"y":1.0398271590655517},{"x":0.3370841883660907,"y2":0,"group":1,"y":1.0429462693369826},{"x":0.3377261006682554,"y2":0,"group":1,"y":1.0460385824616965},{"x":0.3383680129704202,"y2":0,"group":1,"y":1.049103858754097},{"x":0.33900992527258494,"y2":0,"group":1,"y":1.0521418618746001},{"x":0.3396518375747497,"y2":0,"group":1,"y":1.0551523588841394},{"x":0.3402937498769144,"y2":0,"group":1,"y":1.058135120298237},{"x":0.34093566217907917,"y2":0,"group":1,"y":1.0610899201406276},{"x":0.34157757448124393,"y2":0,"group":1,"y":1.0640165359964175},{"x":0.3422194867834087,"y2":0,"group":1,"y":1.0669147490647677},{"x":0.3428613990855734,"y2":0,"group":1,"y":1.0697843442110908},{"x":0.34350331138773815,"y2":0,"group":1,"y":1.0726251100187394},{"x":0.3441452236899029,"y2":0,"group":1,"y":1.075436838840184},{"x":0.3447871359920677,"y2":0,"group":1,"y":1.0782193268476572},{"x":0.34542904829423243,"y2":0,"group":1,"y":1.0809723740832549},{"x":0.34607096059639714,"y2":0,"group":1,"y":1.0836957845084856},{"x":0.3467128728985619,"y2":0,"group":1,"y":1.0863893660532464},{"x":0.34735478520072666,"y2":0,"group":1,"y":1.0890529306642185},{"x":0.3479966975028914,"y2":0,"group":1,"y":1.09168629435267},{"x":0.3486386098050561,"y2":0,"group":1,"y":1.0942892772416468},{"x":0.3492805221072209,"y2":0,"group":1,"y":1.0968617036125492},{"x":0.34992243440938564,"y2":0,"group":1,"y":1.0994034019510701},{"x":0.3505643467115504,"y2":0,"group":1,"y":1.1019142049924944},{"x":0.3512062590137151,"y2":0,"group":1,"y":1.104393949766336},{"x":0.35184817131587987,"y2":0,"group":1,"y":1.1068424776403087},{"x":0.3524900836180446,"y2":0,"group":1,"y":1.1092596343636139},{"x":0.3531319959202094,"y2":0,"group":1,"y":1.1116452701095392},{"x":0.35377390822237414,"y2":0,"group":1,"y":1.1139992395173457},{"x":0.35441582052453885,"y2":0,"group":1,"y":1.1163214017334453},{"x":0.3550577328267036,"y2":0,"group":1,"y":1.1186116204518473},{"x":0.35569964512886837,"y2":0,"group":1,"y":1.1208697639538645},{"x":0.35634155743103313,"y2":0,"group":1,"y":1.123095705147073},{"x":0.35698346973319783,"y2":0,"group":1,"y":1.125289321603506},{"x":0.3576253820353626,"y2":0,"group":1,"y":1.1274504955970808},{"x":0.35826729433752735,"y2":0,"group":1,"y":1.129579114140239},{"x":0.3589092066396921,"y2":0,"group":1,"y":1.131675069019792},{"x":0.3595511189418568,"y2":0,"group":1,"y":1.1337382568319725},{"x":0.3601930312440216,"y2":0,"group":1,"y":1.1357685790166578},{"x":0.36083494354618634,"y2":0,"group":1,"y":1.137765941890782},{"x":0.3614768558483511,"y2":0,"group":1,"y":1.1397302566809067},{"x":0.36211876815051586,"y2":0,"group":1,"y":1.1416614395549531},{"x":0.36276068045268056,"y2":0,"group":1,"y":1.143559411653078},{"x":0.3634025927548453,"y2":0,"group":1,"y":1.1454240991176898},{"x":0.3640445050570101,"y2":0,"group":1,"y":1.147255433122592},{"x":0.36468641735917484,"y2":0,"group":1,"y":1.1490533499012487},{"x":0.36532832966133955,"y2":0,"group":1,"y":1.1508177907741592},{"x":0.3659702419635043,"y2":0,"group":1,"y":1.1525487021753356},{"x":0.36661215426566907,"y2":0,"group":1,"y":1.1542460356778745},{"x":0.3672540665678338,"y2":0,"group":1,"y":1.1559097480186191},{"x":0.36789597886999853,"y2":0,"group":1,"y":1.1575398011218918},{"x":0.3685378911721633,"y2":0,"group":1,"y":1.1591361621223035},{"x":0.36917980347432805,"y2":0,"group":1,"y":1.1606988033866261},{"x":0.3698217157764928,"y2":0,"group":1,"y":1.162227702534714},{"x":0.37046362807865757,"y2":0,"group":1,"y":1.163722842459483},{"x":0.3711055403808223,"y2":0,"group":1,"y":1.1651842113459259},{"x":0.37174745268298703,"y2":0,"group":1,"y":1.1666118026891628},{"x":0.3723893649851518,"y2":0,"group":1,"y":1.1680056153115232},{"x":0.37303127728731655,"y2":0,"group":1,"y":1.1693656533786478},{"x":0.37367318958948126,"y2":0,"group":1,"y":1.1706919264146078},{"x":0.374315101891646,"y2":0,"group":1,"y":1.1719844493160336},{"x":0.3749570141938108,"y2":0,"group":1,"y":1.173243242365252},{"x":0.37559892649597554,"y2":0,"group":1,"y":1.1744683312424182},{"x":0.3762408387981403,"y2":0,"group":1,"y":1.1756597470366457},{"x":0.376882751100305,"y2":0,"group":1,"y":1.176817526256126},{"x":0.37752466340246976,"y2":0,"group":1,"y":1.1779417108372312},{"x":0.3781665757046345,"y2":0,"group":1,"y":1.1790323481525988},{"x":0.3788084880067993,"y2":0,"group":1,"y":1.1800894910181934},{"x":0.379450400308964,"y2":0,"group":1,"y":1.1811131976993394},{"x":0.38009231261112875,"y2":0,"group":1,"y":1.1821035319157254},{"x":0.3807342249132935,"y2":0,"group":1,"y":1.183060562845369},{"x":0.38137613721545827,"y2":0,"group":1,"y":1.1839843651275497},{"x":0.38201804951762297,"y2":0,"group":1,"y":1.1848750188646964},{"x":0.38265996181978773,"y2":0,"group":1,"y":1.1857326096232348},{"x":0.3833018741219525,"y2":0,"group":1,"y":1.1865572284333843},{"x":0.38394378642411725,"y2":0,"group":1,"y":1.1873489717879129},{"x":0.384585698726282,"y2":0,"group":1,"y":1.188107941639835},{"x":0.3852276110284467,"y2":0,"group":1,"y":1.1888342453990628},{"x":0.3858695233306115,"y2":0,"group":1,"y":1.1895279959279992},{"x":0.38651143563277623,"y2":0,"group":1,"y":1.1901893115360807},{"x":0.387153347934941,"y2":0,"group":1,"y":1.19081831597326},{"x":0.3877952602371057,"y2":0,"group":1,"y":1.1914151384224339},{"x":0.38843717253927046,"y2":0,"group":1,"y":1.1919799134908167},{"x":0.3890790848414352,"y2":0,"group":1,"y":1.1925127812002467},{"x":0.3897209971436,"y2":0,"group":1,"y":1.19301388697645},{"x":0.3903629094457647,"y2":0,"group":1,"y":1.1934833816372306},{"x":0.39100482174792944,"y2":0,"group":1,"y":1.193921421379617},{"x":0.3916467340500942,"y2":0,"group":1,"y":1.1943281677659443},{"x":0.39228864635225896,"y2":0,"group":1,"y":1.1947037877088826},{"x":0.3929305586544237,"y2":0,"group":1,"y":1.1950484534554162},{"x":0.3935724709565884,"y2":0,"group":1,"y":1.1953623425697617},{"x":0.3942143832587532,"y2":0,"group":1,"y":1.195645637915243},{"x":0.39485629556091795,"y2":0,"group":1,"y":1.1958985276351115},{"x":0.3954982078630827,"y2":0,"group":1,"y":1.1961212051323233},{"x":0.3961401201652474,"y2":0,"group":1,"y":1.1963138690482698},{"x":0.39678203246741217,"y2":0,"group":1,"y":1.196476723240465},{"x":0.39742394476957693,"y2":0,"group":1,"y":1.1966099767591996},{"x":0.3980658570717417,"y2":0,"group":1,"y":1.1967138438231517},{"x":0.3987077693739064,"y2":0,"group":1,"y":1.1967885437939718},{"x":0.39934968167607116,"y2":0,"group":1,"y":1.1968343011498337},{"x":0.3999915939782359,"y2":0,"group":1,"y":1.1968513454579648},{"x":0.4006335062804007,"y2":0,"group":1,"y":1.1968399113461519},{"x":0.40127541858256544,"y2":0,"group":1,"y":1.1968002384732332},{"x":0.40191733088473014,"y2":0,"group":1,"y":1.196732571498579},{"x":0.4025592431868949,"y2":0,"group":1,"y":1.1966371600505599},{"x":0.40320115548905966,"y2":0,"group":1,"y":1.1965142586940198},{"x":0.4038430677912244,"y2":0,"group":1,"y":1.1963641268967493},{"x":0.4044849800933891,"y2":0,"group":1,"y":1.1961870289949665},{"x":0.4051268923955539,"y2":0,"group":1,"y":1.1959832341578136},{"x":0.40576880469771864,"y2":0,"group":1,"y":1.1957530163508747},{"x":0.4064107169998834,"y2":0,"group":1,"y":1.1954966542987189},{"x":0.4070526293020481,"y2":0,"group":1,"y":1.195214431446477},{"x":0.40769454160421287,"y2":0,"group":1,"y":1.1949066359204585},{"x":0.4083364539063776,"y2":0,"group":1,"y":1.1945735604878136},{"x":0.4089783662085424,"y2":0,"group":1,"y":1.194215502515251},{"x":0.40962027851070715,"y2":0,"group":1,"y":1.1938327639268156},{"x":0.41026219081287185,"y2":0,"group":1,"y":1.1934256511607324},{"x":0.4109041031150366,"y2":0,"group":1,"y":1.1929944751253323},{"x":0.41154601541720137,"y2":0,"group":1,"y":1.1925395511540553},{"x":0.41218792771936613,"y2":0,"group":1,"y":1.1920611989595513},{"x":0.41282984002153084,"y2":0,"group":1,"y":1.1915597425868778},{"x":0.4134717523236956,"y2":0,"group":1,"y":1.1910355103658117},{"x":0.41411366462586036,"y2":0,"group":1,"y":1.1904888348622706},{"x":0.4147555769280251,"y2":0,"group":1,"y":1.189920052828872},{"x":0.4153974892301898,"y2":0,"group":1,"y":1.1893295051546176},{"x":0.4160394015323546,"y2":0,"group":1,"y":1.1887175368137313},{"x":0.41668131383451934,"y2":0,"group":1,"y":1.1880844968136488},{"x":0.4173232261366841,"y2":0,"group":1,"y":1.1874307381421716},{"x":0.41796513843884886,"y2":0,"group":1,"y":1.1867566177137956},{"x":0.41860705074101356,"y2":0,"group":1,"y":1.186062496315226},{"x":0.4192489630431783,"y2":0,"group":1,"y":1.1853487385500867},{"x":0.4198908753453431,"y2":0,"group":1,"y":1.184615712782833},{"x":0.42053278764750784,"y2":0,"group":1,"y":1.1838637910818832},{"x":0.42117469994967255,"y2":0,"group":1,"y":1.1830933491619768},{"x":0.4218166122518373,"y2":0,"group":1,"y":1.1823047663257684},{"x":0.42245852455400207,"y2":0,"group":1,"y":1.181498425404673},{"x":0.42310043685616683,"y2":0,"group":1,"y":1.1806747126989716},{"x":0.42374234915833153,"y2":0,"group":1,"y":1.1798340179171865},{"x":0.4243842614604963,"y2":0,"group":1,"y":1.1789767341147426},{"x":0.42502617376266105,"y2":0,"group":1,"y":1.1781032576319228},{"x":0.4256680860648258,"y2":0,"group":1,"y":1.1772139880311323},{"x":0.4263099983669906,"y2":0,"group":1,"y":1.1763093280334773},{"x":0.4269519106691553,"y2":0,"group":1,"y":1.1753896834546835},{"x":0.42759382297132004,"y2":0,"group":1,"y":1.1744554631403512},{"x":0.4282357352734848,"y2":0,"group":1,"y":1.1735070789005697},{"x":0.42887764757564956,"y2":0,"group":1,"y":1.1725449454439028},{"x":0.42951955987781426,"y2":0,"group":1,"y":1.1715694803107528},{"x":0.430161472179979,"y2":0,"group":1,"y":1.1705811038061222},{"x":0.4308033844821438,"y2":0,"group":1,"y":1.1695802389317822},{"x":0.43144529678430854,"y2":0,"group":1,"y":1.1685673113178616},{"x":0.43208720908647325,"y2":0,"group":1,"y":1.1675427491538726},{"x":0.432729121388638,"y2":0,"group":1,"y":1.1665069831191817},{"x":0.43337103369080276,"y2":0,"group":1,"y":1.1654604463129434},{"x":0.4340129459929675,"y2":0,"group":1,"y":1.1644035741835108},{"x":0.4346548582951323,"y2":0,"group":1,"y":1.1633368044573302},{"x":0.435296770597297,"y2":0,"group":1,"y":1.1622605770673433},{"x":0.43593868289946175,"y2":0,"group":1,"y":1.161175334080903},{"x":0.4365805952016265,"y2":0,"group":1,"y":1.160081519627222},{"x":0.43722250750379127,"y2":0,"group":1,"y":1.158979579824364},{"x":0.437864419805956,"y2":0,"group":1,"y":1.1578699627057953},{"x":0.43850633210812073,"y2":0,"group":1,"y":1.15675311814651},{"x":0.4391482444102855,"y2":0,"group":1,"y":1.1556294977887416},{"x":0.43979015671245025,"y2":0,"group":1,"y":1.154499554967278},{"x":0.44043206901461496,"y2":0,"group":1,"y":1.1533637446343945},{"x":0.4410739813167797,"y2":0,"group":1,"y":1.152222523284414},{"x":0.4417158936189445,"y2":0,"group":1,"y":1.1510763488779174},{"x":0.44235780592110924,"y2":0,"group":1,"y":1.1499256807656149},{"x":0.442999718223274,"y2":0,"group":1,"y":1.1487709796118883},{"x":0.4436416305254387,"y2":0,"group":1,"y":1.1476127073180307},{"x":0.44428354282760346,"y2":0,"group":1,"y":1.1464513269451844},{"x":0.4449254551297682,"y2":0,"group":1,"y":1.1452873026370072},{"x":0.445567367431933,"y2":0,"group":1,"y":1.144121099542066},{"x":0.4462092797340977,"y2":0,"group":1,"y":1.1429531837359885},{"x":0.44685119203626245,"y2":0,"group":1,"y":1.141784022143374},{"x":0.4474931043384272,"y2":0,"group":1,"y":1.140614082459489},{"x":0.44813501664059197,"y2":0,"group":1,"y":1.1394438330717602},{"x":0.44877692894275667,"y2":0,"group":1,"y":1.1382737429810732},{"x":0.44941884124492143,"y2":0,"group":1,"y":1.1371042817229018},{"x":0.4500607535470862,"y2":0,"group":1,"y":1.1359359192882803},{"x":0.45070266584925095,"y2":0,"group":1,"y":1.1347691260446267},{"x":0.4513445781514157,"y2":0,"group":1,"y":1.1336043726564453},{"x":0.4519864904535804,"y2":0,"group":1,"y":1.132442130005912},{"x":0.4526284027557452,"y2":0,"group":1,"y":1.131282869113364},{"x":0.45327031505790993,"y2":0,"group":1,"y":1.130127061057709},{"x":0.4539122273600747,"y2":0,"group":1,"y":1.1289751768967653},{"x":0.4545541396622394,"y2":0,"group":1,"y":1.1278276875875535},{"x":0.45519605196440416,"y2":0,"group":1,"y":1.1266850639065487},{"x":0.4558379642665689,"y2":0,"group":1,"y":1.125547776369917},{"x":0.4564798765687337,"y2":0,"group":1,"y":1.1244162951537426},{"x":0.45712178887089844,"y2":0,"group":1,"y":1.1232910900142659},{"x":0.45776370117306314,"y2":0,"group":1,"y":1.1221726302081492},{"x":0.45840561347522796,"y2":0,"group":1,"y":1.1210613844127786},{"x":0.45904752577739266,"y2":0,"group":1,"y":1.1199578206466252},{"x":0.45968943807955737,"y2":0,"group":1,"y":1.1188624061896761},{"x":0.4603313503817222,"y2":0,"group":1,"y":1.1177756075039496},{"x":0.4609732626838869,"y2":0,"group":1,"y":1.1166978901541156},{"x":0.4616151749860516,"y2":0,"group":1,"y":1.1156297187282254},{"x":0.4622570872882164,"y2":0,"group":1,"y":1.1145715567585812},{"x":0.4628989995903811,"y2":0,"group":1,"y":1.1135238666427436},{"x":0.4635409118925459,"y2":0,"group":1,"y":1.1124871095647066},{"x":0.46418282419471063,"y2":0,"group":1,"y":1.1114617454162434},{"x":0.46482473649687533,"y2":0,"group":1,"y":1.1104482327184468},{"x":0.46546664879904015,"y2":0,"group":1,"y":1.1094470285434743},{"x":0.46610856110120485,"y2":0,"group":1,"y":1.108458588436509},{"x":0.46675047340336967,"y2":0,"group":1,"y":1.1074833663379644},{"x":0.4673923857055344,"y2":0,"group":1,"y":1.10652181450593},{"x":0.4680342980076991,"y2":0,"group":1,"y":1.1055743834388836},{"x":0.4686762103098639,"y2":0,"group":1,"y":1.1046415217986845},{"x":0.4693181226120286,"y2":0,"group":1,"y":1.1037236763338574},{"x":0.4699600349141933,"y2":0,"group":1,"y":1.1028212918031814},{"x":0.4706019472163581,"y2":0,"group":1,"y":1.101934810899604},{"x":0.4712438595185228,"y2":0,"group":1,"y":1.1010646741744876},{"x":0.47188577182068764,"y2":0,"group":1,"y":1.100211319962206},{"x":0.47252768412285234,"y2":0,"group":1,"y":1.0993751843051056},{"x":0.47316959642501705,"y2":0,"group":1,"y":1.0985567008788402},{"x":0.47381150872718186,"y2":0,"group":1,"y":1.0977563009181013},{"x":0.47445342102934657,"y2":0,"group":1,"y":1.0969744131427488},{"x":0.4750953333315114,"y2":0,"group":1,"y":1.0962114636843618},{"x":0.4757372456336761,"y2":0,"group":1,"y":1.0954678760132213},{"x":0.4763791579358408,"y2":0,"group":1,"y":1.094744070865735},{"x":0.4770210702380056,"y2":0,"group":1,"y":1.094040466172323},{"x":0.4776629825401703,"y2":0,"group":1,"y":1.0933574769857746},{"x":0.478304894842335,"y2":0,"group":1,"y":1.0926955154100861},{"x":0.47894680714449983,"y2":0,"group":1,"y":1.0920549905297996},{"x":0.47958871944666454,"y2":0,"group":1,"y":1.0914363083398488},{"x":0.48023063174882935,"y2":0,"group":1,"y":1.090839871675929},{"x":0.48087254405099406,"y2":0,"group":1,"y":1.0902660801454016},{"x":0.48151445635315876,"y2":0,"group":1,"y":1.0897153300587452},{"x":0.4821563686553236,"y2":0,"group":1,"y":1.0891880143615673},{"x":0.4827982809574883,"y2":0,"group":1,"y":1.0886845225671875},{"x":0.4834401932596531,"y2":0,"group":1,"y":1.088205240689806},{"x":0.4840821055618178,"y2":0,"group":1,"y":1.0877505511782646},{"x":0.4847240178639825,"y2":0,"group":1,"y":1.0873208328504194},{"x":0.4853659301661473,"y2":0,"group":1,"y":1.0869164608281285},{"x":0.486007842468312,"y2":0,"group":1,"y":1.0865378064728737},{"x":0.48664975477047673,"y2":0,"group":1,"y":1.0861852373220215},{"x":0.48729166707264154,"y2":0,"group":1,"y":1.0858591170257421},{"x":0.48793357937480625,"y2":0,"group":1,"y":1.0855598052845825},{"x":0.48857549167697106,"y2":0,"group":1,"y":1.0852876577877284},{"x":0.48921740397913577,"y2":0,"group":1,"y":1.0850430261519413},{"x":0.48985931628130047,"y2":0,"group":1,"y":1.0848262578611942},{"x":0.4905012285834653,"y2":0,"group":1,"y":1.0846376962070199},{"x":0.49114314088563,"y2":0,"group":1,"y":1.0844776802295695},{"x":0.4917850531877948,"y2":0,"group":1,"y":1.0843465446594018},{"x":0.4924269654899595,"y2":0,"group":1,"y":1.0842446198600098},{"x":0.4930688777921242,"y2":0,"group":1,"y":1.084172231771094},{"x":0.49371079009428903,"y2":0,"group":1,"y":1.084129701852592},{"x":0.49435270239645374,"y2":0,"group":1,"y":1.0841173470294774},{"x":0.49499461469861844,"y2":0,"group":1,"y":1.0841354796373293},{"x":0.49563652700078326,"y2":0,"group":1,"y":1.0841844073686928},{"x":0.49627843930294796,"y2":0,"group":1,"y":1.0842644332202287},{"x":0.4969203516051128,"y2":0,"group":1,"y":1.084375855440668},{"x":0.4975622639072775,"y2":0,"group":1,"y":1.0845189674795779},{"x":0.4982041762094422,"y2":0,"group":1,"y":1.084694057936949},{"x":0.498846088511607,"y2":0,"group":1,"y":1.0849014105136079},{"x":0.4994880008137717,"y2":0,"group":1,"y":1.08514130396247},{"x":0.5001299131159365,"y2":0,"group":1,"y":1.085414012040633},{"x":0.5007718254181013,"y2":0,"group":1,"y":1.0857198034623274},{"x":0.5014137377202659,"y2":0,"group":1,"y":1.086058941852722},{"x":0.5020556500224307,"y2":0,"group":1,"y":1.0864316857026044},{"x":0.5026975623245955,"y2":0,"group":1,"y":1.0868382883239254},{"x":0.5033394746267601,"y2":0,"group":1,"y":1.0872789978062372},{"x":0.5039813869289249,"y2":0,"group":1,"y":1.0877540569740127},{"x":0.5046232992310897,"y2":0,"group":1,"y":1.0882637033448646},{"x":0.5052652115332545,"y2":0,"group":1,"y":1.0888081690886648},{"x":0.5059071238354191,"y2":0,"group":1,"y":1.0893876809875722},{"x":0.506549036137584,"y2":0,"group":1,"y":1.0900024603969802},{"x":0.5071909484397488,"y2":0,"group":1,"y":1.090652723207378},{"x":0.5078328607419134,"y2":0,"group":1,"y":1.091338679807147},{"x":0.5084747730440782,"y2":0,"group":1,"y":1.0920605350462802},{"x":0.509116685346243,"y2":0,"group":1,"y":1.0928184882010528},{"x":0.5097585976484076,"y2":0,"group":1,"y":1.0936127329396228},{"x":0.5104005099505724,"y2":0,"group":1,"y":1.0944434572885888},{"x":0.5110424222527372,"y2":0,"group":1,"y":1.0953108436005004},{"x":0.5116843345549018,"y2":0,"group":1,"y":1.0962150685223242},{"x":0.5123262468570666,"y2":0,"group":1,"y":1.097156302964875},{"x":0.5129681591592314,"y2":0,"group":1,"y":1.0981347120732163},{"x":0.5136100714613963,"y2":0,"group":1,"y":1.0991504551980253},{"x":0.5142519837635608,"y2":0,"group":1,"y":1.1002036858679443},{"x":0.5148938960657257,"y2":0,"group":1,"y":1.1012945517629016},{"x":0.5155358083678905,"y2":0,"group":1,"y":1.1024231946884218},{"x":0.5161777206700551,"y2":0,"group":1,"y":1.1035897505509218},{"x":0.5168196329722199,"y2":0,"group":1,"y":1.1047943493339933},{"x":0.5174615452743847,"y2":0,"group":1,"y":1.1060371150756845},{"x":0.5181034575765493,"y2":0,"group":1,"y":1.107318165846772},{"x":0.5187453698787141,"y2":0,"group":1,"y":1.1086376137300333},{"x":0.5193872821808789,"y2":0,"group":1,"y":1.1099955648005255},{"x":0.5200291944830435,"y2":0,"group":1,"y":1.111392119106857},{"x":0.5206711067852083,"y2":0,"group":1,"y":1.1128273706534757},{"x":0.5213130190873732,"y2":0,"group":1,"y":1.11430140738396},{"x":0.521954931389538,"y2":0,"group":1,"y":1.1158143111653178},{"x":0.5225968436917026,"y2":0,"group":1,"y":1.1173661577733014},{"x":0.5232387559938674,"y2":0,"group":1,"y":1.1189570168787308},{"x":0.5238806682960322,"y2":0,"group":1,"y":1.1205869520348324},{"x":0.5245225805981968,"y2":0,"group":1,"y":1.1222560206655936},{"x":0.5251644929003616,"y2":0,"group":1,"y":1.1239642740551319},{"x":0.5258064052025264,"y2":0,"group":1,"y":1.1257117573380788},{"x":0.526448317504691,"y2":0,"group":1,"y":1.127498509490986},{"x":0.5270902298068558,"y2":0,"group":1,"y":1.1293245633247415},{"x":0.5277321421090206,"y2":0,"group":1,"y":1.1311899454780088},{"x":0.5283740544111852,"y2":0,"group":1,"y":1.1330946764116816},{"x":0.52901596671335,"y2":0,"group":1,"y":1.1350387704043545},{"x":0.5296578790155149,"y2":0,"group":1,"y":1.1370222355488102},{"x":0.5302997913176797,"y2":0,"group":1,"y":1.139045073749525},{"x":0.5309417036198443,"y2":0,"group":1,"y":1.141107280721187},{"x":0.5315836159220091,"y2":0,"group":1,"y":1.1432088459882297},{"x":0.5322255282241739,"y2":0,"group":1,"y":1.1453497528853787},{"x":0.5328674405263385,"y2":0,"group":1,"y":1.147529978559208},{"x":0.5335093528285033,"y2":0,"group":1,"y":1.1497494939707114},{"x":0.5341512651306681,"y2":0,"group":1,"y":1.152008263898877},{"x":0.5347931774328327,"y2":0,"group":1,"y":1.1543062469452692},{"x":0.5354350897349975,"y2":0,"group":1,"y":1.1566433955396205},{"x":0.5360770020371624,"y2":0,"group":1,"y":1.159019655946417},{"x":0.536718914339327,"y2":0,"group":1,"y":1.1614349682724925},{"x":0.5373608266414918,"y2":0,"group":1,"y":1.1638892664756142},{"x":0.5380027389436566,"y2":0,"group":1,"y":1.1663824783740637},{"x":0.5386446512458214,"y2":0,"group":1,"y":1.1689145256572115},{"x":0.539286563547986,"y2":0,"group":1,"y":1.1714853238970795},{"x":0.5399284758501508,"y2":0,"group":1,"y":1.1740947825608887},{"x":0.5405703881523156,"y2":0,"group":1,"y":1.176742805024586},{"x":0.5412123004544802,"y2":0,"group":1,"y":1.1794292885873547},{"x":0.541854212756645,"y2":0,"group":1,"y":1.1821541244870946},{"x":0.5424961250588098,"y2":0,"group":1,"y":1.184917197916873},{"x":0.5431380373609744,"y2":0,"group":1,"y":1.1877183880423472},{"x":0.5437799496631393,"y2":0,"group":1,"y":1.19055756802014},{"x":0.5444218619653041,"y2":0,"group":1,"y":1.193434605017182},{"x":0.5450637742674687,"y2":0,"group":1,"y":1.1963493602310016},{"x":0.5457056865696335,"y2":0,"group":1,"y":1.1993016889109622},{"x":0.5463475988717983,"y2":0,"group":1,"y":1.2022914403804488},{"x":0.5469895111739631,"y2":0,"group":1,"y":1.2053184580599843},{"x":0.5476314234761277,"y2":0,"group":1,"y":1.2083825794912875},{"x":0.5482733357782925,"y2":0,"group":1,"y":1.2114836363622499},{"x":0.5489152480804573,"y2":0,"group":1,"y":1.2146214545328426},{"x":0.5495571603826219,"y2":0,"group":1,"y":1.217795854061932},{"x":0.5501990726847867,"y2":0,"group":1,"y":1.2210066492350125},{"x":0.5508409849869516,"y2":0,"group":1,"y":1.2242536485928368},{"x":0.5514828972891161,"y2":0,"group":1,"y":1.2275366549609503},{"x":0.552124809591281,"y2":0,"group":1,"y":1.2308554654801123},{"x":0.5527667218934458,"y2":0,"group":1,"y":1.2342098716376064},{"x":0.5534086341956104,"y2":0,"group":1,"y":1.2375996592994258},{"x":0.5540505464977752,"y2":0,"group":1,"y":1.2410246087433334},{"x":0.55469245879994,"y2":0,"group":1,"y":1.2444844946927853},{"x":0.5553343711021048,"y2":0,"group":1,"y":1.2479790863517106},{"x":0.5559762834042694,"y2":0,"group":1,"y":1.2515081474401462},{"x":0.5566181957064342,"y2":0,"group":1,"y":1.2550714362307085},{"x":0.557260108008599,"y2":0,"group":1,"y":1.2586687055859085},{"x":0.5579020203107636,"y2":0,"group":1,"y":1.2622997029962848},{"x":0.5585439326129285,"y2":0,"group":1,"y":1.2659641706193687},{"x":0.5591858449150933,"y2":0,"group":1,"y":1.269661845319451},{"x":0.5598277572172579,"y2":0,"group":1,"y":1.2733924587081606},{"x":0.5604696695194227,"y2":0,"group":1,"y":1.2771557371858342},{"x":0.5611115818215875,"y2":0,"group":1,"y":1.2809514019836772},{"x":0.5617534941237521,"y2":0,"group":1,"y":1.2847791692067032},{"x":0.5623954064259169,"y2":0,"group":1,"y":1.288638749877442},{"x":0.5630373187280817,"y2":0,"group":1,"y":1.2925298499804132},{"x":0.5636792310302465,"y2":0,"group":1,"y":1.2964521705073517},{"x":0.5643211433324111,"y2":0,"group":1,"y":1.3004054075031757},{"x":0.5649630556345759,"y2":0,"group":1,"y":1.304389252112693},{"x":0.5656049679367408,"y2":0,"group":1,"y":1.3084033906280335},{"x":0.5662468802389053,"y2":0,"group":1,"y":1.3124475045367938},{"x":0.5668887925410702,"y2":0,"group":1,"y":1.3165212705708977},{"x":0.567530704843235,"y2":0,"group":1,"y":1.3206243607561454},{"x":0.5681726171453996,"y2":0,"group":1,"y":1.3247564424624585},{"x":0.5688145294475644,"y2":0,"group":1,"y":1.3289171784547986},{"x":0.5694564417497292,"y2":0,"group":1,"y":1.3331062269447576},{"x":0.5700983540518938,"y2":0,"group":1,"y":1.3373232416428074},{"x":0.5707402663540586,"y2":0,"group":1,"y":1.3415678718111945},{"x":0.5713821786562234,"y2":0,"group":1,"y":1.3458397623174758},{"x":0.5720240909583882,"y2":0,"group":1,"y":1.3501385536886843},{"x":0.5726660032605528,"y2":0,"group":1,"y":1.3544638821661108},{"x":0.5733079155627177,"y2":0,"group":1,"y":1.3588153797606914},{"x":0.5739498278648825,"y2":0,"group":1,"y":1.3631926743089964},{"x":0.5745917401670471,"y2":0,"group":1,"y":1.3675953895298023},{"x":0.5752336524692119,"y2":0,"group":1,"y":1.3720231450812408},{"x":0.5758755647713767,"y2":0,"group":1,"y":1.376475556618511},{"x":0.5765174770735413,"y2":0,"group":1,"y":1.3809522358521489},{"x":0.5771593893757061,"y2":0,"group":1,"y":1.3854527906068355},{"x":0.5778013016778709,"y2":0,"group":1,"y":1.3899768248807414},{"x":0.5784432139800355,"y2":0,"group":1,"y":1.3945239389053865},{"x":0.5790851262822003,"y2":0,"group":1,"y":1.3990937292060162},{"x":0.5797270385843651,"y2":0,"group":1,"y":1.403685788662469},{"x":0.58036895088653,"y2":0,"group":1,"y":1.4082997065705358},{"x":0.5810108631886945,"y2":0,"group":1,"y":1.4129350687037925},{"x":0.5816527754908594,"y2":0,"group":1,"y":1.4175914573758948},{"x":0.5822946877930242,"y2":0,"group":1,"y":1.4222684515033297},{"x":0.5829366000951888,"y2":0,"group":1,"y":1.4269656266686037},{"x":0.5835785123973536,"y2":0,"group":1,"y":1.4316825551838614},{"x":0.5842204246995184,"y2":0,"group":1,"y":1.4364188061549212},{"x":0.584862337001683,"y2":0,"group":1,"y":1.4411739455457164},{"x":0.5855042493038478,"y2":0,"group":1,"y":1.4459475362431284},{"x":0.5861461616060126,"y2":0,"group":1,"y":1.4507391381222043},{"x":0.5867880739081772,"y2":0,"group":1,"y":1.455548308111739},{"x":0.587429986210342,"y2":0,"group":1,"y":1.4603746002602223},{"x":0.5880718985125069,"y2":0,"group":1,"y":1.4652175658021207},{"x":0.5887138108146717,"y2":0,"group":1,"y":1.4700767532245038},{"x":0.5893557231168363,"y2":0,"group":1,"y":1.474951708333983},{"x":0.5899976354190011,"y2":0,"group":1,"y":1.479841974323964},{"x":0.5906395477211659,"y2":0,"group":1,"y":1.4847470918421921},{"x":0.5912814600233305,"y2":0,"group":1,"y":1.4896665990585851},{"x":0.5919233723254953,"y2":0,"group":1,"y":1.494600031733335},{"x":0.5925652846276601,"y2":0,"group":1,"y":1.4995469232852723},{"x":0.5932071969298247,"y2":0,"group":1,"y":1.504506804860474},{"x":0.5938491092319895,"y2":0,"group":1,"y":1.5094792054011092},{"x":0.5944910215341543,"y2":0,"group":1,"y":1.5144636517145038},{"x":0.5951329338363189,"y2":0,"group":1,"y":1.5194596685424193},{"x":0.5957748461384837,"y2":0,"group":1,"y":1.5244667786305242},{"x":0.5964167584406486,"y2":0,"group":1,"y":1.5294845027980524},{"x":0.5970586707428134,"y2":0,"group":1,"y":1.534512360007634},{"x":0.597700583044978,"y2":0,"group":1,"y":1.5395498674352859},{"x":0.5983424953471428,"y2":0,"group":1,"y":1.544596540540548},{"x":0.5989844076493076,"y2":0,"group":1,"y":1.549651893136755},{"x":0.5996263199514722,"y2":0,"group":1,"y":1.554715437461431},{"x":0.600268232253637,"y2":0,"group":1,"y":1.5597866842467902},{"x":0.6009101445558018,"y2":0,"group":1,"y":1.5648651427903406},{"x":0.6015520568579664,"y2":0,"group":1,"y":1.569950321025566},{"x":0.6021939691601312,"y2":0,"group":1,"y":1.5750417255926823},{"x":0.602835881462296,"y2":0,"group":1,"y":1.580138861909452},{"x":0.6034777937644606,"y2":0,"group":1,"y":1.585241234242049},{"x":0.6041197060666255,"y2":0,"group":1,"y":1.590348345775952},{"x":0.6047616183687903,"y2":0,"group":1,"y":1.5954596986868623},{"x":0.6054035306709551,"y2":0,"group":1,"y":1.6005747942116328},{"x":0.6060454429731197,"y2":0,"group":1,"y":1.6056931327191888},{"x":0.6066873552752845,"y2":0,"group":1,"y":1.6108142137814374},{"x":0.6073292675774493,"y2":0,"group":1,"y":1.6159375362441466},{"x":0.6079711798796139,"y2":0,"group":1,"y":1.621062598297784},{"x":0.6086130921817787,"y2":0,"group":1,"y":1.6261888975483},{"x":0.6092550044839435,"y2":0,"group":1,"y":1.631315931087854},{"x":0.6098969167861081,"y2":0,"group":1,"y":1.6364431955654497},{"x":0.610538829088273,"y2":0,"group":1,"y":1.6415701872574928},{"x":0.6111807413904378,"y2":0,"group":1,"y":1.64669640213824},{"x":0.6118226536926024,"y2":0,"group":1,"y":1.6518213359501324},{"x":0.6124645659947672,"y2":0,"group":1,"y":1.6569444842740089},{"x":0.613106478296932,"y2":0,"group":1,"y":1.662065342599174},{"x":0.6137483905990968,"y2":0,"group":1,"y":1.6671834063933177},{"x":0.6143903029012614,"y2":0,"group":1,"y":1.6722981711722742},{"x":0.6150322152034262,"y2":0,"group":1,"y":1.6774091325696014},{"x":0.615674127505591,"y2":0,"group":1,"y":1.6825157864059785},{"x":0.6163160398077556,"y2":0,"group":1,"y":1.6876176287584035},{"x":0.6169579521099204,"y2":0,"group":1,"y":1.692714156029178},{"x":0.6175998644120853,"y2":0,"group":1,"y":1.6978048650146735},{"x":0.6182417767142498,"y2":0,"group":1,"y":1.7028892529738617},{"x":0.6188836890164147,"y2":0,"group":1,"y":1.7079668176966014},{"x":0.6195256013185795,"y2":0,"group":1,"y":1.713037057571666},{"x":0.6201675136207441,"y2":0,"group":1,"y":1.718099471654506},{"x":0.6208094259229089,"y2":0,"group":1,"y":1.7231535597347292},{"x":0.6214513382250737,"y2":0,"group":1,"y":1.7281988224032905},{"x":0.6220932505272385,"y2":0,"group":1,"y":1.7332347611193826},{"x":0.6227351628294031,"y2":0,"group":1,"y":1.7382608782770068},{"x":0.6233770751315679,"y2":0,"group":1,"y":1.743276677271228},{"x":0.6240189874337327,"y2":0,"group":1,"y":1.7482816625640876},{"x":0.6246608997358973,"y2":0,"group":1,"y":1.753275339750171},{"x":0.6253028120380622,"y2":0,"group":1,"y":1.7582572156218244},{"x":0.625944724340227,"y2":0,"group":1,"y":1.7632267982339949},{"x":0.6265866366423916,"y2":0,"group":1,"y":1.7681835969686992},{"x":0.6272285489445564,"y2":0,"group":1,"y":1.7731271225990999},{"x":0.6278704612467212,"y2":0,"group":1,"y":1.7780568873531846},{"x":0.6285123735488858,"y2":0,"group":1,"y":1.782972404977034},{"x":0.6291542858510506,"y2":0,"group":1,"y":1.787873190797671},{"x":0.6297961981532154,"y2":0,"group":1,"y":1.7927587617854812},{"x":0.6304381104553802,"y2":0,"group":1,"y":1.7976286366161902},{"x":0.6310800227575448,"y2":0,"group":1,"y":1.8024823357323954},{"x":0.6317219350597096,"y2":0,"group":1,"y":1.807319381404636},{"x":0.6323638473618745,"y2":0,"group":1,"y":1.812139297791994},{"x":0.633005759664039,"y2":0,"group":1,"y":1.816941611002217},{"x":0.6336476719662039,"y2":0,"group":1,"y":1.8217258491513524},{"x":0.6342895842683687,"y2":0,"group":1,"y":1.826491542422886},{"x":0.6349314965705333,"y2":0,"group":1,"y":1.8312382231263724},{"x":0.6355734088726981,"y2":0,"group":1,"y":1.835965425755548},{"x":0.6362153211748629,"y2":0,"group":1,"y":1.8406726870459271},{"x":0.6368572334770275,"y2":0,"group":1,"y":1.8453595460318526},{"x":0.6374991457791923,"y2":0,"group":1,"y":1.8500255441030162},{"x":0.6381410580813571,"y2":0,"group":1,"y":1.8546702250604212},{"x":0.6387829703835219,"y2":0,"group":1,"y":1.859293135171789},{"x":0.6394248826856865,"y2":0,"group":1,"y":1.8638938232263984},{"x":0.6400667949878514,"y2":0,"group":1,"y":1.8684718405893475},{"x":0.6407087072900162,"y2":0,"group":1,"y":1.8730267412552344},{"x":0.6413506195921808,"y2":0,"group":1,"y":1.8775580819012458},{"x":0.6419925318943456,"y2":0,"group":1,"y":1.8820654219396475},{"x":0.6426344441965104,"y2":0,"group":1,"y":1.886548323569667},{"x":0.643276356498675,"y2":0,"group":1,"y":1.8910063518287619},{"x":0.6439182688008398,"y2":0,"group":1,"y":1.8954390746432686},{"x":0.6445601811030046,"y2":0,"group":1,"y":1.8998460628784224},{"x":0.6452020934051692,"y2":0,"group":1,"y":1.9042268903877422},{"x":0.645844005707334,"y2":0,"group":1,"y":1.9085811340617684},{"x":0.6464859180094988,"y2":0,"group":1,"y":1.9129083738761614},{"x":0.6471278303116637,"y2":0,"group":1,"y":1.9172081929391356},{"x":0.6477697426138282,"y2":0,"group":1,"y":1.921480177538235},{"x":0.6484116549159931,"y2":0,"group":1,"y":1.925723917186442},{"x":0.6490535672181579,"y2":0,"group":1,"y":1.9299390046676113},{"x":0.6496954795203225,"y2":0,"group":1,"y":1.9341250360812192},{"x":0.6503373918224873,"y2":0,"group":1,"y":1.9382816108864358},{"x":0.6509793041246521,"y2":0,"group":1,"y":1.9424083319454974},{"x":0.6516212164268167,"y2":0,"group":1,"y":1.946504805566387},{"x":0.6522631287289815,"y2":0,"group":1,"y":1.9505706415448105},{"x":0.6529050410311463,"y2":0,"group":1,"y":1.9546054532054662},{"x":0.6535469533333109,"y2":0,"group":1,"y":1.958608857442602},{"x":0.6541888656354757,"y2":0,"group":1,"y":1.9625804747598563},{"x":0.6548307779376406,"y2":0,"group":1,"y":1.9665199293093751},{"x":0.6554726902398054,"y2":0,"group":1,"y":1.9704268489302061},{"x":0.65611460254197,"y2":0,"group":1,"y":1.9743008651859584},{"x":0.6567565148441348,"y2":0,"group":1,"y":1.9781416134017333},{"x":0.6573984271462996,"y2":0,"group":1,"y":1.9819487327003102},{"x":0.6580403394484642,"y2":0,"group":1,"y":1.9857218660375957},{"x":0.658682251750629,"y2":0,"group":1,"y":1.9894606602373226},{"x":0.6593241640527938,"y2":0,"group":1,"y":1.993164766025004},{"x":0.6599660763549584,"y2":0,"group":1,"y":1.9968338380611304},{"x":0.6606079886571232,"y2":0,"group":1,"y":2.0004675349736143},{"x":0.661249900959288,"y2":0,"group":1,"y":2.004065519389475},{"x":0.6618918132614526,"y2":0,"group":1,"y":2.007627457965762},{"x":0.6625337255636174,"y2":0,"group":1,"y":2.011153021419714},{"x":0.6631756378657823,"y2":0,"group":1,"y":2.014641884558156},{"x":0.6638175501679471,"y2":0,"group":1,"y":2.0180937263061183},{"x":0.6644594624701117,"y2":0,"group":1,"y":2.021508229734695},{"x":0.6651013747722765,"y2":0,"group":1,"y":2.024885082088125},{"x":0.6657432870744413,"y2":0,"group":1,"y":2.0282239748100994},{"x":0.6663851993766059,"y2":0,"group":1,"y":2.0315246035692924},{"x":0.6670271116787707,"y2":0,"group":1,"y":2.034786668284119},{"x":0.6676690239809355,"y2":0,"group":1,"y":2.038009873146707},{"x":0.6683109362831001,"y2":0,"group":1,"y":2.0411939266460983},{"x":0.6689528485852649,"y2":0,"group":1,"y":2.044338541590664},{"x":0.6695947608874298,"y2":0,"group":1,"y":2.0474434351297415},{"x":0.6702366731895943,"y2":0,"group":1,"y":2.0505083287744874},{"x":0.6708785854917592,"y2":0,"group":1,"y":2.0535329484179536},{"x":0.671520497793924,"y2":0,"group":1,"y":2.056517024354377},{"x":0.6721624100960888,"y2":0,"group":1,"y":2.059460291297691},{"x":0.6728043223982534,"y2":0,"group":1,"y":2.0623624883992493},{"x":0.6734462347004182,"y2":0,"group":1,"y":2.065223359264779},{"x":0.674088147002583,"y2":0,"group":1,"y":2.0680426519705417},{"x":0.6747300593047476,"y2":0,"group":1,"y":2.070820119078723},{"x":0.6753719716069124,"y2":0,"group":1,"y":2.073555517652038},{"x":0.6760138839090772,"y2":0,"group":1,"y":2.0762486092675605},{"x":0.6766557962112418,"y2":0,"group":1,"y":2.0788991600297764},{"x":0.6772977085134066,"y2":0,"group":1,"y":2.0815069405828592},{"x":0.6779396208155715,"y2":0,"group":1,"y":2.0840717261221755},{"x":0.6785815331177361,"y2":0,"group":1,"y":2.0865932964050105},{"x":0.6792234454199009,"y2":0,"group":1,"y":2.0890714357605344},{"x":0.6798653577220657,"y2":0,"group":1,"y":2.0915059330989907},{"x":0.6805072700242305,"y2":0,"group":1,"y":2.093896581920124},{"x":0.6811491823263951,"y2":0,"group":1,"y":2.0962431803208412},{"x":0.6817910946285599,"y2":0,"group":1,"y":2.098545531002113},{"x":0.6824330069307247,"y2":0,"group":1,"y":2.1008034412751178},{"x":0.6830749192328893,"y2":0,"group":1,"y":2.1030167230666263},{"x":0.6837168315350541,"y2":0,"group":1,"y":2.105185192923639},{"x":0.684358743837219,"y2":0,"group":1,"y":2.1073086720172665},{"x":0.6850006561393835,"y2":0,"group":1,"y":2.1093869861458754},{"x":0.6856425684415484,"y2":0,"group":1,"y":2.111419965737477},{"x":0.6862844807437132,"y2":0,"group":1,"y":2.1134074458513874},{"x":0.6869263930458778,"y2":0,"group":1,"y":2.1153492661791513},{"x":0.6875683053480426,"y2":0,"group":1,"y":2.1172452710447294},{"x":0.6882102176502074,"y2":0,"group":1,"y":2.1190953094039617},{"x":0.6888521299523722,"y2":0,"group":1,"y":2.1208992348433093},{"x":0.6894940422545368,"y2":0,"group":1,"y":2.1226569055778737},{"x":0.6901359545567016,"y2":0,"group":1,"y":2.1243681844487057},{"x":0.6907778668588664,"y2":0,"group":1,"y":2.126032938919402},{"x":0.691419779161031,"y2":0,"group":1,"y":2.1276510410719975},{"x":0.6920616914631958,"y2":0,"group":1,"y":2.1292223676021584},{"x":0.6927036037653607,"y2":0,"group":1,"y":2.130746799813681},{"x":0.6933455160675253,"y2":0,"group":1,"y":2.1322242236123},{"x":0.6939874283696901,"y2":0,"group":1,"y":2.1336545294988145},{"x":0.6946293406718549,"y2":0,"group":1,"y":2.135037612561528},{"x":0.6952712529740195,"y2":0,"group":1,"y":2.136373372468027},{"x":0.6959131652761843,"y2":0,"group":1,"y":2.1376617134562834},{"x":0.6965550775783491,"y2":0,"group":1,"y":2.1389025443250977},{"x":0.6971969898805139,"y2":0,"group":1,"y":2.140095778423885},{"x":0.6978389021826785,"y2":0,"group":1,"y":2.1412413336418172},{"x":0.6984808144848433,"y2":0,"group":1,"y":2.1423391323963137},{"x":0.6991227267870082,"y2":0,"group":1,"y":2.1433891016209},{"x":0.6997646390891727,"y2":0,"group":1,"y":2.144391172752435},{"x":0.7004065513913376,"y2":0,"group":1,"y":2.1453452817177157},{"x":0.7010484636935024,"y2":0,"group":1,"y":2.146251368919458},{"x":0.701690375995667,"y2":0,"group":1,"y":2.1471093792216775},{"x":0.7023322882978318,"y2":0,"group":1,"y":2.1479192619344554},{"x":0.7029742005999966,"y2":0,"group":1,"y":2.148680970798116},{"x":0.7036161129021612,"y2":0,"group":1,"y":2.14939446396681},{"x":0.704258025204326,"y2":0,"group":1,"y":2.1500597039915186},{"x":0.7048999375064908,"y2":0,"group":1,"y":2.1506766578024763},{"x":0.7055418498086556,"y2":0,"group":1,"y":2.1512452966910347},{"x":0.7061837621108202,"y2":0,"group":1,"y":2.151765596290958},{"x":0.706825674412985,"y2":0,"group":1,"y":2.1522375365591686},{"x":0.7074675867151499,"y2":0,"group":1,"y":2.152661101755951},{"x":0.7081094990173145,"y2":0,"group":1,"y":2.1530362804246095},{"x":0.7087514113194793,"y2":0,"group":1,"y":2.1533630653706055},{"x":0.7093933236216441,"y2":0,"group":1,"y":2.1536414536401685},{"x":0.7100352359238087,"y2":0,"group":1,"y":2.1538714464983917},{"x":0.7106771482259735,"y2":0,"group":1,"y":2.154053049406825},{"x":0.7113190605281383,"y2":0,"group":1,"y":2.1541862720005693},{"x":0.7119609728303029,"y2":0,"group":1,"y":2.154271128064881},{"x":0.7126028851324677,"y2":0,"group":1,"y":2.1543076355112922},{"x":0.7132447974346325,"y2":0,"group":1,"y":2.1542958163532653},{"x":0.7138867097367974,"y2":0,"group":1,"y":2.1542356966813756},{"x":0.714528622038962,"y2":0,"group":1,"y":2.154127306638041},{"x":0.7151705343411268,"y2":0,"group":1,"y":2.1539706803918035},{"x":0.7158124466432916,"y2":0,"group":1,"y":2.153765856111171},{"x":0.7164543589454562,"y2":0,"group":1,"y":2.153512875938029},{"x":0.717096271247621,"y2":0,"group":1,"y":2.153211785960631},{"x":0.7177381835497858,"y2":0,"group":1,"y":2.1528626361861725},{"x":0.7183800958519504,"y2":0,"group":1,"y":2.1524654805129657},{"x":0.7190220081541152,"y2":0,"group":1,"y":2.1520203767022115},{"x":0.71966392045628,"y2":0,"group":1,"y":2.1515273863493913},{"x":0.7203058327584446,"y2":0,"group":1,"y":2.150986574855274},{"x":0.7209477450606094,"y2":0,"group":1,"y":2.1503980113965615},{"x":0.7215896573627743,"y2":0,"group":1,"y":2.1497617688961643},{"x":0.7222315696649391,"y2":0,"group":1,"y":2.149077923993133},{"x":0.7228734819671037,"y2":0,"group":1,"y":2.1483465570122426},{"x":0.7235153942692685,"y2":0,"group":1,"y":2.1475677519332486},{"x":0.7241573065714333,"y2":0,"group":1,"y":2.1467415963598113},{"x":0.7247992188735979,"y2":0,"group":1,"y":2.1458681814881087},{"x":0.7254411311757627,"y2":0,"group":1,"y":2.144947602075142},{"x":0.7260830434779275,"y2":0,"group":1,"y":2.1439799564067425},{"x":0.7267249557800921,"y2":0,"group":1,"y":2.142965346265293},{"x":0.7273668680822569,"y2":0,"group":1,"y":2.1419038768971643},{"x":0.7280087803844217,"y2":0,"group":1,"y":2.1407956569798845},{"x":0.7286506926865863,"y2":0,"group":1,"y":2.1396407985890455},{"x":0.7292926049887511,"y2":0,"group":1,"y":2.1384394171649603},{"x":0.729934517290916,"y2":0,"group":1,"y":2.1371916314790713},{"x":0.7305764295930808,"y2":0,"group":1,"y":2.135897563600126},{"x":0.7312183418952454,"y2":0,"group":1,"y":2.1345573388601298},{"x":0.7318602541974102,"y2":0,"group":1,"y":2.1331710858200794},{"x":0.732502166499575,"y2":0,"group":1,"y":2.13173893623549},{"x":0.7331440788017396,"y2":0,"group":1,"y":2.1302610250217207},{"x":0.7337859911039044,"y2":0,"group":1,"y":2.1287374902191183},{"x":0.7344279034060692,"y2":0,"group":1,"y":2.127168472957969},{"x":0.7350698157082338,"y2":0,"group":1,"y":2.1255541174232864},{"x":0.7357117280103986,"y2":0,"group":1,"y":2.123894570819435},{"x":0.7363536403125635,"y2":0,"group":1,"y":2.122189983334596},{"x":0.736995552614728,"y2":0,"group":1,"y":2.1204405081050934},{"x":0.7376374649168929,"y2":0,"group":1,"y":2.1186463011795764},{"x":0.7382793772190577,"y2":0,"group":1,"y":2.1168075214830786},{"x":0.7389212895212225,"y2":0,"group":1,"y":2.11492433078096},{"x":0.7395632018233871,"y2":0,"group":1,"y":2.112996893642728},{"x":0.7402051141255519,"y2":0,"group":1,"y":2.111025377405771},{"x":0.7408470264277167,"y2":0,"group":1,"y":2.109009952138983},{"x":0.7414889387298813,"y2":0,"group":1,"y":2.106950790606317},{"x":0.7421308510320461,"y2":0,"group":1,"y":2.104848068230252},{"x":0.7427727633342109,"y2":0,"group":1,"y":2.102701963055197},{"x":0.7434146756363755,"y2":0,"group":1,"y":2.100512655710828},{"x":0.7440565879385403,"y2":0,"group":1,"y":2.098280329375386},{"x":0.7446985002407052,"y2":0,"group":1,"y":2.096005169738916},{"x":0.7453404125428698,"y2":0,"group":1,"y":2.093687364966485},{"x":0.7459823248450346,"y2":0,"group":1,"y":2.09132710566136},{"x":0.7466242371471994,"y2":0,"group":1,"y":2.0889245848281752},{"x":0.7472661494493642,"y2":0,"group":1,"y":2.086479997836086},{"x":0.7479080617515288,"y2":0,"group":1,"y":2.083993542381917},{"x":0.7485499740536936,"y2":0,"group":1,"y":2.0814654184533157},{"x":0.7491918863558584,"y2":0,"group":1,"y":2.0788958282919183},{"x":0.749833798658023,"y2":0,"group":1,"y":2.076284976356534},{"x":0.7504757109601878,"y2":0,"group":1,"y":2.07363306928636},{"x":0.7511176232623527,"y2":0,"group":1,"y":2.070940315864225},{"x":0.7517595355645172,"y2":0,"group":1,"y":2.0682069269798795},{"x":0.7524014478666821,"y2":0,"group":1,"y":2.0654331155933336},{"x":0.7530433601688469,"y2":0,"group":1,"y":2.0626190966982527},{"x":0.7536852724710115,"y2":0,"group":1,"y":2.0597650872854123},{"x":0.7543271847731763,"y2":0,"group":1,"y":2.056871306306231},{"x":0.7549690970753411,"y2":0,"group":1,"y":2.0539379746363737},{"x":0.7556110093775059,"y2":0,"group":1,"y":2.0509653150394445},{"x":0.7562529216796705,"y2":0,"group":1,"y":2.0479535521307675},{"x":0.7568948339818353,"y2":0,"group":1,"y":2.0449029123412674},{"x":0.7575367462840001,"y2":0,"group":1,"y":2.0418136238814544},{"x":0.7581786585861647,"y2":0,"group":1,"y":2.0386859167055187},{"x":0.7588205708883295,"y2":0,"group":1,"y":2.035520022475545},{"x":0.7594624831904944,"y2":0,"group":1,"y":2.032316174525846},{"x":0.760104395492659,"y2":0,"group":1,"y":2.0290746078274324},{"x":0.7607463077948238,"y2":0,"group":1,"y":2.025795558952612},{"x":0.7613882200969886,"y2":0,"group":1,"y":2.0224792660397357},{"x":0.7620301323991532,"y2":0,"group":1,"y":2.019125968758086},{"x":0.762672044701318,"y2":0,"group":1,"y":2.015735908272927},{"x":0.7633139570034828,"y2":0,"group":1,"y":2.012309327210705},{"x":0.7639558693056476,"y2":0,"group":1,"y":2.0088464696244164},{"x":0.7645977816078122,"y2":0,"group":1,"y":2.005347580959151},{"x":0.765239693909977,"y2":0,"group":1,"y":2.001812908017802},{"x":0.7658816062121419,"y2":0,"group":1,"y":1.9982426989269628},{"x":0.7665235185143064,"y2":0,"group":1,"y":1.9946372031030082},{"x":0.7671654308164713,"y2":0,"group":1,"y":1.9909966712183642},{"x":0.7678073431186361,"y2":0,"group":1,"y":1.9873213551679751},{"x":0.7684492554208007,"y2":0,"group":1,"y":1.9836115080359717},{"x":0.7690911677229655,"y2":0,"group":1,"y":1.9798673840625445},{"x":0.7697330800251303,"y2":0,"group":1,"y":1.9760892386110214},{"x":0.7703749923272949,"y2":0,"group":1,"y":1.972277328135167},{"x":0.7710169046294597,"y2":0,"group":1,"y":1.9684319101466954},{"x":0.7716588169316245,"y2":0,"group":1,"y":1.9645532431830066},{"x":0.7723007292337893,"y2":0,"group":1,"y":1.9606415867751479},{"x":0.7729426415359539,"y2":0,"group":1,"y":1.9566972014160102},{"x":0.7735845538381187,"y2":0,"group":1,"y":1.9527203485287514},{"x":0.7742264661402836,"y2":0,"group":1,"y":1.9487112904354658},{"x":0.7748683784424482,"y2":0,"group":1,"y":1.9446702903260875},{"x":0.775510290744613,"y2":0,"group":1,"y":1.9405976122275463},{"x":0.7761522030467778,"y2":0,"group":1,"y":1.9364935209731646},{"x":0.7767941153489424,"y2":0,"group":1,"y":1.932358282172312},{"x":0.7774360276511072,"y2":0,"group":1,"y":1.9281921621803115},{"x":0.778077939953272,"y2":0,"group":1,"y":1.9239954280686047},{"x":0.7787198522554366,"y2":0,"group":1,"y":1.9197683475951768},{"x":0.7793617645576014,"y2":0,"group":1,"y":1.9155111891752457},{"x":0.7800036768597662,"y2":0,"group":1,"y":1.9112242218522173},{"x":0.780645589161931,"y2":0,"group":1,"y":1.9069077152689102},{"x":0.7812875014640956,"y2":0,"group":1,"y":1.9025619396390492},{"x":0.7819294137662605,"y2":0,"group":1,"y":1.898187165719035},{"x":0.7825713260684253,"y2":0,"group":1,"y":1.8937836647799866},{"x":0.7832132383705899,"y2":0,"group":1,"y":1.8893517085800648},{"x":0.7838551506727547,"y2":0,"group":1,"y":1.8848915693370718},{"x":0.7844970629749195,"y2":0,"group":1,"y":1.8804035197013353},{"x":0.7851389752770841,"y2":0,"group":1,"y":1.8758878327288737},{"x":0.7857808875792489,"y2":0,"group":1,"y":1.8713447818548468},{"x":0.7864227998814137,"y2":0,"group":1,"y":1.8667746408672934},{"x":0.7870647121835785,"y2":0,"group":1,"y":1.8621776838811566},{"x":0.7877066244857431,"y2":0,"group":1,"y":1.8575541853125945},{"x":0.788348536787908,"y2":0,"group":1,"y":1.8529044198535902},{"x":0.7889904490900728,"y2":0,"group":1,"y":1.8482286624468391},{"x":0.7896323613922374,"y2":0,"group":1,"y":1.8435271882609445},{"x":0.7902742736944022,"y2":0,"group":1,"y":1.83880027266589},{"x":0.790916185996567,"y2":0,"group":1,"y":1.8340481912088205},{"x":0.7915580982987316,"y2":0,"group":1,"y":1.829271219590105},{"x":0.7922000106008964,"y2":0,"group":1,"y":1.8244696336397028},{"x":0.7928419229030612,"y2":0,"group":1,"y":1.8196437092938214},{"x":0.7934838352052258,"y2":0,"group":1,"y":1.814793722571869},{"x":0.7941257475073906,"y2":0,"group":1,"y":1.8099199495537077},{"x":0.7947676598095554,"y2":0,"group":1,"y":1.805022666357193},{"x":0.7954095721117203,"y2":0,"group":1,"y":1.8001021491160218},{"x":0.7960514844138848,"y2":0,"group":1,"y":1.795158673957866},{"x":0.7966933967160497,"y2":0,"group":1,"y":1.790192516982804},{"x":0.7973353090182145,"y2":0,"group":1,"y":1.7852039542420515},{"x":0.7979772213203791,"y2":0,"group":1,"y":1.7801932617169816},{"x":0.7986191336225439,"y2":0,"group":1,"y":1.7751607152984448},{"x":0.7992610459247087,"y2":0,"group":1,"y":1.770106590766375},{"x":0.7999029582268733,"y2":0,"group":1,"y":1.7650311637696983},{"x":0.8005448705290381,"y2":0,"group":1,"y":1.7599347098065246},{"x":0.8011867828312029,"y2":0,"group":1,"y":1.754817504204639},{"x":0.8018286951333675,"y2":0,"group":1,"y":1.7496798221022762},{"x":0.8024706074355323,"y2":0,"group":1,"y":1.7445219384291901},{"x":0.8031125197376972,"y2":0,"group":1,"y":1.7393441278880077},{"x":0.803754432039862,"y2":0,"group":1,"y":1.7341466649358737},{"x":0.8043963443420266,"y2":0,"group":1,"y":1.7289298237663768},{"x":0.8050382566441914,"y2":0,"group":1,"y":1.723693878291763},{"x":0.8056801689463562,"y2":0,"group":1,"y":1.718439102125433},{"x":0.8063220812485208,"y2":0,"group":1,"y":1.7131657685647173},{"x":0.8069639935506856,"y2":0,"group":1,"y":1.707874150573934},{"x":0.8076059058528504,"y2":0,"group":1,"y":1.7025645207677234},{"x":0.808247818155015,"y2":0,"group":1,"y":1.6972371513946602},{"x":0.8088897304571798,"y2":0,"group":1,"y":1.6918923143211382},{"x":0.8095316427593446,"y2":0,"group":1,"y":1.6865302810155296},{"x":0.8101735550615092,"y2":0,"group":1,"y":1.681151322532614},{"x":0.810815467363674,"y2":0,"group":1,"y":1.6757557094982758},{"x":0.8114573796658389,"y2":0,"group":1,"y":1.6703437120944677},{"x":0.8120992919680037,"y2":0,"group":1,"y":1.6649156000444416},{"x":0.8127412042701683,"y2":0,"group":1,"y":1.659471642598235},{"x":0.8133831165723331,"y2":0,"group":1,"y":1.654012108518425},{"x":0.8140250288744979,"y2":0,"group":1,"y":1.648537266066134},{"x":0.8146669411766625,"y2":0,"group":1,"y":1.6430473829872931},{"x":0.8153088534788273,"y2":0,"group":1,"y":1.6375427264991569},{"x":0.8159507657809921,"y2":0,"group":1,"y":1.6320235632770688},{"x":0.8165926780831567,"y2":0,"group":1,"y":1.6264901594414736},{"x":0.8172345903853215,"y2":0,"group":1,"y":1.6209427805451753},{"x":0.8178765026874864,"y2":0,"group":1,"y":1.615381691560836},{"x":0.8185184149896509,"y2":0,"group":1,"y":1.609807156868717},{"x":0.8191603272918158,"y2":0,"group":1,"y":1.6042194402446501},{"x":0.8198022395939806,"y2":0,"group":1,"y":1.5986188048482526},{"x":0.8204441518961454,"y2":0,"group":1,"y":1.5930055132113647},{"x":0.82108606419831,"y2":0,"group":1,"y":1.5873798272267177},{"x":0.8217279765004748,"y2":0,"group":1,"y":1.5817420081368294},{"x":0.8223698888026396,"y2":0,"group":1,"y":1.576092316523121},{"x":0.8230118011048042,"y2":0,"group":1,"y":1.5704310122952507},{"x":0.823653713406969,"y2":0,"group":1,"y":1.564758354680665},{"x":0.8242956257091338,"y2":0,"group":1,"y":1.5590746022143658},{"x":0.8249375380112984,"y2":0,"group":1,"y":1.553380012728883},{"x":0.8255794503134632,"y2":0,"group":1,"y":1.5476748433444587},{"x":0.8262213626156281,"y2":0,"group":1,"y":1.541959350459429},{"x":0.8268632749177927,"y2":0,"group":1,"y":1.5362337897408147},{"x":0.8275051872199575,"y2":0,"group":1,"y":1.5304984161151025},{"x":0.8281470995221223,"y2":0,"group":1,"y":1.5247534837592231},{"x":0.8287890118242871,"y2":0,"group":1,"y":1.518999246091722},{"x":0.8294309241264517,"y2":0,"group":1,"y":1.513235955764114},{"x":0.8300728364286165,"y2":0,"group":1,"y":1.507463864652427},{"x":0.8307147487307813,"y2":0,"group":1,"y":1.5016832238489222},{"x":0.8313566610329459,"y2":0,"group":1,"y":1.4958942836539988},{"x":0.8319985733351107,"y2":0,"group":1,"y":1.4900972935682646},{"x":0.8326404856372756,"y2":0,"group":1,"y":1.4842925022847875},{"x":0.8332823979394401,"y2":0,"group":1,"y":1.4784801576815079},{"x":0.833924310241605,"y2":0,"group":1,"y":1.4726605068138197},{"x":0.8345662225437698,"y2":0,"group":1,"y":1.4668337959073101},{"x":0.8352081348459344,"y2":0,"group":1,"y":1.4610002703506624},{"x":0.8358500471480992,"y2":0,"group":1,"y":1.4551601746887075},{"x":0.836491959450264,"y2":0,"group":1,"y":1.4493137526156323},{"x":0.8371338717524288,"y2":0,"group":1,"y":1.4434612469683323},{"x":0.8377757840545934,"y2":0,"group":1,"y":1.437602899719912},{"x":0.8384176963567582,"y2":0,"group":1,"y":1.4317389519733268},{"x":0.839059608658923,"y2":0,"group":1,"y":1.4258696439551606},{"x":0.8397015209610876,"y2":0,"group":1,"y":1.419995215009543},{"x":0.8403434332632524,"y2":0,"group":1,"y":1.4141159035921942},{"x":0.8409853455654173,"y2":0,"group":1,"y":1.4082319472645997},{"x":0.8416272578675819,"y2":0,"group":1,"y":1.4023435826883126},{"x":0.8422691701697467,"y2":0,"group":1,"y":1.3964510456193768},{"x":0.8429110824719115,"y2":0,"group":1,"y":1.3905545709028662},{"x":0.8435529947740761,"y2":0,"group":1,"y":1.384654392467545},{"x":0.8441949070762409,"y2":0,"group":1,"y":1.3787507433206363},{"x":0.8448368193784057,"y2":0,"group":1,"y":1.372843855542703},{"x":0.8454787316805705,"y2":0,"group":1,"y":1.366933960282633},{"x":0.8461206439827351,"y2":0,"group":1,"y":1.3610212877527295},{"x":0.8467625562848999,"y2":0,"group":1,"y":1.3551060672238993},{"x":0.8474044685870648,"y2":0,"group":1,"y":1.3491885270209423},{"x":0.8480463808892293,"y2":0,"group":1,"y":1.3432688945179292},{"x":0.8486882931913942,"y2":0,"group":1,"y":1.3373473961336755},{"x":0.849330205493559,"y2":0,"group":1,"y":1.331424257327302},{"x":0.8499721177957236,"y2":0,"group":1,"y":1.3254997025938802},{"x":0.8506140300978884,"y2":0,"group":1,"y":1.3195739554601595},{"x":0.8512559424000532,"y2":0,"group":1,"y":1.3136472384803766},{"x":0.8518978547022178,"y2":0,"group":1,"y":1.3077197732321377},{"x":0.8525397670043826,"y2":0,"group":1,"y":1.3017917803123795},{"x":0.8531816793065474,"y2":0,"group":1,"y":1.2958634793333963},{"x":0.8538235916087122,"y2":0,"group":1,"y":1.2899350889189387},{"x":0.8544655039108768,"y2":0,"group":1,"y":1.284006826700379},{"x":0.8551074162130416,"y2":0,"group":1,"y":1.2780789093129348},{"x":0.8557493285152065,"y2":0,"group":1,"y":1.2721515523919595},{"x":0.8563912408173711,"y2":0,"group":1,"y":1.2662249705692883},{"x":0.8570331531195359,"y2":0,"group":1,"y":1.2602993774696354},{"x":0.8576750654217007,"y2":0,"group":1,"y":1.2543749857070536},{"x":0.8583169777238653,"y2":0,"group":1,"y":1.2484520068814342},{"x":0.8589588900260301,"y2":0,"group":1,"y":1.2425306515750634},{"x":0.8596008023281949,"y2":0,"group":1,"y":1.2366111293492201},{"x":0.8602427146303595,"y2":0,"group":1,"y":1.2306936487408193},{"x":0.8608846269325243,"y2":0,"group":1,"y":1.2247784172590954},{"x":0.8615265392346891,"y2":0,"group":1,"y":1.2188656413823258},{"x":0.862168451536854,"y2":0,"group":1,"y":1.2129555265545924},{"x":0.8628103638390185,"y2":0,"group":1,"y":1.2070482771825763},{"x":0.8634522761411834,"y2":0,"group":1,"y":1.2011440966323863},{"x":0.8640941884433482,"y2":0,"group":1,"y":1.1952431872264186},{"x":0.8647361007455128,"y2":0,"group":1,"y":1.1893457502402458},{"x":0.8653780130476776,"y2":0,"group":1,"y":1.1834519858995327},{"x":0.8660199253498424,"y2":0,"group":1,"y":1.1775620933769761},{"x":0.866661837652007,"y2":0,"group":1,"y":1.1716762707892707},{"x":0.8673037499541718,"y2":0,"group":1,"y":1.1657947151940953},{"x":0.8679456622563366,"y2":0,"group":1,"y":1.15991762258712},{"x":0.8685875745585012,"y2":0,"group":1,"y":1.1540451878990299},{"x":0.869229486860666,"y2":0,"group":1,"y":1.1481776049925674},{"x":0.8698713991628308,"y2":0,"group":1,"y":1.1423150666595925},{"x":0.8705133114649957,"y2":0,"group":1,"y":1.13645776461815},{"x":0.8711552237671603,"y2":0,"group":1,"y":1.1306058895095574},{"x":0.8717971360693251,"y2":0,"group":1,"y":1.1247596308954972},{"x":0.8724390483714899,"y2":0,"group":1,"y":1.1189191772551248},{"x":0.8730809606736545,"y2":0,"group":1,"y":1.1130847159821804},{"x":0.8737228729758193,"y2":0,"group":1,"y":1.1072564333821104},{"x":0.8743647852779841,"y2":0,"group":1,"y":1.1014345146691977},{"x":0.8750066975801487,"y2":0,"group":1,"y":1.0956191439636909},{"x":0.8756486098823135,"y2":0,"group":1,"y":1.0898105042889448},{"x":0.8762905221844783,"y2":0,"group":1,"y":1.0840087775685592},{"x":0.8769324344866429,"y2":0,"group":1,"y":1.0782141446235225},{"x":0.8775743467888077,"y2":0,"group":1,"y":1.0724267851693563},{"x":0.8782162590909726,"y2":0,"group":1,"y":1.0666468778132594},{"x":0.8788581713931374,"y2":0,"group":1,"y":1.0608746000512537},{"x":0.879500083695302,"y2":0,"group":1,"y":1.0551101282653303},{"x":0.8801419959974668,"y2":0,"group":1,"y":1.0493536377205894},{"x":0.8807839082996316,"y2":0,"group":1,"y":1.0436053025623828},{"x":0.8814258206017962,"y2":0,"group":1,"y":1.0378652958134527},{"x":0.882067732903961,"y2":0,"group":1,"y":1.0321337893710685},{"x":0.8827096452061258,"y2":0,"group":1,"y":1.026410954004155},{"x":0.8833515575082904,"y2":0,"group":1,"y":1.0206969593504258},{"x":0.8839934698104552,"y2":0,"group":1,"y":1.0149919739135045},{"x":0.88463538211262,"y2":0,"group":1,"y":1.0092961650600456},{"x":0.8852772944147846,"y2":0,"group":1,"y":1.00360969901685},{"x":0.8859192067169495,"y2":0,"group":1,"y":0.9979327408679757},{"x":0.8865611190191143,"y2":0,"group":1,"y":0.9922654545518442},{"x":0.8872030313212791,"y2":0,"group":1,"y":0.9866080028583408},{"x":0.8878449436234437,"y2":0,"group":1,"y":0.9809605474259109},{"x":0.8884868559256085,"y2":0,"group":1,"y":0.9753232487386514},{"x":0.8891287682277733,"y2":0,"group":1,"y":0.9696962661233978},{"x":0.8897706805299379,"y2":0,"group":1,"y":0.9640797577468042},{"x":0.8904125928321027,"y2":0,"group":1,"y":0.9584738806124231},{"x":0.8910545051342675,"y2":0,"group":1,"y":0.9528787905577767},{"x":0.8916964174364321,"y2":0,"group":1,"y":0.9472946422514269},{"x":0.892338329738597,"y2":0,"group":1,"y":0.9417215891900412},{"x":0.8929802420407618,"y2":0,"group":1,"y":0.9361597836954537},{"x":0.8936221543429264,"y2":0,"group":1,"y":0.9306093769117268},{"x":0.8942640666450912,"y2":0,"group":1,"y":0.9250705188022068},{"x":0.894905978947256,"y2":0,"group":1,"y":0.919543358146579},{"x":0.8955478912494208,"y2":0,"group":1,"y":0.9140280425379232},{"x":0.8961898035515854,"y2":0,"group":1,"y":0.9085247183797656},{"x":0.8968317158537502,"y2":0,"group":1,"y":0.9030335308831328},{"x":0.897473628155915,"y2":0,"group":1,"y":0.8975546240636048},{"x":0.8981155404580796,"y2":0,"group":1,"y":0.8920881407383713},{"x":0.8987574527602444,"y2":0,"group":1,"y":0.8866342225232873},{"x":0.8993993650624093,"y2":0,"group":1,"y":0.881193009829934},{"x":0.9000412773645738,"y2":0,"group":1,"y":0.8757646418626829},{"x":0.9006831896667387,"y2":0,"group":1,"y":0.8703492566157619},{"x":0.9013251019689035,"y2":0,"group":1,"y":0.8649469908703311},{"x":0.9019670142710681,"y2":0,"group":1,"y":0.8595579801915607},{"x":0.9026089265732329,"y2":0,"group":1,"y":0.8541823589257207},{"x":0.9032508388753977,"y2":0,"group":1,"y":0.8488202601972737},{"x":0.9038927511775625,"y2":0,"group":1,"y":0.843471815905981},{"x":0.9045346634797271,"y2":0,"group":1,"y":0.8381371567240182},{"x":0.9051765757818919,"y2":0,"group":1,"y":0.8328164120931021},{"x":0.9058184880840567,"y2":0,"group":1,"y":0.8275097102216286},{"x":0.9064604003862213,"y2":0,"group":1,"y":0.8222171780818275},{"x":0.9071023126883861,"y2":0,"group":1,"y":0.8169389414069306},{"x":0.907744224990551,"y2":0,"group":1,"y":0.8116751246883537},{"x":0.9083861372927156,"y2":0,"group":1,"y":0.8064258511729032},{"x":0.9090280495948804,"y2":0,"group":1,"y":0.801191242859991},{"x":0.9096699618970452,"y2":0,"group":1,"y":0.7959714204988783},{"x":0.9103118741992098,"y2":0,"group":1,"y":0.7907665035859361},{"x":0.9109537865013746,"y2":0,"group":1,"y":0.785576610361929},{"x":0.9115956988035394,"y2":0,"group":1,"y":0.780401857809324},{"x":0.9122376111057042,"y2":0,"group":1,"y":0.7752423616496237},{"x":0.9128795234078688,"y2":0,"group":1,"y":0.770098236340725},{"x":0.9135214357100336,"y2":0,"group":1,"y":0.7649695950743084},{"x":0.9141633480121985,"y2":0,"group":1,"y":0.7598565497732559},{"x":0.914805260314363,"y2":0,"group":1,"y":0.7547592110890977},{"x":0.9154471726165279,"y2":0,"group":1,"y":0.7496776883994938},{"x":0.9160890849186927,"y2":0,"group":1,"y":0.7446120898057491},{"x":0.9167309972208573,"y2":0,"group":1,"y":0.739562522130362},{"x":0.9173729095230221,"y2":0,"group":1,"y":0.734529090914612},{"x":0.9180148218251869,"y2":0,"group":1,"y":0.7295119004161839},{"x":0.9186567341273515,"y2":0,"group":1,"y":0.7245110536068329},{"x":0.9192986464295163,"y2":0,"group":1,"y":0.7195266521700896},{"x":0.9199405587316811,"y2":0,"group":1,"y":0.7145587964990097},{"x":0.9205824710338459,"y2":0,"group":1,"y":0.7096075856939668},{"x":0.9212243833360105,"y2":0,"group":1,"y":0.7046731175604906},{"x":0.9218662956381753,"y2":0,"group":1,"y":0.699755488607154},{"x":0.9225082079403402,"y2":0,"group":1,"y":0.6948547940435069},{"x":0.9231501202425048,"y2":0,"group":1,"y":0.6899711277780651},{"x":0.9237920325446696,"y2":0,"group":1,"y":0.6851045824163433},{"x":0.9244339448468344,"y2":0,"group":1,"y":0.6802552492589502},{"x":0.925075857148999,"y2":0,"group":1,"y":0.675423218299734},{"x":0.9257177694511638,"y2":0,"group":1,"y":0.6706085782239838},{"x":0.9263596817533286,"y2":0,"group":1,"y":0.665811416406693},{"x":0.9270015940554932,"y2":0,"group":1,"y":0.6610318189108793},{"x":0.927643506357658,"y2":0,"group":1,"y":0.6562698704859666},{"x":0.9282854186598228,"y2":0,"group":1,"y":0.6515256545662316},{"x":0.9289273309619877,"y2":0,"group":1,"y":0.6467992532693105},{"x":0.9295692432641522,"y2":0,"group":1,"y":0.6420907473947769},{"x":0.9302111555663171,"y2":0,"group":1,"y":0.6374002164227837},{"x":0.9308530678684819,"y2":0,"group":1,"y":0.6327277385127731},{"x":0.9314949801706465,"y2":0,"group":1,"y":0.6280733905022597},{"x":0.9321368924728113,"y2":0,"group":1,"y":0.6234372479056839},{"x":0.9327788047749761,"y2":0,"group":1,"y":0.6188193849133381},{"x":0.9334207170771407,"y2":0,"group":1,"y":0.6142198743903686},{"x":0.9340626293793055,"y2":0,"group":1,"y":0.6096387878758536},{"x":0.9347045416814703,"y2":0,"group":1,"y":0.6050761955819579},{"x":0.9353464539836349,"y2":0,"group":1,"y":0.6005321663931671},{"x":0.9359883662857997,"y2":0,"group":1,"y":0.5960067678656027},{"x":0.9366302785879645,"y2":0,"group":1,"y":0.5915000662264185},{"x":0.9372721908901294,"y2":0,"group":1,"y":0.5870121263732797},{"x":0.937914103192294,"y2":0,"group":1,"y":0.5825430118739274},{"x":0.9385560154944588,"y2":0,"group":1,"y":0.5780927849658288},{"x":0.9391979277966236,"y2":0,"group":1,"y":0.5736615065559121},{"x":0.9398398400987882,"y2":0,"group":1,"y":0.5692492362203941},{"x":0.940481752400953,"y2":0,"group":1,"y":0.5648560322046943},{"x":0.9411236647031178,"y2":0,"group":1,"y":0.5604819514234403},{"x":0.9417655770052824,"y2":0,"group":1,"y":0.5561270494605662},{"x":0.9424074893074472,"y2":0,"group":1,"y":0.5517913805695037},{"x":0.943049401609612,"y2":0,"group":1,"y":0.5474749976734674},{"x":0.9436913139117766,"y2":0,"group":1,"y":0.5431779523658365},{"x":0.9443332262139414,"y2":0,"group":1,"y":0.5389002949106322},{"x":0.9449751385161063,"y2":0,"group":1,"y":0.5346420742430948},{"x":0.9456170508182711,"y2":0,"group":1,"y":0.5304033379703563},{"x":0.9462589631204357,"y2":0,"group":1,"y":0.5261841323722172},{"x":0.9469008754226005,"y2":0,"group":1,"y":0.5219845024020202},{"x":0.9475427877247653,"y2":0,"group":1,"y":0.5178044916876285},{"x":0.9481847000269299,"y2":0,"group":1,"y":0.5136441425325065},{"x":0.9488266123290947,"y2":0,"group":1,"y":0.5095034959169031},{"x":0.9494685246312595,"y2":0,"group":1,"y":0.5053825914991412},{"x":0.9501104369334241,"y2":0,"group":1,"y":0.5012814676170116},{"x":0.9507523492355889,"y2":0,"group":1,"y":0.4972001612892738},{"x":0.9513942615377537,"y2":0,"group":1,"y":0.49313870821726413},{"x":0.9520361738399183,"y2":0,"group":1,"y":0.4890971427866124},{"x":0.9526780861420832,"y2":0,"group":1,"y":0.48507549806906514},{"x":0.953319998444248,"y2":0,"group":1,"y":0.4810738058244203},{"x":0.9539619107464128,"y2":0,"group":1,"y":0.4770920965025724},{"x":0.9546038230485774,"y2":0,"group":1,"y":0.47313039924566613},{"x":0.9552457353507422,"y2":0,"group":1,"y":0.46918874189036236},{"x":0.955887647652907,"y2":0,"group":1,"y":0.4652671509702182},{"x":0.9565295599550716,"y2":0,"group":1,"y":0.461365651718176},{"x":0.9571714722572364,"y2":0,"group":1,"y":0.45748426806916703},{"x":0.957813384559401,"y2":0,"group":1,"y":0.4536230226628296},{"x":0.9584552968615658,"y2":0,"group":1,"y":0.4497819368463385},{"x":0.9590972091637306,"y2":0,"group":1,"y":0.4459610306773514},{"x":0.9597391214658955,"y2":0,"group":1,"y":0.44216032292706786},{"x":0.96038103376806,"y2":0,"group":1,"y":0.4383798310834045},{"x":0.9610229460702249,"y2":0,"group":1,"y":0.4346195713542852},{"x":0.9616648583723897,"y2":0,"group":1,"y":0.4308795586710458},{"x":0.9623067706745543,"y2":0,"group":1,"y":0.42715980669195586},{"x":0.9629486829767191,"y2":0,"group":1,"y":0.4234603278058562},{"x":0.9635905952788839,"y2":0,"group":1,"y":0.41978113313591137},{"x":0.9642325075810485,"y2":0,"group":1,"y":0.41612223254348035},{"x":0.9648744198832133,"y2":0,"group":1,"y":0.4124836346320999},{"x":0.9655163321853781,"y2":0,"group":1,"y":0.40886534675158875},{"x":0.9661582444875427,"y2":0,"group":1,"y":0.40526737500226595},{"x":0.9668001567897075,"y2":0,"group":1,"y":0.40168972423928273},{"x":0.9674420690918724,"y2":0,"group":1,"y":0.3981323980770758},{"x":0.9680839813940372,"y2":0,"group":1,"y":0.39459539889393347},{"x":0.9687258936962018,"y2":0,"group":1,"y":0.3910787278366775},{"x":0.9693678059983666,"y2":0,"group":1,"y":0.38758238482546276},{"x":0.9700097183005314,"y2":0,"group":1,"y":0.3841063685586899},{"x":0.970651630602696,"y2":0,"group":1,"y":0.38065067651803564},{"x":0.9712935429048608,"y2":0,"group":1,"y":0.3772153049735967},{"x":0.9719354552070256,"y2":0,"group":1,"y":0.37380024898914843},{"x":0.9725773675091902,"y2":0,"group":1,"y":0.37040550242751813},{"x":0.973219279811355,"y2":0,"group":1,"y":0.3670310579560722},{"x":0.9738611921135198,"y2":0,"group":1,"y":0.36367690705231615},{"x":0.9745031044156844,"y2":0,"group":1,"y":0.36034304000960937},{"x":0.9751450167178493,"y2":0,"group":1,"y":0.35702944594298913},{"x":0.9757869290200141,"y2":0,"group":1,"y":0.3537361127951104},{"x":0.9764288413221789,"y2":0,"group":1,"y":0.3504630273422939},{"x":0.9770707536243435,"y2":0,"group":1,"y":0.34721017520068653},{"x":0.9777126659265083,"y2":0,"group":1,"y":0.3439775408325313},{"x":0.9783545782286731,"y2":0,"group":1,"y":0.34076510755254696},{"x":0.9789964905308377,"y2":0,"group":1,"y":0.33757285753441607},{"x":0.9796384028330025,"y2":0,"group":1,"y":0.33440077181738204},{"x":0.9802803151351673,"y2":0,"group":1,"y":0.3312488303129502},{"x":0.9809222274373319,"y2":0,"group":1,"y":0.3281170118116997},{"x":0.9815641397394967,"y2":0,"group":1,"y":0.32500529399019856},{"x":0.9822060520416616,"y2":0,"group":1,"y":0.3219136534180213},{"x":0.9828479643438262,"y2":0,"group":1,"y":0.31884206556487504},{"x":0.983489876645991,"y2":0,"group":1,"y":0.3157905048078242},{"x":0.9841317889481558,"y2":0,"group":1,"y":0.31275894443861907},{"x":0.9847737012503206,"y2":0,"group":1,"y":0.30974735667112463},{"x":0.9854156135524852,"y2":0,"group":1,"y":0.30675571264884904},{"x":0.98605752585465,"y2":0,"group":1,"y":0.30378398245257043},{"x":0.9866994381568148,"y2":0,"group":1,"y":0.3008321351080627},{"x":0.9873413504589794,"y2":0,"group":1,"y":0.2979001385939163},{"x":0.9879832627611442,"y2":0,"group":1,"y":0.29498795984945414},{"x":0.988625175063309,"y2":0,"group":1,"y":0.29209556478274307},{"x":0.9892670873654736,"y2":0,"group":1,"y":0.28922291827869806},{"x":0.9899089996676385,"y2":0,"group":1,"y":0.2863699842072747},{"x":0.9905509119698033,"y2":0,"group":1,"y":0.28353672543175923},{"x":0.9911928242719679,"y2":0,"group":1,"y":0.28072310381713883},{"x":0.9918347365741327,"y2":0,"group":1,"y":0.27792908023856694},{"x":0.9924766488762975,"y2":0,"group":1,"y":0.2751546145899114},{"x":0.9931185611784623,"y2":0,"group":1,"y":0.27239966579238795},{"x":0.9937604734806269,"y2":0,"group":1,"y":0.2696641918032786},{"x":0.9944023857827917,"y2":0,"group":1,"y":0.26694814962473207},{"x":0.9950442980849565,"y2":0,"group":1,"y":0.2642514953126433},{"x":0.9956862103871211,"y2":0,"group":1,"y":0.2615741839856143},{"x":0.9963281226892859,"y2":0,"group":1,"y":0.2589161698339928},{"x":0.9969700349914508,"y2":0,"group":1,"y":0.25627740612898553},{"x":0.9976119472936154,"y2":0,"group":1,"y":0.25365784523184826},{"x":0.9982538595957802,"y2":0,"group":1,"y":0.2510574386031481},{"x":0.998895771897945,"y2":0,"group":1,"y":0.24847613681209818},{"x":0.9995376842001096,"y2":0,"group":1,"y":0.24591388954596147},{"x":1.0001795965022744,"y2":0,"group":1,"y":0.2433706456195246},{"x":1.0008215088044392,"y2":0,"group":1,"y":0.24084635298463736},{"x":1.001463421106604,"y2":0,"group":1,"y":0.23834095873981775},{"x":1.0021053334087686,"y2":0,"group":1,"y":0.23585440913992115},{"x":1.0027472457109334,"y2":0,"group":1,"y":0.23338664960587158},{"x":1.0033891580130982,"y2":0,"group":1,"y":0.23093762473445206},{"x":1.0040310703152628,"y2":0,"group":1,"y":0.2285072783081547},{"x":1.0046729826174277,"y2":0,"group":1,"y":0.22609555330508718},{"x":1.0053148949195925,"y2":0,"group":1,"y":0.22370239190893476},{"x":1.005956807221757,"y2":0,"group":1,"y":0.22132773551897464},{"x":1.0065987195239219,"y2":0,"group":1,"y":0.21897152476014364},{"x":1.0072406318260867,"y2":0,"group":1,"y":0.21663369949315367},{"x":1.0078825441282513,"y2":0,"group":1,"y":0.21431419882465763},{"x":1.008524456430416,"y2":0,"group":1,"y":0.21201296111745954},{"x":1.009166368732581,"y2":0,"group":1,"y":0.20972992400076995},{"x":1.0098082810347457,"y2":0,"group":1,"y":0.20746502438050507},{"x":1.0104501933369103,"y2":0,"group":1,"y":0.20521819844962463},{"x":1.0110921056390751,"y2":0,"group":1,"y":0.202989381698511},{"x":1.01173401794124,"y2":0,"group":1,"y":0.20077850892538437},{"x":1.0123759302434046,"y2":0,"group":1,"y":0.19858551424675291},{"x":1.0130178425455694,"y2":0,"group":1,"y":0.19641033110789774},{"x":1.0136597548477342,"y2":0,"group":1,"y":0.1942528922933892},{"x":1.0143016671498988,"y2":0,"group":1,"y":0.1921131299376317},{"x":1.0149435794520636,"y2":0,"group":1,"y":0.18999097553543925},{"x":1.0155854917542284,"y2":0,"group":1,"y":0.18788635995263353},{"x":1.016227404056393,"y2":0,"group":1,"y":0.18579921343667022},{"x":1.0168693163585578,"y2":0,"group":1,"y":0.18372946562728446},{"x":1.0175112286607226,"y2":0,"group":1,"y":0.18167704556715852},{"x":1.0181531409628874,"y2":0,"group":1,"y":0.1796418817126082},{"x":1.018795053265052,"y2":0,"group":1,"y":0.17762390194428518},{"x":1.0194369655672169,"y2":0,"group":1,"y":0.17562303357789458},{"x":1.0200788778693817,"y2":0,"group":1,"y":0.1736392033749272},{"x":1.0207207901715463,"y2":0,"group":1,"y":0.17167233755339972},{"x":1.021362702473711,"y2":0,"group":1,"y":0.1697223617986084},{"x":1.022004614775876,"y2":0,"group":1,"y":0.16778920127388575},{"x":1.0226465270780405,"y2":0,"group":1,"y":0.16587278063136643},{"x":1.0232884393802053,"y2":0,"group":1,"y":0.16397302402275699},{"x":1.0239303516823701,"y2":0,"group":1,"y":0.1620898551101032},{"x":1.0245722639845347,"y2":0,"group":1,"y":0.16022319707656363},{"x":1.0252141762866995,"y2":0,"group":1,"y":0.1583729726371765},{"x":1.0258560885888643,"y2":0,"group":1,"y":0.15653910404962673},{"x":1.0264980008910292,"y2":0,"group":1,"y":0.1547215131250036},{"x":1.0271399131931938,"y2":0,"group":1,"y":0.15292012123855583},{"x":1.0277818254953586,"y2":0,"group":1,"y":0.15113484934043445},{"x":1.0284237377975234,"y2":0,"group":1,"y":0.14936561796642472},{"x":1.029065650099688,"y2":0,"group":1,"y":0.1476123472486687},{"x":1.0297075624018528,"y2":0,"group":1,"y":0.14587495692636995},{"x":1.0303494747040176,"y2":0,"group":1,"y":0.1441533663564837},{"x":1.0309913870061822,"y2":0,"group":1,"y":0.14244749452438912},{"x":1.031633299308347,"y2":0,"group":1,"y":0.14075726005454156},{"x":1.0322752116105118,"y2":0,"group":1,"y":0.13908258122110406},{"x":1.0329171239126764,"y2":0,"group":1,"y":0.13742337595855564},{"x":1.0335590362148412,"y2":0,"group":1,"y":0.13577956187227294},{"x":1.034200948517006,"y2":0,"group":1,"y":0.1341510562490895},{"x":1.0348428608191709,"y2":0,"group":1,"y":0.1325377760678219},{"x":1.0354847731213355,"y2":0,"group":1,"y":0.13093963800977104},{"x":1.0361266854235003,"y2":0,"group":1,"y":0.1293565584691863},{"x":1.036768597725665,"y2":0,"group":1,"y":0.12778845356370283},{"x":1.0374105100278297,"y2":0,"group":1,"y":0.12623523914473836},{"x":1.0380524223299945,"y2":0,"group":1,"y":0.12469683080785782},{"x":1.0386943346321593,"y2":0,"group":1,"y":0.12317314390309785},{"x":1.039336246934324,"y2":0,"group":1,"y":0.12166409354525148},{"x":1.0399781592364887,"y2":0,"group":1,"y":0.12016959462411365},{"x":1.0406200715386535,"y2":0,"group":1,"y":0.11868956181468296},{"x":1.0412619838408181,"y2":0,"group":1,"y":0.1172239095873177},{"x":1.041903896142983,"y2":0,"group":1,"y":0.11577255221784956},{"x":1.0425458084451478,"y2":0,"group":1,"y":0.11433540379764702},{"x":1.0431877207473126,"y2":0,"group":1,"y":0.112912378243632},{"x":1.0438296330494772,"y2":0,"group":1,"y":0.11150338930824574},{"x":1.044471545351642,"y2":0,"group":1,"y":0.11010835058936341},{"x":1.0451134576538068,"y2":0,"group":1,"y":0.1087271755401549},{"x":1.0457553699559714,"y2":0,"group":1,"y":0.10735977747889316},{"x":1.0463972822581362,"y2":0,"group":1,"y":0.10600606959870539},{"x":1.047039194560301,"y2":0,"group":1,"y":0.10466596497726598},{"x":1.0476811068624656,"y2":0,"group":1,"y":0.10333937658643581},{"x":1.0483230191646304,"y2":0,"group":1,"y":0.10202621730183516},{"x":1.0489649314667953,"y2":0,"group":1,"y":0.10072639991236171},{"x":1.0496068437689599,"y2":0,"group":1,"y":0.09943983712964292},{"x":1.0502487560711247,"y2":0,"group":1,"y":0.0981664415974255},{"x":1.0508906683732895,"y2":0,"group":1,"y":0.09690612590090113},{"x":1.0515325806754543,"y2":0,"group":1,"y":0.09565880257596544},{"x":1.052174492977619,"y2":0,"group":1,"y":0.0944243841184095},{"x":1.0528164052797837,"y2":0,"group":1,"y":0.09320278299304546},{"x":1.0534583175819485,"y2":0,"group":1,"y":0.0919939116427594},{"x":1.0541002298841131,"y2":0,"group":1,"y":0.09079768249749609},{"x":1.054742142186278,"y2":0,"group":1,"y":0.08961400798317298},{"x":1.0553840544884427,"y2":0,"group":1,"y":0.08844280053051728},{"x":1.0560259667906073,"y2":0,"group":1,"y":0.08728397258383579},{"x":1.0566678790927722,"y2":0,"group":1,"y":0.08613743660970406},{"x":1.057309791394937,"y2":0,"group":1,"y":0.08500310510558426},{"x":1.0579517036971016,"y2":0,"group":1,"y":0.08388089060836423},{"x":1.0585936159992664,"y2":0,"group":1,"y":0.08277070570281894},{"x":1.0592355283014312,"y2":0,"group":1,"y":0.08167246302999595},{"x":1.059877440603596,"y2":0,"group":1,"y":0.08058607529551759},{"x":1.0605193529057606,"y2":0,"group":1,"y":0.07951145527780727},{"x":1.0611612652079254,"y2":0,"group":1,"y":0.0784485158362308},{"x":1.0618031775100902,"y2":0,"group":1,"y":0.07739716991915913},{"x":1.0624450898122548,"y2":0,"group":1,"y":0.07635733057194641},{"x":1.0630870021144196,"y2":0,"group":1,"y":0.0753289109448251},{"x":1.0637289144165845,"y2":0,"group":1,"y":0.07431182430071803},{"x":1.064370826718749,"y2":0,"group":1,"y":0.07330598402296401},{"x":1.0650127390209139,"y2":0,"group":1,"y":0.07231130362295887},{"x":1.0656546513230787,"y2":0,"group":1,"y":0.07132769674771},{"x":1.0662965636252433,"y2":0,"group":1,"y":0.0703550771873045},{"x":1.066938475927408,"y2":0,"group":1,"y":0.06939335888228906},{"x":1.067580388229573,"y2":0,"group":1,"y":0.06844245593096195},{"x":1.0682223005317377,"y2":0,"group":1,"y":0.06750228259657765},{"x":1.0688642128339023,"y2":0,"group":1,"y":0.06657275331446064},{"x":1.0695061251360671,"y2":0,"group":1,"y":0.06565378269903051},{"x":1.070148037438232,"y2":0,"group":1,"y":0.0647452855507357},{"x":1.0707899497403965,"y2":0,"group":1,"y":0.06384717686289956},{"x":1.0714318620425614,"y2":0,"group":1,"y":0.0629593718284709},{"x":1.0720737743447262,"y2":0,"group":1,"y":0.062081785846686804},{"x":1.0727156866468908,"y2":0,"group":1,"y":0.06121433452964274},{"x":1.0733575989490556,"y2":0,"group":1,"y":0.06035693370876813},{"x":1.0739995112512204,"y2":0,"group":1,"y":0.05950949944121248},{"x":1.074641423553385,"y2":0,"group":1,"y":0.05867194801613708},{"x":1.0752833358555498,"y2":0,"group":1,"y":0.057844195960912524},{"x":1.0759252481577146,"y2":0,"group":1,"y":0.0570261600472252},{"x":1.0765671604598794,"y2":0,"group":1,"y":0.05621775729708796},{"x":1.077209072762044,"y2":0,"group":1,"y":0.05541890498875717},{"x":1.0778509850642088,"y2":0,"group":1,"y":0.054629520662556955},{"x":1.0784928973663737,"y2":0,"group":1,"y":0.053849522126606986},{"x":1.0791348096685383,"y2":0,"group":1,"y":0.053078827462457984},{"x":1.079776721970703,"y2":0,"group":1,"y":0.052317355030630404},{"x":1.0804186342728679,"y2":0,"group":1,"y":0.051565023476060484},{"x":1.0810605465750325,"y2":0,"group":1,"y":0.050821751733450035},{"x":1.0817024588771973,"y2":0,"group":1,"y":0.050087459032522014},{"x":1.082344371179362,"y2":0,"group":1,"y":0.04936206490318037},{"x":1.0829862834815267,"y2":0,"group":1,"y":0.04864548918057613},{"x":1.0836281957836915,"y2":0,"group":1,"y":0.04793765201007802},{"x":1.0842701080858563,"y2":0,"group":1,"y":0.04723847385214755},{"x":1.0849120203880211,"y2":0,"group":1,"y":0.04654787548712051},{"x":1.0855539326901857,"y2":0,"group":1,"y":0.04586577801989225},{"x":1.0861958449923506,"y2":0,"group":1,"y":0.04519210288450996},{"x":1.0868377572945154,"y2":0,"group":1,"y":0.04452677184866796},{"x":1.08747966959668,"y2":0,"group":1,"y":0.04386970701811102},{"x":1.0881215818988448,"y2":0,"group":1,"y":0.043220830840941915},{"x":1.0887634942010096,"y2":0,"group":1,"y":0.04258006611183521},{"x":1.0894054065031742,"y2":0,"group":1,"y":0.04194733597615741},{"x":1.090047318805339,"y2":0,"group":1,"y":0.04132256393399167},{"x":1.0906892311075038,"y2":0,"group":1,"y":0.04070567384407278},{"x":1.0913311434096684,"y2":0,"group":1,"y":0.04009658992762408},{"x":1.0919730557118332,"y2":0,"group":1,"y":0.03949523677210473},{"x":1.092614968013998,"y2":0,"group":1,"y":0.03890153933486418},{"x":1.0932568803161629,"y2":0,"group":1,"y":0.038315422946701705},{"x":1.0938987926183275,"y2":0,"group":1,"y":0.0377368133153371},{"x":1.0945407049204923,"y2":0,"group":1,"y":0.037165636528786655},{"x":1.095182617222657,"y2":0,"group":1,"y":0.03660181905865108},{"x":1.0958245295248217,"y2":0,"group":1,"y":0.036045287763308426},{"x":1.0964664418269865,"y2":0,"group":1,"y":0.03549596989102023},{"x":1.0971083541291513,"y2":0,"group":1,"y":0.034953793082944606},{"x":1.097750266431316,"y2":0,"group":1,"y":0.034418685376061964},{"x":1.0983921787334807,"y2":0,"group":1,"y":0.0338905752060089},{"x":1.0990340910356455,"y2":0,"group":1,"y":0.03336939140982588},{"x":1.0996760033378101,"y2":0,"group":1,"y":0.0328550632286137},{"x":1.100317915639975,"y2":0,"group":1,"y":0.032347520310104946},{"x":1.1009598279421398,"y2":0,"group":1,"y":0.031846692711145874},{"x":1.1016017402443046,"y2":0,"group":1,"y":0.03135251090009257},{"x":1.1022436525464692,"y2":0,"group":1,"y":0.03086490575912021},{"x":1.102885564848634,"y2":0,"group":1,"y":0.030383808586447097},{"x":1.1035274771507988,"y2":0,"group":1,"y":0.029909151098472864},{"x":1.1041693894529634,"y2":0,"group":1,"y":0.029440865431832025},{"x":1.1048113017551282,"y2":0,"group":1,"y":0.028978884145365244},{"x":1.105453214057293,"y2":0,"group":1,"y":0.02852314022200267},{"x":1.1060951263594576,"y2":0,"group":1,"y":0.028073567070571556},{"x":1.1067370386616224,"y2":0,"group":1,"y":0.027630098527514094},{"x":1.1073789509637872,"y2":0,"group":1,"y":0.027192668858528057},{"x":1.1080208632659518,"y2":0,"group":1,"y":0.0267612127601273},{"x":1.1086627755681167,"y2":0,"group":1,"y":0.026335665361117422},{"x":1.1093046878702815,"y2":0,"group":1,"y":0.02591596222399728},{"x":1.1099466001724463,"y2":0,"group":1,"y":0.025502039346277733},{"x":1.1105885124746109,"y2":0,"group":1,"y":0.025093833161723172},{"x":1.1112304247767757,"y2":0,"group":1,"y":0.024691280541517507},{"x":1.1118723370789405,"y2":0,"group":1,"y":0.024294318795349462},{"x":1.112514249381105,"y2":0,"group":1,"y":0.02390288567242535},{"x":1.11315616168327,"y2":0,"group":1,"y":0.023516919362406075},{"x":1.1137980739854347,"y2":0,"group":1,"y":0.02313635849626594},{"x":1.1144399862875993,"y2":0,"group":1,"y":0.022761142147083885},{"x":1.1150818985897641,"y2":0,"group":1,"y":0.022391209830754044},{"x":1.115723810891929,"y2":0,"group":1,"y":0.022026501506632185},{"x":1.1163657231940936,"y2":0,"group":1,"y":0.02166695757810344},{"x":1.1170076354962584,"y2":0,"group":1,"y":0.021312518893082832},{"x":1.1176495477984232,"y2":0,"group":1,"y":0.020963126744446947},{"x":1.118291460100588,"y2":0,"group":1,"y":0.02061872287039171},{"x":1.1189333724027526,"y2":0,"group":1,"y":0.0202792494547282},{"x":1.1195752847049174,"y2":0,"group":1,"y":0.019944649127104225},{"x":1.1202171970070822,"y2":0,"group":1,"y":0.019614864963165085},{"x":1.1208591093092468,"y2":0,"group":1,"y":0.01928984048464394},{"x":1.1215010216114116,"y2":0,"group":1,"y":0.01896951965938934},{"x":1.1221429339135764,"y2":0,"group":1,"y":0.01865384690132804},{"x":1.122784846215741,"y2":0,"group":1,"y":0.018342767070364308},{"x":1.1234267585179059,"y2":0,"group":1,"y":0.018036225472215783},{"x":1.1240686708200707,"y2":0,"group":1,"y":0.01773416785818907},{"x":1.1247105831222353,"y2":0,"group":1,"y":0.017436540424893898},{"x":1.1253524954244,"y2":0,"group":1,"y":0.01714328981389568},{"x":1.125994407726565,"y2":0,"group":1,"y":0.016854363111310583},{"x":1.1266363200287297,"y2":0,"group":1,"y":0.016569707847343157},{"x":1.1272782323308943,"y2":0,"group":1,"y":0.01628927199576255},{"x":1.1279201446330591,"y2":0,"group":1,"y":0.016013003973326},{"x":1.128562056935224,"y2":0,"group":1,"y":0.01574085263914496},{"x":1.1292039692373885,"y2":0,"group":1,"y":0.015472767293996187},{"x":1.1298458815395533,"y2":0,"group":1,"y":0.015208697679578442},{"x":1.1304877938417182,"y2":0,"group":1,"y":0.014948593977717417},{"x":1.1311297061438828,"y2":0,"group":1,"y":0.014692406809516223},{"x":1.1317716184460476,"y2":0,"group":1,"y":0.014440087234457416},{"x":1.1324135307482124,"y2":0,"group":1,"y":0.014191586749450813},{"x":1.133055443050377,"y2":0,"group":1,"y":0.013946857287833903},{"x":1.1336973553525418,"y2":0,"group":1,"y":0.013705851218323878},{"x":1.1343392676547066,"y2":0,"group":1,"y":0.013468521343918904},{"x":1.1349811799568714,"y2":0,"group":1,"y":0.013234820900756394},{"x":1.135623092259036,"y2":0,"group":1,"y":0.01300470355692021},{"x":1.1362650045612008,"y2":0,"group":1,"y":0.012778123411207681},{"x":1.1369069168633656,"y2":0,"group":1,"y":0.012555034991846847},{"x":1.1375488291655302,"y2":0,"group":1,"y":0.012335393255175325},{"x":1.138190741467695,"y2":0,"group":1,"y":0.012119153584271486},{"x":1.1388326537698599,"y2":0,"group":1,"y":0.011906271787548245},{"x":1.1394745660720245,"y2":0,"group":1,"y":0.01169670409730092},{"x":1.1401164783741893,"y2":0,"group":1,"y":0.011490407168221445},{"x":1.140758390676354,"y2":0,"group":1,"y":0.011287338075865433},{"x":1.1414003029785187,"y2":0,"group":1,"y":0.011087454315087591},{"x":1.1420422152806835,"y2":0,"group":1,"y":0.010890713798435397},{"x":1.1426841275828483,"y2":0,"group":1,"y":0.010697074854507965},{"x":1.1433260398850131,"y2":0,"group":1,"y":0.01050649622627886},{"x":1.1439679521871777,"y2":0,"group":1,"y":0.010318937069382389},{"x":1.1446098644893425,"y2":0,"group":1,"y":0.010134356950366658},{"x":1.1452517767915074,"y2":0,"group":1,"y":0.009952715844915277},{"x":1.145893689093672,"y2":0,"group":1,"y":0.009773974136032038},{"x":1.1465356013958368,"y2":0,"group":1,"y":0.009598092612197351},{"x":1.1471775136980016,"y2":0,"group":1,"y":0.00942503246549442},{"x":1.1478194260001662,"y2":0,"group":1,"y":0.009254755289700545},{"x":1.148461338302331,"y2":0,"group":1,"y":0.009087223078355411},{"x":1.1491032506044958,"y2":0,"group":1,"y":0.008922398222796446},{"x":1.1497451629066604,"y2":0,"group":1,"y":0.00876024351016813},{"x":1.1503870752088252,"y2":0,"group":1,"y":0.008600722121404836},{"x":1.15102898751099,"y2":0,"group":1,"y":0.0084437976291869},{"x":1.1516708998131548,"y2":0,"group":1,"y":0.00828943399587212},{"x":1.1523128121153194,"y2":0,"group":1,"y":0.008137595571401646},{"x":1.1529547244174843,"y2":0,"group":1,"y":0.007988247091184675},{"x":1.153596636719649,"y2":0,"group":1,"y":0.007841353673956442},{"x":1.1542385490218137,"y2":0,"group":1,"y":0.007696880819618335},{"x":1.1548804613239785,"y2":0,"group":1,"y":0.007554794407054755},{"x":1.1555223736261433,"y2":0,"group":1,"y":0.007415060691925934},{"x":1.1561642859283079,"y2":0,"group":1,"y":0.007277646304448576},{"x":1.1568061982304727,"y2":0,"group":1,"y":0.007142518247151675},{"x":1.1574481105326375,"y2":0,"group":1,"y":0.007009643892614353},{"x":1.1580900228348021,"y2":0,"group":1,"y":0.006878990981187966},{"x":1.158731935136967,"y2":0,"group":1,"y":0.006750527618702318},{"x":1.1593738474391317,"y2":0,"group":1,"y":0.0066242222741502155},{"x":1.1600157597412966,"y2":0,"group":1,"y":0.0065000437773655995},{"x":1.1606576720434612,"y2":0,"group":1,"y":0.006377961316677283},{"x":1.161299584345626,"y2":0,"group":1,"y":0.006257944436555557},{"x":1.1619414966477908,"y2":0,"group":1,"y":0.006139963035243057},{"x":1.1625834089499554,"y2":0,"group":1,"y":0.006023987362373112},{"x":1.1632253212521202,"y2":0,"group":1,"y":0.005909988016575052},{"x":1.163867233554285,"y2":0,"group":1,"y":0.005797935943071847},{"x":1.1645091458564496,"y2":0,"group":1,"y":0.00568780243126224},{"x":1.1651510581586144,"y2":0,"group":1,"y":0.005579559112297627},{"x":1.1657929704607792,"y2":0,"group":1,"y":0.00547317795664648},{"x":1.1664348827629438,"y2":0,"group":1,"y":0.005368631271650687},{"x":1.1670767950651086,"y2":0,"group":1,"y":0.005265891699076819},{"x":1.1677187073672735,"y2":0,"group":1,"y":0.0051649322126553765},{"x":1.1683606196694383,"y2":0,"group":1,"y":0.005065726115617453},{"x":1.1690025319716029,"y2":0,"group":1,"y":0.004968247038222462},{"x":1.1696444442737677,"y2":0,"group":1,"y":0.004872468935282703},{"x":1.1702863565759325,"y2":0,"group":1,"y":0.004778366083680485},{"x":1.170928268878097,"y2":0,"group":1,"y":0.004685913079884807},{"x":1.171570181180262,"y2":0,"group":1,"y":0.004595084837459074},{"x":1.1722120934824267,"y2":0,"group":1,"y":0.00450585658457181},{"x":1.1728540057845913,"y2":0,"group":1,"y":0.004418203861500103},{"x":1.1734959180867561,"y2":0,"group":1,"y":0.004332102518132097},{"x":1.174137830388921,"y2":0,"group":1,"y":0.004247528711471493},{"x":1.1747797426910855,"y2":0,"group":1,"y":0.00416445890313577},{"x":1.1754216549932504,"y2":0,"group":1,"y":0.004082869856858629},{"x":1.1760635672954152,"y2":0,"group":1,"y":0.004002738635991154},{"x":1.17670547959758,"y2":0,"group":1,"y":0.003924042601002153},{"x":1.1773473918997446,"y2":0,"group":1,"y":0.0038467594069835354},{"x":1.1779893042019094,"y2":0,"group":1,"y":0.003770867001153977},{"x":1.1786312165040742,"y2":0,"group":1,"y":0.0036963436203666378},{"x":1.1792731288062388,"y2":0,"group":1,"y":0.003623167788620152},{"x":1.1799150411084036,"y2":0,"group":1,"y":0.0035513183145710614},{"x":1.1805569534105684,"y2":0,"group":1,"y":0.003480774289052857},{"x":1.181198865712733,"y2":0,"group":1,"y":0.0034115150825965723},{"x":1.1818407780148978,"y2":0,"group":1,"y":0.003343520342958206},{"x":1.1824826903170627,"y2":0,"group":1,"y":0.0032767699926514182},{"x":1.1831246026192273,"y2":0,"group":1,"y":0.0032112442264829177},{"x":1.183766514921392,"y2":0,"group":1,"y":0.003146923509099009},{"x":1.1844084272235569,"y2":0,"group":1,"y":0.0030837885725344127},{"x":1.1850503395257217,"y2":0,"group":1,"y":0.0030218204137704707},{"x":1.1856922518278863,"y2":0,"group":1,"y":0.002961000292300453},{"x":1.186334164130051,"y2":0,"group":1,"y":0.002901309727701945},{"x":1.186976076432216,"y2":0,"group":1,"y":0.0028427304972182543},{"x":1.1876179887343805,"y2":0,"group":1,"y":0.0027852446333496983},{"x":1.1882599010365453,"y2":0,"group":1,"y":0.002728834421451165},{"x":1.1889018133387101,"y2":0,"group":1,"y":0.0026734823973422472},{"x":1.1895437256408747,"y2":0,"group":1,"y":0.0026191713449243703},{"x":1.1901856379430396,"y2":0,"group":1,"y":0.0025658842938122983},{"x":1.1908275502452044,"y2":0,"group":1,"y":0.002513604516971135},{"x":1.191469462547369,"y2":0,"group":1,"y":0.0024623155283690357},{"x":1.1921113748495338,"y2":0,"group":1,"y":0.0024120010806382985},{"x":1.1927532871516986,"y2":0,"group":1,"y":0.0023626451627500045},{"x":1.1933951994538634,"y2":0,"group":1,"y":0.0023142319976999504},{"x":1.194037111756028,"y2":0,"group":1,"y":0.002266746040206713},{"x":1.1946790240581928,"y2":0,"group":1,"y":0.002220171974422841},{"x":1.1953209363603576,"y2":0,"group":1,"y":0.002174494711661007},{"x":1.1959628486625222,"y2":0,"group":1,"y":0.0021296993881290693},{"x":1.196604760964687,"y2":0,"group":1,"y":0.002085771362684563},{"x":1.1972466732668519,"y2":0,"group":1,"y":0.0020426962145976557},{"x":1.1978885855690165,"y2":0,"group":1,"y":0.0020004597413346425},{"x":1.1985304978711813,"y2":0,"group":1,"y":0.001959047956349158},{"x":1.199172410173346,"y2":0,"group":1,"y":0.0019184470868938708},{"x":1.1998143224755107,"y2":0,"group":1,"y":0.0018786435718449734},{"x":1.2004562347776755,"y2":0,"group":1,"y":0.0018396240595431324},{"x":1.2010981470798403,"y2":0,"group":1,"y":0.0018013754056492581},{"x":1.2017400593820051,"y2":0,"group":1,"y":0.0017638846710166523},{"x":1.2023819716841697,"y2":0,"group":1,"y":0.0017271391195796126},{"x":1.2030238839863345,"y2":0,"group":1,"y":0.0016911262162611906},{"x":1.2036657962884993,"y2":0,"group":1,"y":0.001655833624890718},{"x":1.204307708590664,"y2":0,"group":1,"y":0.0016212492061467576},{"x":1.2049496208928288,"y2":0,"group":1,"y":0.001587361015513128},{"x":1.2055915331949936,"y2":0,"group":1,"y":0.0015541573012504972},{"x":1.2062334454971582,"y2":0,"group":1,"y":0.0015216265023917375},{"x":1.206875357799323,"y2":0,"group":1,"y":0.0014897572467495457},{"x":1.2075172701014878,"y2":0,"group":1,"y":0.0014585383489453263},{"x":1.2081591824036524,"y2":0,"group":1,"y":0.0014279588084555317},{"x":1.2088010947058172,"y2":0,"group":1,"y":0.001398007807677848},{"x":1.209443007007982,"y2":0,"group":1,"y":0.0013686747100136454},{"x":1.2100849193101468,"y2":0,"group":1,"y":0.0013399490579728557},{"x":1.2107268316123114,"y2":0,"group":1,"y":0.001311820571295974},{"x":1.2113687439144762,"y2":0,"group":1,"y":0.0012842791450957403},{"x":1.212010656216641,"y2":0,"group":1,"y":0.0012573148480186092},{"x":1.2126525685188057,"y2":0,"group":1,"y":0.001230917920426844},{"x":1.2132944808209705,"y2":0,"group":1,"y":0.0012050787725995382},{"x":1.2139363931231353,"y2":0,"group":1,"y":0.001179787982955094},{"x":1.2145783054252999,"y2":0,"group":1,"y":0.0011550362962912397},{"x":1.2152202177274647,"y2":0,"group":1,"y":0.0011308146220506687},{"x":1.2158621300296295,"y2":0,"group":1,"y":0.0011071140326008866},{"x":1.216504042331794,"y2":0,"group":1,"y":0.001083925761541621},{"x":1.217145954633959,"y2":0,"group":1,"y":0.0010612412020274409},{"x":1.2177878669361237,"y2":0,"group":1,"y":0.001039051905114552},{"x":1.2184297792382885,"y2":0,"group":1,"y":0.0010173495781301833},{"x":1.2190716915404531,"y2":0,"group":1,"y":0.0009961260830594343},{"x":1.219713603842618,"y2":0,"group":1,"y":0.0009753734349562393},{"x":1.2203555161447828,"y2":0,"group":1,"y":0.0009550838003782314},{"x":1.2209974284469474,"y2":0,"group":1,"y":0.0009352494958365098},{"x":1.2216393407491122,"y2":0,"group":1,"y":0.0009158629862768275},{"x":1.222281253051277,"y2":0,"group":1,"y":0.0008969168835743224},{"x":1.2229231653534416,"y2":0,"group":1,"y":0.00087840394505595},{"x":1.2235650776556064,"y2":0,"group":1,"y":0.0008603170720437592},{"x":1.2242069899577712,"y2":0,"group":1,"y":0.0008426493084182918},{"x":1.2248489022599358,"y2":0,"group":1,"y":0.0008253938392100701},{"x":1.2254908145621006,"y2":0,"group":1,"y":0.0008085439892080715},{"x":1.2261327268642654,"y2":0,"group":1,"y":0.0007920932215937371},{"x":1.2267746391664303,"y2":0,"group":1,"y":0.0007760351365989349},{"x":1.2274165514685949,"y2":0,"group":1,"y":0.0007603634701850714},{"x":1.2280584637707597,"y2":0,"group":1,"y":0.0007450720927451027},{"x":1.2287003760729245,"y2":0,"group":1,"y":0.0007301550078324115},{"x":1.229342288375089,"y2":0,"group":1,"y":0.0007156063509068389},{"x":1.229984200677254,"y2":0,"group":1,"y":0.0007014203881116066},{"x":1.2306261129794187,"y2":0,"group":1,"y":0.0006875915150666434},{"x":1.2312680252815833,"y2":0,"group":1,"y":0.0006741142556914714},{"x":1.2319099375837481,"y2":0,"group":1,"y":0.0006609832610484367},{"x":1.232551849885913,"y2":0,"group":1,"y":0.0006481933082094216},{"x":1.2331937621880775,"y2":0,"group":1,"y":0.000635739299150534},{"x":1.2338356744902423,"y2":0,"group":1,"y":0.0006236162596657269},{"x":1.2344775867924072,"y2":0,"group":1,"y":0.0006118193383076453},{"x":1.235119499094572,"y2":0,"group":1,"y":0.0006003438053515664},{"x":1.2357614113967366,"y2":0,"group":1,"y":0.0005891850517856512},{"x":1.2364033236989014,"y2":0,"group":1,"y":0.0005783385883215142},{"x":1.2370452360010662,"y2":0,"group":1,"y":0.0005678000444328812},{"x":1.2376871483032308,"y2":0,"group":1,"y":0.0005575651674178672},{"x":1.2383290606053956,"y2":0,"group":1,"y":0.0005476298214840969},{"x":1.2389709729075604,"y2":0,"group":1,"y":0.0005379899868613869},{"x":1.239612885209725,"y2":0,"group":1,"y":0.0005286417589369374},{"x":1.2402547975118898,"y2":0,"group":1,"y":0.0005195813474163369},{"x":1.2408967098140546,"y2":0,"group":1,"y":0.0005108050755098514},{"x":1.2415386221162192,"y2":0,"group":1,"y":0.0005023093791413913},{"x":1.242180534418384,"y2":0,"group":1,"y":0.0004940908061859262},{"x":1.2428224467205489,"y2":0,"group":1,"y":0.00048614601573057636},{"x":1.2434643590227137,"y2":0,"group":1,"y":0.0004784717773589353},{"x":1.2441062713248783,"y2":0,"group":1,"y":0.00047106497046364715},{"x":1.244748183627043,"y2":0,"group":1,"y":0.00046392258358252114},{"x":1.245390095929208,"y2":0,"group":1,"y":0.00045704171376245717},{"x":1.2460320082313725,"y2":0,"group":1,"y":0.00045041956594307764},{"x":1.2466739205335373,"y2":0,"group":1,"y":0.0004440534523748596},{"x":1.2473158328357021,"y2":0,"group":1,"y":0.00043794079205228265},{"x":1.2479577451378667,"y2":0,"group":1,"y":0.0004320791101826149},{"x":1.2485996574400315,"y2":0,"group":1,"y":0.0004264660376747387},{"x":1.2492415697421964,"y2":0,"group":1,"y":0.0004210993106538996},{"x":1.249883482044361,"y2":0,"group":1,"y":0.00041597677000457245},{"x":1.2505253943465258,"y2":0,"group":1,"y":0.0004110963609376128},{"x":1.2511673066486906,"y2":0,"group":1,"y":0.000406456132584318},{"x":1.2518092189508554,"y2":0,"group":1,"y":0.00040205423761678627},{"x":1.25245113125302,"y2":0,"group":1,"y":0.0003978889318917156},{"x":1.2530930435551848,"y2":0,"group":1,"y":0.00039395857412442836},{"x":1.2537349558573496,"y2":0,"group":1,"y":0.00039026162558705724},{"x":1.2543768681595142,"y2":0,"group":1,"y":0.0003867966498322939},{"x":1.255018780461679,"y2":0,"group":1,"y":0.0003835623124449905},{"x":1.2556606927638438,"y2":0,"group":1,"y":0.00038055738081906043},{"x":1.2563026050660084,"y2":0,"group":1,"y":0.00037778072396214835},{"x":1.2569445173681733,"y2":0,"group":1,"y":0.0003752313123265433},{"x":1.257586429670338,"y2":0,"group":1,"y":0.0003729082176654047},{"x":1.2582283419725027,"y2":0,"group":1,"y":0.00037081061291667494},{"x":1.2588702542746675,"y2":0,"group":1,"y":0.0003689377721140946},{"x":1.2595121665768323,"y2":0,"group":1,"y":0.0003672890703256826},{"x":1.260154078878997,"y2":0,"group":1,"y":0.0003658639836165434},{"x":1.2607959911811617,"y2":0,"group":1,"y":0.0003646620890407626},{"x":1.2614379034833265,"y2":0,"group":1,"y":0.0003636830646601419},{"x":1.2620798157854913,"y2":0,"group":1,"y":0.00036292668958937124},{"x":1.262721728087656,"y2":0,"group":1,"y":0.0003623928440690677},{"x":1.2633636403898207,"y2":0,"group":1,"y":0.0003620815095643215},{"x":1.2640055526919856,"y2":0,"group":1,"y":0.00036199276889395393},{"x":1.2646474649941502,"y2":0,"group":1,"y":0.00036212680638281203},{"x":1.265289377296315,"y2":0,"group":1,"y":0.0003624839080441089},{"x":1.2659312895984798,"y2":0,"group":1,"y":0.00036306446179058816},{"x":1.2665732019006444,"y2":0,"group":1,"y":0.00036386895766944705},{"x":1.2672151142028092,"y2":0,"group":1,"y":0.0003648979881264308},{"x":1.267857026504974,"y2":0,"group":1,"y":0.00036615224829993076},{"x":1.2684989388071388,"y2":0,"group":1,"y":0.0003676325363392158},{"x":1.2691408511093034,"y2":0,"group":1,"y":0.00036933975375282035},{"x":1.2697827634114682,"y2":0,"group":1,"y":0.0003712749057833975},{"x":1.270424675713633,"y2":0,"group":1,"y":0.00037343910181231177},{"x":1.2710665880157976,"y2":0,"group":1,"y":0.0003758335557895176}]}],"scales":[{"name":"x","zero":false,"range":"width","domain":{"data":"table","field":"x"},"type":"linear"},{"name":"y","range":"height","domain":{"data":"table","field":"y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450}

	);
</script>
\endhtmlonly

### Stacked Area


\htmlonly
<div id="stackedarea"></div>
<script type="text/javascript">
plot("stackedarea",

{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"marks":[{"properties":{"enter":{"interpolate":{"value":"monotone"},"x":{"field":"x","scale":"x"},"y2":{"field":"layout_end","scale":"y"},"fill":{"field":"group","scale":"group"},"y":{"field":"layout_start","scale":"y"}}},"type":"area"}],"from":{"data":"table","transform":[{"sortby":["group"],"offset":"zero","field":"y","groupby":["x"],"type":"stack"},{"groupby":["group"],"type":"facet"}]},"type":"group"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":0,"y2":0,"group":0,"y":28},{"x":1,"y2":0,"group":0,"y":43},{"x":2,"y2":0,"group":0,"y":81},{"x":3,"y2":0,"group":0,"y":19},{"x":4,"y2":0,"group":0,"y":52},{"x":5,"y2":0,"group":0,"y":24},{"x":6,"y2":0,"group":0,"y":87},{"x":7,"y2":0,"group":0,"y":17},{"x":8,"y2":0,"group":0,"y":68},{"x":9,"y2":0,"group":0,"y":49},{"x":0,"y2":0,"group":1,"y":55},{"x":1,"y2":0,"group":1,"y":91},{"x":2,"y2":0,"group":1,"y":53},{"x":3,"y2":0,"group":1,"y":87},{"x":4,"y2":0,"group":1,"y":48},{"x":5,"y2":0,"group":1,"y":49},{"x":6,"y2":0,"group":1,"y":66},{"x":7,"y2":0,"group":1,"y":27},{"x":8,"y2":0,"group":1,"y":16},{"x":9,"y2":0,"group":1,"y":15}]},{"name":"stats","source":"table","transform":[{"groupby":["x"],"type":"aggregate","summarize":[{"ops":["sum"],"field":"y"}]}]}],"scales":[{"name":"x","points":true,"range":"width","domain":{"data":"table","field":"x"}},{"name":"y","range":"height","domain":{"data":"stats","field":"sum_y"},"type":"linear"},{"name":"group","range":["rgb(166,206,227)","rgb( 31,120,180)","rgb(178,223,138)","rgb( 51,160, 44)","rgb(251,154,153)","rgb(227, 26, 28)","rgb(253,191,111)","rgb(255,127,  0)","rgb(202,178,214)","rgb(106, 61,154)","rgb(255,255,153)","rgb(177, 89, 40)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

	);
</script>
\endhtmlonly

### Normalized Stacked Area (with `colorscheme!`, `hover!` and `hline`)


\htmlonly
<div id="hlinearea"></div>
<script type="text/javascript">
plot("hlinearea",


{"name":"Vega Visualization","height":450,"padding":"auto","marks":[{"marks":[{"properties":{"update":{"fillOpacity":{"value":1},"strokeOpacity":{"value":1}},"hover":{"fillOpacity":{"value":0.75},"strokeOpacity":{"value":0.75}},"enter":{"interpolate":{"value":"monotone"},"x":{"field":"x","scale":"x"},"y2":{"field":"layout_end","scale":"y"},"fill":{"field":"group","scale":"group"},"y":{"field":"layout_start","scale":"y"}}},"type":"area"}],"from":{"data":"table","transform":[{"sortby":["group"],"offset":"normalize","field":"y","groupby":["x"],"type":"stack"},{"groupby":["group"],"type":"facet"}]},"type":"group"},{"properties":{"enter":{"strokeDash":{"value":[5]},"stroke":{"value":"#000"},"x2":{"field":{"group":"width"}},"x":{"field":{"group":"x"}},"strokeWidth":{"value":1.5},"y":{"value":0.5,"scale":"y"}}},"type":"rule"}],"axes":[{"properties":{"title":{"fontSize":{"value":14}}},"title":"x","type":"x","scale":"x"},{"properties":{"title":{"fontSize":{"value":14}}},"title":"y","type":"y","scale":"y"}],"data":[{"name":"table","values":[{"x":0,"y2":0,"group":0,"y":28},{"x":1,"y2":0,"group":0,"y":43},{"x":2,"y2":0,"group":0,"y":81},{"x":3,"y2":0,"group":0,"y":19},{"x":4,"y2":0,"group":0,"y":52},{"x":5,"y2":0,"group":0,"y":24},{"x":6,"y2":0,"group":0,"y":87},{"x":7,"y2":0,"group":0,"y":17},{"x":8,"y2":0,"group":0,"y":68},{"x":9,"y2":0,"group":0,"y":49},{"x":0,"y2":0,"group":1,"y":55},{"x":1,"y2":0,"group":1,"y":91},{"x":2,"y2":0,"group":1,"y":53},{"x":3,"y2":0,"group":1,"y":87},{"x":4,"y2":0,"group":1,"y":48},{"x":5,"y2":0,"group":1,"y":49},{"x":6,"y2":0,"group":1,"y":66},{"x":7,"y2":0,"group":1,"y":27},{"x":8,"y2":0,"group":1,"y":16},{"x":9,"y2":0,"group":1,"y":15}]},{"name":"stats","source":"table","transform":[{"groupby":["x"],"type":"aggregate","summarize":[{"ops":["sum"],"field":"y"}]}]}],"scales":[{"name":"x","points":true,"range":"width","domain":{"data":"table","field":"x"}},{"reverse":false,"name":"y","zero":true,"domainMax":1,"domain":{"data":"stats","field":"sum_y"},"range":"height","type":"linear","round":false},{"name":"group","range":["rgb(254,229,217)","rgb(252,174,145)","rgb(251,106, 74)","rgb(203, 24, 29)"],"domain":{"data":"table","field":"group"},"type":"ordinal"}],"width":450,"legends":[{"title":"Group","fill":"group"}]}

	);
</script>
\endhtmlonly

